@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: check_netlist.cpp,v 1.74 2005/02/08 23:08:16 raimi Exp $
+ * $Id: check_netlist.cpp,v 1.82 2005/07/04 08:33:58 raimi Exp $
  *
  */
 
@@ -51,7 +51,7 @@ struct pair_t * pair_root = NULL;
 // List of available microstrip components.
 static char * strip_available[] = {
   "MLIN", "MCORN", "MMBEND", "MSTEP", "MOPEN", "MGAP", "MCOUPLED", "MTEE",
-  "MCROSS", "MVIA", "CLIN", NULL };
+  "MCROSS", "MVIA", "CLIN", "COPEN", "CSHORT", "CGAP", "CSTEP", NULL };
 
 /* The function counts the nodes in a definition line. */
 static int checker_count_nodes (struct definition_t * def) {
@@ -199,14 +199,22 @@ static struct special_t checker_specials[] = {
   { "DC",     "convHelper",  { "none", "SourceStepping", "gMinStepping", 
 			       "LineSearch", "Attenuation", "SteepestDescent",
 			       NULL } },
+  { "DC",     "Solver",      { "CroutLU", "DoolittleLU", "HouseholderQR",
+			       "HouseholderLQ", NULL } },
   { "TR",     "Type",        { "lin", "log", NULL } },
   { "TR",     "IntegrationMethod",
     { "Euler", "Trapezoidal", "Gear", "AdamsMoulton", NULL } },
+  { "TR",     "Solver",      { "CroutLU", "DoolittleLU", "HouseholderQR",
+			       "HouseholderLQ", NULL } },
   { "MLIN",   "DispModel",   { "Kirschning", "Kobayashi", "Yamashita",
 			       "Getsinger", "Schneider", "Pramanick",
 			       "Hammerstad", NULL } },
   { "MLIN",   "Model",       { "Wheeler", "Schneider", "Hammerstad", NULL } },
   { "CLIN",   "Backside",    { "Metal", "Air", NULL } },
+  { "CLIN",   "Approx",      { "yes", "no", NULL } },
+  { "COPEN",  "Backside",    { "Metal", "Air", NULL } },
+  { "CSHORT", "Backside",    { "Metal", "Air", NULL } },
+  { "CSTEP",  "Backside",    { "Metal", "Air", NULL } },
   { "SW",     "Type",        { "lin", "log", "list", "const", NULL } },
   { "SPfile", "Data",        { "rectangular", "polar", NULL } },
   { "MSTEP",  "MSDispModel", { "Kirschning", "Kobayashi", "Yamashita",
@@ -661,7 +669,7 @@ static int checker_validate_lists (struct definition_t * root) {
 static int checker_validate_actions (struct definition_t * root) {
   int a, c, n, errors = 0;
   if ((n = checker_count_definitions (root, NULL, 1)) < 1) {
-    logprint (LOG_ERROR, "checker error, no actions .XX defined\n");
+    logprint (LOG_ERROR, "checker error, no actions defined: nothing to do\n");
     errors++;
   }
   else {
@@ -970,6 +978,7 @@ checker_copy_subcircuit (struct definition_t * sub) {
   copy->nodeset = sub->nodeset;
   copy->define = sub->define;
   copy->pairs = sub->pairs;
+  copy->ncount = sub->ncount;
   copy->type = strdup (sub->type);
   copy->copy = 1;
   return copy;
