@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: component.h,v 1.24 2005/01/28 19:24:39 raimi Exp $
+ * $Id: component.h,v 1.28 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -76,6 +76,7 @@
 #include "devices/mosfet.h"
 
 #include "microstrip/substrate.h"
+
 #include "microstrip/msline.h"
 #include "microstrip/mscorner.h"
 #include "microstrip/msmbend.h"
@@ -86,6 +87,11 @@
 #include "microstrip/mstee.h"
 #include "microstrip/mscross.h"
 #include "microstrip/msvia.h"
+
 #include "microstrip/cpwline.h"
+#include "microstrip/cpwopen.h"
+#include "microstrip/cpwshort.h"
+#include "microstrip/cpwgap.h"
+#include "microstrip/cpwstep.h"
 
 #endif /* __COMPONENT_H__ */
