@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: component_id.h,v 1.22 2005/01/28 19:24:40 raimi Exp $
+ * $Id: component_id.h,v 1.26 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -83,6 +83,10 @@ enum circuit_type {
 
   // coplanar components
   CIR_CPWLINE,
+  CIR_CPWOPEN,
+  CIR_CPWSHORT,
+  CIR_CPWGAP,
+  CIR_CPWSTEP,
 
   // non-linear components
   CIR_DIODE,
