@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: bjt.cpp,v 1.26 2005/02/03 20:40:19 raimi Exp $
+ * $Id: bjt.cpp,v 1.31 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -42,12 +42,12 @@
 #include "device.h"
 #include "bjt.h"
 
-#define NEWSGP 1
+#define NEWSGP 0
 
-#define NODE_B 1 /* base node       */
-#define NODE_C 2 /* collector node  */
-#define NODE_E 3 /* emitter node    */
-#define NODE_S 4 /* substrate node  */
+#define NODE_B 0 /* base node       */
+#define NODE_C 1 /* collector node  */
+#define NODE_E 2 /* emitter node    */
+#define NODE_S 3 /* substrate node  */
 
 bjt::bjt () : circuit (4) {
   cbcx = rb = re = rc = NULL;
@@ -66,8 +66,13 @@ matrix bjt::calcMatrixY (nr_double_t frequency) {
   nr_double_t Cbci = getOperatingPoint ("Cbci");
   nr_double_t gbc  = getOperatingPoint ("gmu");
   nr_double_t Ccs  = getOperatingPoint ("Ccs");
+#if NEWSGP && 0
   nr_double_t gmfr = getOperatingPoint ("gmf");
   nr_double_t gmr  = getOperatingPoint ("gmr");
+#else
+  nr_double_t gmfr = getOperatingPoint ("gm");
+  nr_double_t go   = getOperatingPoint ("go");
+#endif
   nr_double_t Ptf  = getPropertyDouble ("Ptf");
   nr_double_t Tf   = getPropertyDouble ("Tf");
 
@@ -82,6 +87,8 @@ matrix bjt::calcMatrixY (nr_double_t frequency) {
 
   // build admittance matrix and convert it to S-parameter matrix
   matrix y (4);
+#if NEWSGP && 0
+  // for some reason this small signal equivalent can't be used
   y.set (NODE_B, NODE_B, Ybc + Ybe);
   y.set (NODE_B, NODE_C, -Ybc);
   y.set (NODE_B, NODE_E, -Ybe);
@@ -98,6 +105,24 @@ matrix bjt::calcMatrixY (nr_double_t frequency) {
   y.set (NODE_S, NODE_C, -Ycs);
   y.set (NODE_S, NODE_E, 0);
   y.set (NODE_S, NODE_S, Ycs);
+#else /* !NEWSGP */
+  y.set (NODE_B, NODE_B, Ybc + Ybe);
+  y.set (NODE_B, NODE_C, -Ybc);
+  y.set (NODE_B, NODE_E, -Ybe);
+  y.set (NODE_B, NODE_S, 0);
+  y.set (NODE_C, NODE_B, -Ybc + gmf);
+  y.set (NODE_C, NODE_C, Ybc + Ycs + go);
+  y.set (NODE_C, NODE_E, -gmf - go);
+  y.set (NODE_C, NODE_S, -Ycs);
+  y.set (NODE_E, NODE_B, -Ybe - gmf);
+  y.set (NODE_E, NODE_C, -go);
+  y.set (NODE_E, NODE_E, Ybe + gmf + go);
+  y.set (NODE_E, NODE_S, 0);
+  y.set (NODE_S, NODE_B, 0);
+  y.set (NODE_S, NODE_C, -Ycs);
+  y.set (NODE_S, NODE_E, 0);
+  y.set (NODE_S, NODE_S, Ycs);
+#endif /* !NEWSGP */
   return y;
 }
 
@@ -255,6 +280,7 @@ void bjt::calcDC (void) {
 
   // base-emitter diodes
   gtiny = Ube < - 10 * Ut * Nf ? (Is + Ise) : 0;
+#if 0
   If = pnCurrent (Ube, Is, Ut * Nf);
   Ibei = If / Bf;
   gif = pnConductance (Ube, Is, Ut * Nf);
@@ -263,9 +289,20 @@ void bjt::calcDC (void) {
   gben = pnConductance (Ube, Ise, Ut * Ne);
   Ibe = Ibei + Iben + gtiny * Ube;
   gbe = gbei + gben + gtiny;
+#else
+  pnJunctionBIP (Ube, Is, Ut * Nf, If, gif);
+  Ibei = If / Bf;
+  gbei = gif / Bf;
+  pnJunctionBIP (Ube, Ise, Ut * Ne, Iben, gben);
+  Iben += gtiny * Ube;
+  gben += gtiny;
+  Ibe = Ibei + Iben;
+  gbe = gbei + gben;
+#endif
 
   // base-collector diodes
   gtiny = Ubc < - 10 * Ut * Nr ? (Is + Isc) : 0;
+#if 0
   Ir = pnCurrent (Ubc, Is, Ut * Nr);
   Ibci = Ir / Br;
   gir = pnConductance (Ubc, Is, Ut * Nr);
@@ -274,6 +311,16 @@ void bjt::calcDC (void) {
   gbcn = pnConductance (Ubc, Isc, Ut * Nc);
   Ibc = Ibci + Ibcn + gtiny * Ubc;
   gbc = gbci + gbcn + gtiny;
+#else
+  pnJunctionBIP (Ubc, Is, Ut * Nr, Ir, gir);
+  Ibci = Ir / Br;
+  gbci = gir / Br;
+  pnJunctionBIP (Ubc, Isc, Ut * Nc, Ibcn, gbcn);
+  Ibcn += gtiny * Ubc;
+  gbcn += gtiny;
+  Ibc = Ibci + Ibcn;
+  gbc = gbci + gbcn;
+#endif
 
   // compute base charge quantities
   Q1 = 1 / (1 - Ubc * Vaf - Ube * Var);
@@ -315,12 +362,12 @@ void bjt::calcDC (void) {
   }
 
   // compute autonomic current sources
-  IeqB = Ibe - gbe * Ube;
-  IeqC = Ibc - gbc * Ubc;
+  IeqB = Ibe - Ube * gbe;
+  IeqC = Ibc - Ubc * gbc;
 #if NEWSGP
   IeqE = It - gitf * Ube + gitr * Ubc;
 #else
-  IeqE = It - gm * Ube - go * Uce;
+  IeqE = It - Ube * gm - Uce * go;
 #endif
   IeqS = 0;
   setI (NODE_B, (-IeqB - IeqC) * pol);
@@ -366,6 +413,17 @@ void bjt::calcDC (void) {
 #endif
 }
 
+void bjt::saveOperatingPoints (void) {
+  nr_double_t Ube, Ubc, Ucs;
+  Ube = real (getV (NODE_B) - getV (NODE_E)) * pol;
+  Ubc = real (getV (NODE_B) - getV (NODE_C)) * pol;
+  Ucs = real (getV (NODE_S) - getV (NODE_C)) * pol;
+  setOperatingPoint ("Vbe", Ube);
+  setOperatingPoint ("Vbc", Ubc);
+  setOperatingPoint ("Vce", Ube - Ubc);
+  setOperatingPoint ("Vcs", Ucs);
+}
+
 void bjt::calcOperatingPoints (void) {
 
   // fetch device model parameters
@@ -391,9 +449,9 @@ void bjt::calcOperatingPoints (void) {
   // interpret zero as infinity for that model parameter
   Vtf = Vtf > 0 ? 1.0 / Vtf : 0;
 
-  Ube = real (getV (NODE_B) - getV (NODE_E)) * pol;
-  Ubc = real (getV (NODE_B) - getV (NODE_C)) * pol;
-  Ucs = real (getV (NODE_S) - getV (NODE_C)) * pol;
+  Ube = getOperatingPoint ("Vbe");
+  Ubc = getOperatingPoint ("Vbc");
+  Ucs = getOperatingPoint ("Vcs");
 
   // depletion capacitance of base-emitter diode
   Cbe = pnCapacitance (Ube, Cje0, Vje, Mje, Fc);
@@ -431,10 +489,6 @@ void bjt::calcOperatingPoints (void) {
   setOperatingPoint ("gmr", gitr);
   setOperatingPoint ("gmu", gbci + gbcn);
   setOperatingPoint ("gpi", gbei + gben);
-  setOperatingPoint ("Vbe", Ube);
-  setOperatingPoint ("Vbc", Ubc);
-  setOperatingPoint ("Vce", Ube - Ubc);
-  setOperatingPoint ("Vcs", Ucs);
   setOperatingPoint ("Rbb", Rbb);
   setOperatingPoint ("Ibe", Ibe);
   setOperatingPoint ("Ice", It);
@@ -455,7 +509,7 @@ void bjt::processCbcx (void) {
      collector node and external base node */
   if (Rbm != 0.0 && Cjc0 != 0.0 && Xcjc != 1.0) {
     if (!deviceEnabled (cbcx)) {
-      cbcx = splitCapacitance (this, cbcx, getNet (), "Cbcx", rb->getNode (1),
+      cbcx = splitCapacitance (this, cbcx, getNet (), "Cbcx", rb->getNode (0),
 			       getNode (NODE_C));
     }
     cbcx->setProperty ("C", getOperatingPoint ("Cbcx"));
@@ -504,6 +558,7 @@ void bjt::initTR (void) {
 
 void bjt::calcTR (nr_double_t t) {
   calcDC ();
+  saveOperatingPoints ();
   calcOperatingPoints ();
 
   nr_double_t Ube = getOperatingPoint ("Vbe");
