@@ -1,7 +1,7 @@
 /*
  * device.cpp - device class implementation
  *
- * Copyright (C) 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: device.cpp,v 1.13 2004/12/15 19:55:31 raimi Exp $
+ * $Id: device.cpp,v 1.18 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -57,13 +57,13 @@ circuit * splitResistance (circuit * base, circuit * res, net * subnet,
     char * name = circuit::createInternal (c, base->getName ());
     char * node = circuit::createInternal (n, base->getName ());
     res->setName (name);
-    res->setNode (1, base->getNode(internal)->getName ());
-    res->setNode (2, node, 1);
+    res->setNode (0, base->getNode(internal)->getName ());
+    res->setNode (1, node, 1);
     subnet->insertCircuit (res);
     free (name);
     free (node);
   }
-  base->setNode (internal, res->getNode(2)->getName (), 1);
+  base->setNode (internal, res->getNode(1)->getName (), 1);
   return res;
 }
 
@@ -88,8 +88,8 @@ circuit * splitCapacitance (circuit * base, circuit * cap, net * subnet,
     cap = new capacitor ();
     char * name = circuit::createInternal (c, base->getName ());
     cap->setName (name);
-    cap->setNode (1, n1->getName ());
-    cap->setNode (2, n2->getName ());
+    cap->setNode (0, n1->getName ());
+    cap->setNode (1, n2->getName ());
     free (name);
   }
   subnet->insertCircuit (cap);
@@ -137,14 +137,44 @@ nr_double_t pnVoltage (nr_double_t Ud, nr_double_t Uold,
   return Ud;
 }
 
+// Computes current and its derivative for a MOS pn-junction.
+void pnJunctionMOS (nr_double_t Upn, nr_double_t Iss, nr_double_t Ute,
+		    nr_double_t& I, nr_double_t& g) {
+  if (Upn <= 0) {
+    g = Iss / Ute;
+    I = g * Upn;
+  }
+  else {
+    nr_double_t e = exp (MIN (Upn / Ute, 709));
+    I = Iss * (e - 1);
+    g = Iss * e / Ute;
+  }
+}
+
+// Computes current and its derivative for a bipolar pn-junction.
+void pnJunctionBIP (nr_double_t Upn, nr_double_t Iss, nr_double_t Ute,
+		    nr_double_t& I, nr_double_t& g) {
+  if (Upn < -3 * Ute) {
+    nr_double_t a = 3 * Ute / (Upn * M_E);
+    a = cubic (a);
+    I = -Iss * (1 + a);
+    g = +Iss * 3 * a / Upn;
+  }
+  else { 
+    nr_double_t e = exp (MIN (Upn / Ute, 709));
+    I = Iss * (e - 1);
+    g = Iss * e / Ute;
+  }
+}
+
 // The function computes the exponential pn-junction current.
 nr_double_t pnCurrent (nr_double_t Upn, nr_double_t Iss, nr_double_t Ute) {
-  return Iss * (exp (Upn / Ute) - 1);
+  return Iss * (exp (MIN (Upn / Ute, 709)) - 1);
 }
 
 // The function computes the exponential pn-junction current's derivative.
 nr_double_t pnConductance (nr_double_t Upn, nr_double_t Iss, nr_double_t Ute) {
-  return Iss * exp (Upn / Ute) / Ute;
+  return Iss * exp (MIN (Upn / Ute, 709)) / Ute;
 }
 
 // Computes pn-junction depletion capacitance.
@@ -169,6 +199,7 @@ nr_double_t pnCharge (nr_double_t Uj, nr_double_t Cj, nr_double_t Vj,
     q = Cj * Vj / (1 - Mj) * (1 - b);
   }
   else {
+#if 0
     a = 1 - Fc;
     b = exp ((1 - Mj) * log (a));
     a = exp ((1 + Mj) * log (a));
@@ -176,6 +207,15 @@ nr_double_t pnCharge (nr_double_t Uj, nr_double_t Cj, nr_double_t Vj,
     nr_double_t d = Fc * Vj;
     nr_double_t e = Vj * (1 - b) / (1 - Mj);
     q = Cj * (e + (c * (Uj - d) + Mj / 2 / Vj * (sqr (Uj) - sqr (d))) / a);
+#else /* this variant is numerically more stable */
+    a = 1 - Fc;
+    b = exp (-Mj * log (a));
+    nr_double_t f = Fc * Vj;
+    nr_double_t c = Cj * (1 - Fc * (1 + Mj)) * b / a;
+    nr_double_t d = Cj * Mj * b / a / Vj;
+    nr_double_t e = Cj * Vj * (1 - a * b) / (1 - Mj) - d / 2 * f * f - f * c;
+    q = e + Uj * (c + Uj * d / 2);
+#endif
   }
   return q;
 }
@@ -214,7 +254,7 @@ nr_double_t pnCriticalVoltage (nr_double_t Iss, nr_double_t Ute) {
    convergence. */
 nr_double_t fetVoltage (nr_double_t Ufet, nr_double_t Uold, nr_double_t Uth) {
   nr_double_t Utsthi = fabs (2 * (Uold - Uth)) + 2.0;
-  nr_double_t Utstlo = Utsthi / 2 + 2.0;
+  nr_double_t Utstlo = Utsthi / 2;
   nr_double_t Utox   = Uth + 3.5;
   nr_double_t DeltaU = Ufet - Uold;
 
