@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: device.h,v 1.8 2004/10/17 09:44:30 ela Exp $
+ * $Id: device.h,v 1.11 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -39,6 +39,10 @@ class device
 				     char *, node *, node *);
   friend void disableCapacitance (circuit *, circuit *, net *);
   friend int deviceEnabled (circuit *);
+  friend void pnJunctionMOS (nr_double_t, nr_double_t, nr_double_t,
+			     nr_double_t&, nr_double_t&);
+  friend void pnJunctionBIP (nr_double_t, nr_double_t, nr_double_t,
+			     nr_double_t&, nr_double_t&);
   friend nr_double_t pnVoltage (nr_double_t, nr_double_t, nr_double_t,
 				nr_double_t);
   friend nr_double_t pnCurrent (nr_double_t, nr_double_t, nr_double_t);
