@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: jfet.cpp,v 1.23 2005/02/03 20:40:19 raimi Exp $
+ * $Id: jfet.cpp,v 1.26 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -42,9 +42,9 @@
 #include "device.h"
 #include "jfet.h"
 
-#define NODE_G 1 /* gate node   */
-#define NODE_D 2 /* drain node  */
-#define NODE_S 3 /* source node */
+#define NODE_G 0 /* gate node   */
+#define NODE_D 1 /* drain node  */
+#define NODE_S 2 /* source node */
 
 jfet::jfet () : circuit (3) {
   rs = rd = NULL;
@@ -270,6 +270,15 @@ void jfet::calcDC (void) {
   setY (NODE_S, NODE_S, ggs + gds + gm);
 }
 
+void jfet::saveOperatingPoints (void) {
+  nr_double_t Ugs, Ugd;
+  Ugd = real (getV (NODE_G) - getV (NODE_D)) * pol;
+  Ugs = real (getV (NODE_G) - getV (NODE_S)) * pol;
+  setOperatingPoint ("Vgs", Ugs);
+  setOperatingPoint ("Vgd", Ugd);
+  setOperatingPoint ("Vds", Ugs - Ugd);
+}
+
 void jfet::calcOperatingPoints (void) {
 
   // fetch device model parameters
@@ -284,8 +293,8 @@ void jfet::calcOperatingPoints (void) {
 
   T = kelvin (T);
   Ut = kB * T / Q;
-  Ugd = real (getV (NODE_G) - getV (NODE_D)) * pol;
-  Ugs = real (getV (NODE_G) - getV (NODE_S)) * pol;
+  Ugd = getOperatingPoint ("Vgd");
+  Ugs = getOperatingPoint ("Vgs");
 
   // capacitance of gate-drain diode
   Cgd = pnCapacitance (Ugd, Cgd0, Pb, z, Fc);
@@ -301,9 +310,6 @@ void jfet::calcOperatingPoints (void) {
   setOperatingPoint ("gds", gds);
   setOperatingPoint ("gm", gm);
   setOperatingPoint ("Id", Ids);
-  setOperatingPoint ("Vgs", Ugs);
-  setOperatingPoint ("Vgd", Ugd);
-  setOperatingPoint ("Vds", Ugs - Ugd);
   setOperatingPoint ("Cgd", Cgd);
   setOperatingPoint ("Cgs", Cgs);
 }
@@ -333,6 +339,7 @@ void jfet::initTR (void) {
 
 void jfet::calcTR (nr_double_t) {
   calcDC ();
+  saveOperatingPoints ();
   calcOperatingPoints ();
 
   nr_double_t Ugs = getOperatingPoint ("Vgs");
