@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: mosfet.cpp,v 1.24 2005/02/03 20:40:19 raimi Exp $
+ * $Id: mosfet.cpp,v 1.29 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -43,10 +43,10 @@
 #include "device.h"
 #include "mosfet.h"
 
-#define NODE_G 1 /* gate node   */
-#define NODE_D 2 /* drain node  */
-#define NODE_S 3 /* source node */
-#define NODE_B 4 /* bulk node   */
+#define NODE_G 0 /* gate node   */
+#define NODE_D 1 /* drain node  */
+#define NODE_S 2 /* source node */
+#define NODE_B 3 /* bulk node   */
 
 // silicon bandgap as function of T
 #define Egap(T) (1.16 - (7.02e-4 * sqr (T)) / ((T) + 1108))
@@ -382,13 +382,13 @@ void mosfet::calcDC (void) {
   if (Uds >= 0) {
     Ugs = fetVoltage (Ugs, UgsPrev, Vto * pol);
     Uds = Ugs - Ugd;
-    Uds = fetVoltage (Uds, UdsPrev, Vto * pol);
+    Uds = fetVoltageDS (Uds, UdsPrev);
     Ugd = Ugs - Uds;
   }
   else {
     Ugd = fetVoltage (Ugd, UgdPrev, Vto * pol);
     Uds = Ugs - Ugd;
-    Uds = -fetVoltage (-Uds, -UdsPrev, Vto * pol);
+    Uds = -fetVoltageDS (-Uds, -UdsPrev);
     Ugs = Ugd + Uds;
   }
   if (Uds >= 0) {
@@ -402,14 +402,16 @@ void mosfet::calcDC (void) {
   UgsPrev = Ugs; UgdPrev = Ugd; UbdPrev = Ubd; UdsPrev = Uds; UbsPrev = Ubs;
 
   // parasitic bulk-source diode
-  gtiny = Ubs < - 10 * Ut * n ? Iss : 0;
-  gbs = pnConductance (Ubs, Iss, Ut * n) + gtiny;
-  Ibs = pnCurrent (Ubs, Iss, Ut * n) + gtiny * Ubs;
+  gtiny = Iss;
+  pnJunctionMOS (Ubs, Iss, Ut * n, Ibs, gbs);
+  Ibs += gtiny * Ubs;
+  gbs += gtiny;
 
   // parasitic bulk-drain diode
-  gtiny = Ubd < - 10 * Ut * n ? Isd : 0;
-  gbd = pnConductance (Ubd, Isd, Ut * n) + gtiny;
-  Ibd = pnCurrent (Ubd, Isd, Ut * n) + gtiny * Ubd;
+  gtiny = Isd;
+  pnJunctionMOS (Ubd, Isd, Ut * n, Ibd, gbd);
+  Ibd += gtiny * Ubd;
+  gbd += gtiny;
 
   // differentiate inverse and forward mode
   MOSdir = (Uds >= 0) ? +1 : -1;
@@ -517,6 +519,19 @@ void mosfet::calcDC (void) {
 #define vgbState 14 // gate-bulk voltage state
 #define cgbState 15 // gate-bulk capacitance state
 
+void mosfet::saveOperatingPoints (void) {
+  nr_double_t Ugs, Ugd, Ubs, Ubd;
+  Ugd = real (getV (NODE_G) - getV (NODE_D)) * pol;
+  Ugs = real (getV (NODE_G) - getV (NODE_S)) * pol;
+  Ubs = real (getV (NODE_B) - getV (NODE_S)) * pol;
+  Ubd = real (getV (NODE_B) - getV (NODE_D)) * pol;
+  setOperatingPoint ("Vgs", Ugs);
+  setOperatingPoint ("Vgd", Ugd);
+  setOperatingPoint ("Vbs", Ubs);
+  setOperatingPoint ("Vbd", Ubd);
+  setOperatingPoint ("Vds", Ugs - Ugd);
+}
+
 void mosfet::calcOperatingPoints (void) {
 
   // fetch device model parameters
@@ -537,10 +552,10 @@ void mosfet::calcOperatingPoints (void) {
   nr_double_t Ubs, Ubd, Cbs, Cbd, Ugs, Ugd, Uds, Ugb;
   nr_double_t Cgd, Cgb, Cgs;
 
-  Ugd = real (getV (NODE_G) - getV (NODE_D)) * pol;
-  Ugs = real (getV (NODE_G) - getV (NODE_S)) * pol;
-  Ubs = real (getV (NODE_B) - getV (NODE_S)) * pol;
-  Ubd = real (getV (NODE_B) - getV (NODE_D)) * pol;
+  Ugd = getOperatingPoint ("Vgd");
+  Ugs = getOperatingPoint ("Vgs");
+  Ubs = getOperatingPoint ("Vbs");
+  Ubd = getOperatingPoint ("Vbd");
   Uds = Ugs - Ugd;
   Ugb = Ugs - Ubs;
 
@@ -595,11 +610,6 @@ void mosfet::calcOperatingPoints (void) {
   setOperatingPoint ("Vdsat", Udsat);
   setOperatingPoint ("gbs", gbs);
   setOperatingPoint ("gbd", gbd);
-  setOperatingPoint ("Vgs", Ugs);
-  setOperatingPoint ("Vgd", Ugd);
-  setOperatingPoint ("Vbs", Ubs);
-  setOperatingPoint ("Vbd", Ubd);
-  setOperatingPoint ("Vds", Ugs - Ugd);
   setOperatingPoint ("Cbd", Cbd);
   setOperatingPoint ("Cbs", Cbs);
   setOperatingPoint ("Cgs", Cgs);
@@ -628,6 +638,7 @@ void mosfet::initTR (void) {
 void mosfet::calcTR (nr_double_t) {
   calcDC ();
   transientMode = getPropertyInteger ("capModel");
+  saveOperatingPoints ();
   calcOperatingPoints ();
   transientMode = 0;
 
