@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: mosfet.h,v 1.14 2005/02/03 20:40:19 raimi Exp $
+ * $Id: mosfet.h,v 1.16 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -34,6 +34,7 @@ class mosfet : public circuit
   void calcDC (void);
   void initDC (void);
   void initModel (void);
+  void saveOperatingPoints (void);
   void calcOperatingPoints (void);
   void initAC (void);
   void calcAC (nr_double_t);
