@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: msline.cpp,v 1.41 2005/02/14 19:56:45 raimi Exp $
+ * $Id: msline.cpp,v 1.45 2005/06/02 18:17:55 raimi Exp $
  *
  */
 
@@ -33,6 +33,7 @@
 
 #include "logging.h"
 #include "complex.h"
+#include "matrix.h"
 #include "object.h"
 #include "node.h"
 #include "circuit.h"
@@ -47,15 +48,13 @@ msline::msline () : circuit (2) {
 }
 
 void msline::calcNoiseSP (nr_double_t) {
+  nr_double_t l = getPropertyDouble ("L");
+  if (l < 0) return;
+  // calculate noise using Bosma's theorem
   nr_double_t T = getPropertyDouble ("Temp");
-  nr_double_t l = exp (alpha * getPropertyDouble ("L"));
-  nr_double_t z = zl;
-  nr_double_t r = (z - z0) / (z + z0);
-  nr_double_t f = (l - 1) * (r * r - 1) / sqr (l - r * r) * kelvin (T) / T0;
-  setN (1, 1, -f * (r * r + l));
-  setN (2, 2, -f * (r * r + l));
-  setN (1, 2, +f * 2 * r * sqrt (l));
-  setN (2, 1, +f * 2 * r * sqrt (l));
+  matrix s = getMatrixS ();
+  matrix e = eye (getSize ());
+  setMatrixN (kelvin (T) / T0 * (e - s * transpose (conj (s))));
 }
 
 void msline::calcPropagation (nr_double_t frequency) {
@@ -83,11 +82,11 @@ void msline::calcPropagation (nr_double_t frequency) {
   analyseQuasiStatic (W, h, t, er, SModel, ZlEff, ErEff, WEff);
 
   // analyse dispersion of Zl and Er (use WEff here?)
-  analyseDispersion (WEff, h, er, ZlEff, ErEff, frequency, DModel,
+  analyseDispersion (W, h, er, ZlEff, ErEff, frequency, DModel,
 		     ZlEffFreq, ErEffFreq);
 
   // analyse losses of line
-  analyseLoss (WEff, t, er, rho, D, tand, ZlEffFreq, ZlEffFreq, ErEffFreq,
+  analyseLoss (W, t, er, rho, D, tand, ZlEff, ZlEff, ErEff,
 	       frequency, "Hammerstad", ac, ad);
 
   // calculate propagation constants and reference impedance
@@ -109,8 +108,8 @@ void msline::calcSP (nr_double_t frequency) {
   complex n = 2 * cosh (g * l) + (z + y) * sinh (g * l);
   complex s11 = (z - y) * sinh (g * l) / n;
   complex s21 = 2 / n;
-  setS (1, 1, s11); setS (2, 2, s11);
-  setS (1, 2, s21); setS (2, 1, s21);
+  setS (NODE_1, NODE_1, s11); setS (NODE_2, NODE_2, s11);
+  setS (NODE_1, NODE_2, s21); setS (NODE_2, NODE_1, s21);
 }
 
 /* This function calculates the quasi-static impedance of a microstrip
@@ -202,8 +201,7 @@ void msline::analyseQuasiStatic (nr_double_t W, nr_double_t h, nr_double_t t,
 
     // compute strip thickness effect
     if (t != 0) {
-      du1 = t / M_PI * log (1 + 4 * M_E / t / 
-			    sqr (coth (sqrt (6.517 * W / h))));
+      du1 = t / M_PI * log (1 + 4 * M_E / t / sqr (coth (sqrt (6.517 * u))));
     }
     else du1 = 0;
     du = du1 * (1 + sech (sqrt (er - 1))) / 2;
@@ -451,7 +449,8 @@ void msline::initDC (void) {
     nr_double_t g = t * W / rho / l;
     setVoltageSources (0);
     allocMatrixMNA ();
-    setY (1, 1, +g); setY (2, 2, +g); setY (1, 2, -g); setY (2, 1, -g);
+    setY (NODE_1, NODE_1, +g); setY (NODE_2, NODE_2, +g);
+    setY (NODE_1, NODE_2, -g); setY (NODE_2, NODE_1, -g);
   }
   else {
     // a DC short (voltage source V = 0 volts)
@@ -459,7 +458,7 @@ void msline::initDC (void) {
     setInternalVoltageSource (1);
     allocMatrixMNA ();
     clearY ();
-    voltageSource (1, 1, 2);
+    voltageSource (VSRC_1, NODE_1, NODE_2);
   }
 }
 
@@ -478,15 +477,14 @@ void msline::calcAC (nr_double_t frequency) {
   complex g = rect (alpha, beta);
   complex y11 = coth (g * l) / zl;
   complex y21 = -cosech (g * l) / zl;
-  setY (1, 1, y11); setY (2, 2, y11);
-  setY (1, 2, y21); setY (2, 1, y21);
+  setY (NODE_1, NODE_1, y11); setY (NODE_2, NODE_2, y11);
+  setY (NODE_1, NODE_2, y21); setY (NODE_2, NODE_1, y21);
 }
 
 void msline::calcNoiseAC (nr_double_t) {
+  nr_double_t l = getPropertyDouble ("L");
+  if (l < 0) return;
+  // calculate noise using Bosma's theorem
   nr_double_t T = getPropertyDouble ("Temp");
-  nr_double_t l = exp (alpha * getPropertyDouble ("L"));
-  nr_double_t z = zl;
-  nr_double_t f = 4.0 * kelvin (T) / T0 / z / (l - 1);
-  setN (1, 1, +f * (l + 1)); setN (1, 2, -f * 2 * sqrt (l));
-  setN (2, 2, +f * (l + 1)); setN (2, 1, -f * 2 * sqrt (l));
+  setMatrixN (4 * kelvin (T) / T0 * real (getMatrixY ()));
 }
