@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: opamp.h,v 1.3 2004/11/24 19:15:49 raimi Exp $
+ * $Id: opamp.h,v 1.5 2005/06/27 14:18:09 raimi Exp $
  *
  */
 
@@ -38,7 +38,7 @@ class opamp : public circuit
   void calcOperatingPoints (void);
 
  private:
-  nr_double_t gv, Uprev, Uold;
+  nr_double_t gv;
 };
 
 #endif /* __OPAMP_H__ */
