@@ -1,7 +1,7 @@
 /*
  * pac.h - AC power source class definitions
  *
- * Copyright (C) 2003, 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: pac.h,v 1.6 2004/09/12 14:09:19 ela Exp $
+ * $Id: pac.h,v 1.8 2005/06/02 18:17:52 raimi Exp $
  *
  */
 
@@ -30,8 +30,10 @@ class pac : public circuit
  public:
   pac ();
   void calcSP (nr_double_t);
+  void calcNoiseSP (nr_double_t);
   void calcDC (void);
   void calcAC (nr_double_t);
+  void calcNoiseAC (nr_double_t);
   void calcTR (nr_double_t);
 };
 
