@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: check_spice.cpp,v 1.11 2005/02/08 23:08:39 raimi Exp $
+ * $Id: check_spice.cpp,v 1.15 2005/06/02 18:17:56 raimi Exp $
  *
  */
 
@@ -40,10 +40,12 @@
 
 #include "constants.h"
 #include "check_spice.h"
+#include "qucs_producer.h"
 
 /* Global definitions for parser and checker. */
 struct definition_t * definition_root = NULL;
 struct definition_t * subcircuit_root = NULL;
+struct definition_t * device_root = NULL;
 int spice_errors = 0;
 char * spice_title = NULL;
 
@@ -258,7 +260,7 @@ static struct define_t * spice_find_definition (char * n) {
 static struct node_t * spice_translate_node (char * node) {
   struct node_t * n = create_node ();
   if (!strcmp (node, "0")) { // translate ground node
-    n->node = strdup ("gnd");
+    n->node = strdup (qucs_gnd);
   }
   else {                     // translated other node than ground
     n->node = (char *) malloc (5 + strlen (node));
@@ -588,15 +590,19 @@ spice_device_table[] = {
 
 /* This little function translates the Spice type definitions into Qucs
    types. */
-static void spice_translate_type (struct definition_t * def) {
+static int spice_translate_type (struct definition_t * def) {
   struct spice_device_table_t * tran;
-  for (tran = spice_device_table; tran->type; tran++) {
+  int found = 0;
+  for (tran = spice_device_table; tran->type && def->type; tran++) {
+    found = 1;
     if (!strcasecmp (tran->type, def->type)) {
       free (def->type);
       def->type = strdup (tran->trans);
+      found++;
       break;
     }
   }
+  return found;
 }
 
 // Helper structure for node translations.
@@ -704,11 +710,18 @@ static void spice_translate_nodes (struct definition_t * def, int pass) {
     for (int n = spice_count_nodes (def); n < entry->nodes; n++) {
       node = create_node ();
       if (nodes->Default[n] == 0) { // default is the ground node
-	node->node = strdup ("gnd");
+	node->node = strdup (qucs_gnd);
       }
       else {                        // default is some other node
 	struct node_t * t = spice_get_node (def, nodes->Default[n]);
-	node->node = strdup (t->node);
+	if (t != NULL)
+	  node->node = strdup (t->node);
+	else {
+	  // no node given, occurs in device descriptions
+	  char txt[16];
+	  sprintf (txt, "%s%d", "_node", n);
+	  node->node = strdup (txt);
+	}
       }
       def->nodes = netlist_append_nodes (def->nodes, node);
     }
@@ -770,7 +783,7 @@ static void spice_translate_units (struct definition_t * def) {
   }
 }
 
-/* This function append all the required Qucs properties not given in
+/* This function appends all the required Qucs properties not given in
    the Spice netlist. */
 static void spice_adjust_default_properties (struct definition_t * def) {
   struct define_t * entry = spice_get_qucs_definition (def);
@@ -794,6 +807,30 @@ static void spice_adjust_default_properties (struct definition_t * def) {
   }
 }
 
+/* This function appends the optional Qucs properties not given in the
+   Spice netlist. */
+static void spice_adjust_optional_properties (struct definition_t * def) {
+  struct define_t * entry = spice_get_qucs_definition (def);
+  if (entry == NULL) return;
+
+  // handle optional properties if requested
+  for (int i = 0; PROP_IS_PROP (entry->optional[i]); i++) {
+    struct property_t * prop = &entry->optional[i];
+    if (spice_find_property (def, prop->key) == NULL) {
+      struct pair_t * pair = create_pair ();
+      pair->key = strdup (prop->key);
+      pair->value = create_value ();
+      if (PROP_IS_VAL (*prop)) {
+	pair->value->value = prop->defaultval.d;
+      }
+      else {
+	pair->value->ident = strdup (prop->defaultval.s);
+      }
+      def->pairs = netlist_append_pairs (def->pairs, pair);
+    }
+  }
+}
+
 // Helper structure for property translations and aliases in devices.
 struct property_translation_t {
   char * key;
@@ -1326,6 +1363,65 @@ static void spice_free_values (struct definition_t * def) {
   }
 }
 
+/* Spice node lists. */
+struct node_t * spice_nodes = NULL;
+
+/* Collects the nodes specified in the PLOT and PRINT statement
+   marking them as potential "external" nodes. */
+static void spice_collect_external_nodes (struct definition_t * def) {
+  struct value_t * val;
+  foreach_value (def->values, val) {
+    if (val->hint & HINT_MSTART && val->next && val->ident[0] == 'V') {
+      struct node_t * n = spice_translate_node (val->next->ident);
+      if (!qucs_find_node (spice_nodes, n->node)) {
+	n->next = spice_nodes;
+	spice_nodes = n;
+      }
+      else spice_free_nodes (n);
+    }
+  }
+}
+
+/* This function goes through the list of device descriptions and
+   checks whether the given model is already in the list.  Returns
+   NULL if there is no such model. */
+static struct definition_t * spice_find_Model (char * instance) {
+  struct definition_t * dev;
+  for (dev = device_root; dev; dev = dev->next) {
+    if (!strcmp (dev->instance, instance))
+      return dev;
+  }
+  return NULL;
+}
+
+/* The function appends a new device model.  It creates the actual
+   model description and adjusts the nodes and optional/required
+   properties. */
+static struct definition_t * spice_add_Model (struct definition_t * def) {
+  struct definition_t * Model;
+  if (strcasecmp (def->type, "MODEL"))
+    return NULL;
+  if (spice_find_Model (def->instance))
+    return NULL;
+  Model = create_definition ();
+  Model->action = PROP_COMPONENT;
+  Model->instance = strdup (def->instance);
+  Model->define = def->define;
+  spice_adjust_device (Model, def);
+  if (spice_translate_type (Model)) {
+    spice_fixup_definition (Model);
+    spice_adjust_optional_properties (Model);
+    Model->next = device_root;
+    device_root = Model;
+  }
+  else {
+    free (Model->instance);
+    free (Model);
+    Model = NULL;
+  }
+  return Model;
+}
+
 /* This function must be called after the actual Spice netlist
    translator in order to adjust some references or whatever in the
    resulting netlist. */
@@ -1481,6 +1577,11 @@ spice_post_translator (struct definition_t * root) {
 	spice_add_property (root, "TR", "MaxIter", val->unit);
       }
     }
+    // post-process print and plot statements
+    if (def->action && !strcmp (def->type, "PRINT") ||
+	!strcmp (def->type, "PLOT")) {
+      spice_collect_external_nodes (def);
+    }
     // post-process untranslated netlist lines
     if (def->define == NULL) {
       def->text = spice_untranslated_text (def);
@@ -1680,7 +1781,10 @@ static struct definition_t * spice_translator (struct definition_t * root) {
       spice_fixup_definition (def);
     }
     else {
-      // skipped
+      // handle device description
+      if (!strcasecmp (def->type, "MODEL")) {
+	spice_add_Model (def);
+      }
     }
     // remove values if possible
     spice_free_values (def);
