@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: qucs_producer.h,v 1.3 2005/02/08 23:08:40 raimi Exp $
+ * $Id: qucs_producer.h,v 1.6 2005/06/02 18:17:56 raimi Exp $
  *
  */
 
@@ -28,8 +28,11 @@
 /* Externalize variables. */
 extern FILE * qucs_out;
 extern int    qucs_actions;
+extern char * qucs_gnd;
 
-/* Available functions of the checker. */
+/* Available functions of the producers. */
 void qucs_producer (void);
+int  qucs_find_node (struct node_t *, char *);
+void qucslib_producer (void);
 
 #endif /* __QUCS_PRODUCER_H__ */
