@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: equation.cpp,v 1.28 2005/02/08 23:08:32 raimi Exp $
+ * $Id: equation.cpp,v 1.34 2005/06/02 18:17:49 raimi Exp $
  *
  */
 
@@ -40,6 +40,7 @@
 #include "dataset.h"
 #include "strlist.h"
 #include "equation.h"
+#include "evaluate.h"
 #include "constants.h"
 #include "exception.h"
 #include "exceptionstack.h"
@@ -261,6 +262,7 @@ int assignment::evalType (void) {
 
 // Returns the result of the assignment.
 constant * assignment::evaluate (void) {
+  body->solvee = solvee;
   setResult (body->evaluate ());
   return getResult ();
 }
@@ -375,6 +377,7 @@ constant * application::evaluate (void) {
   // first evaluate each argument
   for (node * arg = args; arg != NULL; arg = arg->getNext ()) {
     if (arg->evaluated == 0) {
+      arg->solvee = solvee;
       arg->evaluate ();
       arg->evaluated++;
     }
@@ -392,24 +395,27 @@ constant * application::evaluate (void) {
 // Constructor creates an untyped instance of the equation node class.
 node::node () {
   tag = UNKNOWN;
-  output = evaluated = evalPossible = cycle = duplicate = 0;
+  dropdeps = output = evaluated = evalPossible = cycle = duplicate = 0;
   next = NULL;
   dependencies = NULL;
   dataDependencies = NULL;
   dropDependencies = NULL;
+  prepDependencies = NULL;
   txt = NULL;
   res = NULL;
   instance = NULL;
+  solvee = NULL;
 }
 
 // This constructor creates an typed instance of the equation node class.
 node::node (int type) {
   tag = type;
-  output = evaluated = evalPossible = cycle = duplicate = 0;
+  dropdeps = output = evaluated = evalPossible = cycle = duplicate = 0;
   next = NULL;
   dependencies = NULL;
   dataDependencies = NULL;
   dropDependencies = NULL;
+  prepDependencies = NULL;
   txt = NULL;
   res = NULL;
   instance = NULL;
@@ -420,6 +426,7 @@ node::~node () {
   if (dependencies) delete dependencies;
   if (dataDependencies) delete dataDependencies;
   if (dropDependencies) delete dropDependencies;
+  if (prepDependencies) delete prepDependencies;
   if (txt) free (txt);
   if (instance) free (instance);
 }
@@ -547,12 +554,28 @@ void node::addDropDependencies (char * dep) {
   dropDependencies->add (dep);
 }
 
+/* The function adds the given data dependency to the list of
+   dependencies which are going to be prepend. */
+void node::addPrepDependencies (char * dep) {
+  if (prepDependencies == NULL) prepDependencies = new strlist ();
+  prepDependencies->add (dep);
+}
+
 /* The function sets the data dependency list of the equation node. */
 void node::setDataDependencies (strlist * deps) {
   if (dataDependencies != NULL) delete dataDependencies;
   dataDependencies = deps ? new strlist (*deps) : NULL;
 }
 
+/* Evaluates the equation node and applies the data dependencies. */
+constant * node::calculate (void) {
+  constant * res = evaluate ();
+  strlist * deps = solvee->collectDataDependencies (this);
+  getResult()->setDataDependencies (deps);
+  delete deps;
+  return res;
+}
+
 // Constructor creates an instance of the checker class.
 checker::checker () {
   equations = NULL;
@@ -869,6 +892,7 @@ int equation_checker (int noundefined) {
 solver::solver () {
   equations = NULL;
   data = NULL;
+  generated = 0;
 }
 
 // Destructor deletes an instance of the solver class.
@@ -886,10 +910,8 @@ void solver::solve (void) {
     if (eqn->evalPossible && eqn->evaluated == 0) {
       // exception handling around evaluation
       try_running () {
-	eqn->evaluate ();
-	strlist * deps = collectDataDependencies (eqn);
-	eqn->getResult()->setDataDependencies (deps);
-	delete deps;
+	eqn->solvee = this;
+	eqn->calculate ();
       }
       // handle evaluation exceptions
       catch_exception () {
@@ -919,6 +941,29 @@ node * solver::addEquationData (vector * v) {
   return assign;
 }
 
+/* The function puts the given vector into the equation set.  The
+   resulting data vector is going to be copied and exported - given a
+   generated name based upon the second argument. */
+node * solver::addGeneratedEquation (vector * v, char * n) {
+  // create generated name
+  char * str = (char *) malloc (strlen (n) + 6);
+  sprintf (str, "%s.%04d", n, ++generated);
+  // copy data vector
+  vector * c = new vector (*v);
+  c->setName (str);
+  // put vector into the equation set and ensure data export as
+  // independent variable
+  node * res = addEquationData (c);
+  res->setInstance ("#generated");
+  res->setDependencies (new strlist ());
+  res->evalType ();
+  res->solvee = this;
+  res->evaluate ();
+  res->output = 1;
+  free (str);
+  return res;
+}
+
 /* Depending on the type of equation result the function converts the
    given equation node to one or more valid dataset vector(s). */
 vector * solver::dataVector (node * eqn) {
@@ -941,8 +986,8 @@ vector * solver::dataVector (node * eqn) {
       // convert matrix vector to a list of vectors
       matvec * mv = eqn->getResult()->mv;
       mv->setName (A(eqn)->result);
-      for (int r = 1; r <= mv->getRows (); r++) {
-	for (int c = 1; c <= mv->getCols (); c++) {
+      for (int r = 0; r < mv->getRows (); r++) {
+	for (int c = 0; c < mv->getCols (); c++) {
 	  // name gets automatically assigned
 	  vector * t = new vector (mv->get (r, c));
 	  // chain the vectors appropriately
@@ -955,8 +1000,8 @@ vector * solver::dataVector (node * eqn) {
     {
       // convert matrix to a list of vectors
       matrix * m = eqn->getResult()->m;
-      for (int r = 1; r <= m->getRows (); r++) {
-	for (int c = 1; c <= m->getCols (); c++) {
+      for (int r = 0; r < m->getRows (); r++) {
+	for (int c = 0; c < m->getCols (); c++) {
 	  vector * t = new vector ();
 	  t->setName (matvec::createMatrixString (A(eqn)->result, r, c));
 	  t->add (m->get (r, c));
@@ -1013,7 +1058,7 @@ void solver::findMatrixVectors (vector * v) {
 
   // loop through the dataset vector until no more matrix vector is found
   do {
-    r = c = s = 0; cand = NULL; deps = NULL;
+    r = c = s = -1; cand = NULL; deps = NULL;
     // go through the dataset
     for (vec = v; vec != NULL; vec = (vector *) vec->getNext ()) {
       // skip detected vectors
@@ -1047,7 +1092,7 @@ void solver::findMatrixVectors (vector * v) {
     // new matrix vector detected
     if (cand != NULL) {
       // create a new matrix vector and set the appropriate name
-      matvec * mv = new matvec (s, r, c);
+      matvec * mv = new matvec (s, r + 1, c + 1);
       mv->setName (cand);
       // go through the dataset vector once again
       for (vec = v; vec != NULL; vec = (vector *) vec->getNext ()) {
@@ -1061,6 +1106,7 @@ void solver::findMatrixVectors (vector * v) {
       }
       // now store this new matrix vector into the set of equations
       node * eqn = addEquationData (mv);
+      eqn->solvee = this;
       eqn->evaluate ();
       if (deps == NULL) {
 	strlist * deps = new strlist ();
@@ -1147,16 +1193,25 @@ int solver::dataSize (strlist * deps) {
    returns NULL if there are no such dependencies. */
 strlist * solver::collectDataDependencies (node * eqn) {
   strlist * sub, * datadeps = NULL;
-  strlist * deps = eqn->getDependencies ();
-  datadeps = eqn->getDataDependencies ();
-  datadeps = datadeps ? new strlist (*datadeps) : NULL;
-  for (int i = 0; i < deps->length (); i++) {
-    char * var = deps->get (i);
-    node * n = checker::findEquation (eqn::equations, var);
-    sub = strlist::join (datadeps, n->getDataDependencies ());
-    sub->del (n->getResult()->getDropDependencies ());
-    if (datadeps) delete datadeps;
-    datadeps = sub;
+  if (!eqn->getResult()->dropdeps) {
+    strlist * deps = eqn->getDependencies ();
+    datadeps = eqn->getDataDependencies ();
+    datadeps = datadeps ? new strlist (*datadeps) : NULL;
+    for (int i = 0; deps && i < deps->length (); i++) {
+      char * var = deps->get (i);
+      node * n = checker::findEquation (eqn::equations, var);
+      sub = strlist::join (datadeps, n->getDataDependencies ());
+      sub->del (n->getResult()->getDropDependencies ());
+      sub->add (n->getResult()->getPrepDependencies ());
+      if (datadeps) delete datadeps;
+      datadeps = sub;
+    }
+  }
+  if (datadeps) {
+    datadeps->add (eqn->getResult()->getPrepDependencies ());
+  } else {
+    datadeps = strlist::join (datadeps,
+			      eqn->getResult()->getPrepDependencies ());
   }
   datadeps = checker::foldDependencies (datadeps);
   datadeps->del (eqn->getResult()->getDropDependencies ());
@@ -1221,8 +1276,8 @@ int solver::findEquationResult (node * eqn) {
   // check each vector of a given matrix vector
   if (eqn->getType () == TAG_MATVEC) {
     matvec * mv = eqn->getResult()->mv;
-    for (int r = 1; r <= mv->getRows (); r++) {
-      for (int c = 1; c <= mv->getCols (); c++) {
+    for (int r = 0; r < mv->getRows (); r++) {
+      for (int c = 0; c < mv->getCols (); c++) {
 	char * str = matvec::createMatrixString (A(eqn)->result, r, c);
 	if (data->findDependency (str) || data->findVariable (str))
 	  return 1;
