@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: evaluate.cpp,v 1.25 2005/02/28 09:28:45 raimi Exp $
+ * $Id: evaluate.cpp,v 1.34 2005/06/07 07:49:06 raimi Exp $
  *
  */
 
@@ -37,7 +37,10 @@
 #include "object.h"
 #include "vector.h"
 #include "matrix.h"
+#include "poly.h"
+#include "spline.h"
 #include "constants.h"
+#include "circuit.h"
 #include "equation.h"
 #include "evaluate.h"
 #include "exception.h"
@@ -57,6 +60,9 @@ using namespace qucs;
 #define CHR(con) ((constant *) (con))->chr
 #define INT(con) ((int) D (con))
 
+#define A(a) ((assignment *) (a))
+#define R(r) ((reference *) (r))
+
 // Throws a math exception.
 #define THROW_MATH_EXCEPTION(txt) do { \
   qucs::exception * e = new qucs::exception (EXCEPTION_MATH); \
@@ -1048,6 +1054,69 @@ constant * evaluate::arg_mv (constant * args) {
   return res;
 }
 
+// ******* unwrap phase in radians ************
+constant * evaluate::unwrap_v_1 (constant * args) {
+  vector *    v1 = V (args->getResult (0));
+  constant * res = new constant (TAG_VECTOR);
+  res->v = new vector (unwrap (*v1));
+  return res;
+}
+
+constant * evaluate::unwrap_v_2 (constant * args) {
+  vector *    v1 = V (args->getResult (0));
+  nr_double_t d1 = D (args->getResult (1));
+  constant * res = new constant (TAG_VECTOR);
+  res->v = new vector (unwrap (*v1, fabs (d1)));
+  return res;
+}
+
+// ******** radian/degree conversion **********
+constant * evaluate::deg2rad_d (constant * args) {
+  nr_double_t d1 = D (args->getResult (0));
+  constant * res = new constant (TAG_DOUBLE);
+  res->d = rad (d1);
+  return res;
+}
+
+constant * evaluate::deg2rad_c (constant * args) {
+  complex *   c1 = C (args->getResult (0));
+  constant * res = new constant (TAG_DOUBLE);
+  res->c = new complex (rad (real (*c1)));
+  return res;
+}
+
+constant * evaluate::deg2rad_v (constant * args) {
+  vector *    v1 = V (args->getResult (0));
+  constant * res = new constant (TAG_VECTOR);
+  vector *     v = new vector ();
+  for (int i = 0; i < v1->getSize (); i++) v->add (rad (real (v1->get (i))));
+  res->v = new vector (*v);
+  return res;
+}
+
+constant * evaluate::rad2deg_d (constant * args) {
+  nr_double_t d1 = D (args->getResult (0));
+  constant * res = new constant (TAG_DOUBLE);
+  res->d = deg (d1);
+  return res;
+}
+
+constant * evaluate::rad2deg_c (constant * args) {
+  complex *   c1 = C (args->getResult (0));
+  constant * res = new constant (TAG_DOUBLE);
+  res->c = new complex (deg (real (*c1)));
+  return res;
+}
+
+constant * evaluate::rad2deg_v (constant * args) {
+  vector *    v1 = V (args->getResult (0));
+  constant * res = new constant (TAG_VECTOR);
+  vector *     v = new vector ();
+  for (int i = 0; i < v1->getSize (); i++) v->add (deg (real (v1->get (i))));
+  res->v = new vector (*v);
+  return res;
+}
+
 // ********** voltage decibel *****************
 constant * evaluate::dB_d (constant * args) {
   nr_double_t d1 = D (args->getResult (0));
@@ -1612,7 +1681,14 @@ constant * evaluate::index_mv_2 (constant * args) {
   int r = INT (args->getResult (1));
   int c = INT (args->getResult (2));
   constant * res = new constant (TAG_VECTOR);
-  res->v = new vector (mv->get (r, c));
+  if (r < 1 || r > mv->getRows () || c < 1 || c > mv->getCols ()) {
+    char txt[256];
+    sprintf (txt, "matvec indices [%d,%d] out of bounds [1-%d,1-%d]",
+	     r, c, mv->getRows (), mv->getCols ());
+    THROW_MATH_EXCEPTION (txt);
+  } else {
+    res->v = new vector (mv->get (r - 1, c - 1));
+  }
   return res;
 }
 
@@ -1620,7 +1696,13 @@ constant * evaluate::index_mv_1 (constant * args) {
   matvec * mv = MV (args->getResult (0));
   int i = INT (args->getResult (1));
   constant * res = new constant (TAG_MATRIX);
-  res->m = new matrix (mv->get (i));
+  if (i < 1 || i > mv->getSize ()) {
+    char txt[256];
+    sprintf (txt, "matvec index [%d] out of bounds [1-%d]", i, mv->getSize ());
+    THROW_MATH_EXCEPTION (txt);
+  } else {
+    res->m = new matrix (mv->get (i - 1));
+  }
   return res;
 }
 
@@ -1703,7 +1785,14 @@ constant * evaluate::index_m_2 (constant * args) {
   int r = INT (args->getResult (1));
   int c = INT (args->getResult (2));
   constant * res = new constant (TAG_COMPLEX);
-  res->c = new complex (m->get (r, c));
+  if (r < 1 || r > m->getRows () || c < 1 || c > m->getCols ()) {
+    char txt[256];
+    sprintf (txt, "matrix indices [%d,%d] out of bounds [1-%d,1-%d]",
+	     r, c, m->getRows (), m->getCols ());
+    THROW_MATH_EXCEPTION (txt);
+  } else {
+    res->c = new complex (m->get (r - 1, c - 1));
+  }
   return res;
 }
 
@@ -1715,6 +1804,50 @@ constant * evaluate::index_s_1 (constant * args) {
   return res;
 }
 
+// Interpolator helper macro without number of points given.
+#define INTERPOL_HELPER()                     \
+  constant * arg = new constant (TAG_DOUBLE); \
+  arg->d = 64;                                \
+  arg->solvee = args->getResult(0)->solvee;   \
+  arg->evaluate ();                           \
+  args->append (arg);
+
+// ***************** interpolation *****************
+constant * evaluate::interpolate_v_v (constant * args) {
+  INTERPOL_HELPER();
+  return interpolate_v_v_d (args);
+}
+
+constant * evaluate::interpolate_v_v_d (constant * args) {
+  vector * v1 = V (args->getResult (0));
+  vector * v2 = V (args->getResult (1));
+  int n  = INT (args->getResult (2));
+  constant * res = new constant (TAG_VECTOR);
+  if (v1->getSize () < 3) {
+    THROW_MATH_EXCEPTION ("interpolate: number of datapoints must be greater "
+			  "than 2");
+    res->v = new vector ();
+    return res;
+  }
+  nr_double_t last  = real (v2->get (v2->getSize () - 1));
+  nr_double_t first = real (v2->get (0));
+  constant * arg = new constant (TAG_VECTOR);
+  arg->v = new vector (::linspace (first, last, n));
+  arg->solvee = args->getResult(0)->solvee;
+  arg->evaluate ();
+  vector * val = new vector (n);
+  spline s (v1, v2);
+  for (int k = 0; k < arg->v->getSize (); k++) {
+    val->set (s.evaluate(real (arg->v->get (k))).f0, k);
+  }
+  res->v = val;
+  node * gen =
+    args->get(0)->solvee->addGeneratedEquation (arg->v, "Interpolate");
+  res->addPrepDependencies (A(gen)->result);
+  res->dropdeps = 1;
+  return res;
+}
+
 // ***************** matrix conversions *****************
 constant * evaluate::stoy_m (constant * args) {
   matrix * m = M (args->getResult (0));
@@ -1805,7 +1938,13 @@ constant * evaluate::twoport_m (constant * args) {
   char f = CHR (args->getResult (1));
   char t = CHR (args->getResult (2));
   constant * res = new constant (TAG_MATRIX);
-  res->m = new matrix (twoport (*m, toupper (f), toupper (t)));
+  if (m->getRows () < 2 || m->getCols () < 2) {
+    THROW_MATH_EXCEPTION ("invalid matrix dimensions for twoport "
+			  "transformation");
+    res->m = new matrix (*m);
+  }
+  else
+    res->m = new matrix (twoport (*m, toupper (f), toupper (t)));
   return res;
 }
 
@@ -1814,7 +1953,13 @@ constant * evaluate::twoport_mv (constant * args) {
   char f = CHR (args->getResult (1));
   char t = CHR (args->getResult (2));
   constant * res = new constant (TAG_MATVEC);
-  res->mv = new matvec (twoport (*mv, toupper (f), toupper (t)));
+  if (mv->getRows () < 2 || mv->getCols () < 2) {
+    THROW_MATH_EXCEPTION ("invalid matrix dimensions for twoport "
+			  "transformation");
+    res->mv = new matvec (*mv);
+  }
+  else
+    res->mv = new matvec (twoport (*mv, toupper (f), toupper (t)));
   return res;
 }
 
@@ -1881,24 +2026,18 @@ constant * evaluate::adjoint_mv (constant * args) {
   return res;
 }
 
-// ***************** s-parameter applications *****************
+// ***************** s-parameter stability factors *****************
 constant * evaluate::rollet_m (constant * args) {
   matrix * m = M (args->getResult (0));
   constant * res = new constant (TAG_DOUBLE);
-  nr_double_t k;
-  k = (1 - norm (m->get (1, 1)) - norm (m->get (2, 2)) + norm (det (*m))) /
-    2 / abs (m->get (1, 2) * m->get (2, 1));
-  res->d = k;
+  res->d = rollet (*m);
   return res;
 }
 
 constant * evaluate::rollet_mv (constant * args) {
   matvec * mv = MV (args->getResult (0));
   constant * res = new constant (TAG_VECTOR);
-  vector k;
-  k = (1 - norm (mv->get (1, 1)) - norm (mv->get (2, 2)) + norm (det (*mv))) /
-    2 / abs (mv->get (1, 2) * mv->get (2, 1));
-  res->v = new vector (k);
+  res->v = new vector (rollet (*mv));
   return res;
 }
 
@@ -1906,9 +2045,9 @@ constant * evaluate::mu1_m (constant * args) {
   matrix * m = M (args->getResult (0));
   constant * res = new constant (TAG_DOUBLE);
   nr_double_t k;
-  k = (1 - norm (m->get (1, 1))) /
-    (abs (m->get (2, 2) - conj (m->get (1, 1)) * det (*m)) +
-     abs (m->get (1, 2) * m->get (2, 1)));
+  k = (1 - norm (m->get (0, 0))) /
+    (abs (m->get (1, 1) - conj (m->get (0, 0)) * det (*m)) +
+     abs (m->get (0, 1) * m->get (1, 0)));
   res->d = k;
   return res;
 }
@@ -1917,9 +2056,9 @@ constant * evaluate::mu1_mv (constant * args) {
   matvec * mv = MV (args->getResult (0));
   constant * res = new constant (TAG_VECTOR);
   vector k;
-  k = (1 - norm (mv->get (1, 1))) /
-    (abs (mv->get (2, 2) - conj (mv->get (1, 1)) * det (*mv)) +
-     abs (mv->get (1, 2) * mv->get (2, 1)));
+  k = (1 - norm (mv->get (0, 0))) /
+    (abs (mv->get (1, 1) - conj (mv->get (0, 0)) * det (*mv)) +
+     abs (mv->get (0, 1) * mv->get (1, 0)));
   res->v = new vector (k);
   return res;
 }
@@ -1928,9 +2067,9 @@ constant * evaluate::mu2_m (constant * args) {
   matrix * m = M (args->getResult (0));
   constant * res = new constant (TAG_DOUBLE);
   nr_double_t k;
-  k = (1 - norm (m->get (2, 2))) /
-    (abs (m->get (1, 1) - conj (m->get (2, 2)) * det (*m)) +
-     abs (m->get (1, 2) * m->get (2, 1)));
+  k = (1 - norm (m->get (1, 1))) /
+    (abs (m->get (0, 0) - conj (m->get (1, 1)) * det (*m)) +
+     abs (m->get (0, 1) * m->get (1, 0)));
   res->d = k;
   return res;
 }
@@ -1939,13 +2078,414 @@ constant * evaluate::mu2_mv (constant * args) {
   matvec * mv = MV (args->getResult (0));
   constant * res = new constant (TAG_VECTOR);
   vector k;
-  k = (1 - norm (mv->get (2, 2))) /
-    (abs (mv->get (1, 1) - conj (mv->get (2, 2)) * det (*mv)) +
-     abs (mv->get (1, 2) * mv->get (2, 1)));
+  k = (1 - norm (mv->get (1, 1))) /
+    (abs (mv->get (0, 0) - conj (mv->get (1, 1)) * det (*mv)) +
+     abs (mv->get (0, 1) * mv->get (1, 0)));
   res->v = new vector (k);
   return res;
 }
 
+
+// ***************** vector creation *****************
+constant * evaluate::linspace (constant * args) {
+  nr_double_t start = D (args->getResult (0));
+  nr_double_t stop = D (args->getResult (1));
+  int points = INT (args->getResult (2));
+  constant * res = new constant (TAG_VECTOR);
+  if (points < 2) {
+    THROW_MATH_EXCEPTION ("linspace: number of points must be greater than 1");
+    res->v = new vector ();
+    return res;
+  }
+  res->v = new vector (::linspace (start, stop, points));
+  return res;
+}
+
+constant * evaluate::logspace (constant * args) {
+  nr_double_t start = D (args->getResult (0));
+  nr_double_t stop = D (args->getResult (1));
+  int points = INT (args->getResult (2));
+  constant * res = new constant (TAG_VECTOR);
+  if (points < 2) {
+    THROW_MATH_EXCEPTION ("logspace: number of points must be greater than 1");
+    res->v = new vector ();
+    return res;
+  }
+  if (start * stop <= 0) {
+    THROW_MATH_EXCEPTION ("logspace: invalid start/stop values");
+    res->v = new vector (points);
+    return res;
+  }
+  res->v = new vector (::logspace (start, stop, points));
+  return res;
+}
+
+// Circle helper macro with a number of points given.
+#define CIRCLE_HELPER_D(argi)                   \
+  int n = INT (args->getResult (argi));         \
+  if (n < 2) {                                  \
+    THROW_MATH_EXCEPTION ("Circle: number of points must be greater than 1"); \
+    constant * res = new constant (TAG_VECTOR); \
+    res->v = new vector ();                     \
+    return res;                                 \
+  }                                             \
+  constant * arg = new constant (TAG_VECTOR);   \
+  arg->v = new vector (::linspace (0, 360, n)); \
+  arg->solvee = args->getResult(0)->solvee;     \
+  arg->evaluate ();                             \
+  delete args->get(argi);                       \
+  args->get((argi)-1)->setNext (NULL);          \
+  args->append (arg);
+
+// Circle helper macro with no additional argument given.
+#define CIRCLE_HELPER_A()                        \
+  constant * arg = new constant (TAG_VECTOR);    \
+  arg->v = new vector (::linspace (0, 360, 64)); \
+  arg->solvee = args->getResult(0)->solvee;      \
+  arg->evaluate ();                              \
+  args->append (arg);
+
+// ***************** s-parameter noise circles *****************
+constant * evaluate::noise_circle_d_v (constant * args) {
+  vector * Sopt = V (args->getResult (0));
+  vector * Fmin = V (args->getResult (1));
+  vector * Rn   = V (args->getResult (2));
+  nr_double_t F = D (args->getResult (3));
+  vector * arc  = V (args->getResult (4));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector N = circuit::z0 / 4 / *Rn * (F - *Fmin) * norm (1 + *Sopt);
+  vector R = sqrt (N * N + N * (1 - norm (*Sopt))) / (1 + N);
+  vector C = *Sopt / (1 + N);
+  vector * circle = new vector (C.getSize () * arc->getSize ());
+  int i, a, j; complex v;
+  for (i = 0, j = 0; i < C.getSize (); i++) {
+    for (a = 0; a < arc->getSize (); a++, j++) {
+      v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+      circle->set (v, j);
+    }
+  }
+
+  node * gen = args->get(4)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::noise_circle_d_d (constant * args) {
+  CIRCLE_HELPER_D (4);
+  return noise_circle_d_v (args);
+}
+
+constant * evaluate::noise_circle_d (constant * args) {
+  CIRCLE_HELPER_A ();
+  return noise_circle_d_v (args);
+}
+
+constant * evaluate::noise_circle_v_v (constant * args) {
+  vector * Sopt = V (args->getResult (0));
+  vector * Fmin = V (args->getResult (1));
+  vector * Rn   = V (args->getResult (2));
+  vector * F    = V (args->getResult (3));
+  vector * arc  = V (args->getResult (4));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector * circle =
+    new vector (Sopt->getSize () * arc->getSize () * F->getSize ());
+  int i, a, j, f; complex v; vector N, R, C;
+  for (f = 0; f < F->getSize (); f++) {
+    N = circuit::z0 / 4 / *Rn * (F->get (f) - *Fmin) * norm (1 + *Sopt);
+    R = sqrt (N * N + N * (1 - norm (*Sopt))) / (1 + N);
+    C = *Sopt / (1 + N);
+    for (i = 0; i < C.getSize (); i++) {
+      for (a = 0; a < arc->getSize (); a++) {
+	v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+	j = i * F->getSize () * arc->getSize () + f * arc->getSize () + a;
+	circle->set (v, j);
+      }
+    }
+  }
+
+  node * gen;
+  gen = args->get(3)->solvee->addGeneratedEquation (F, "NF");
+  res->addPrepDependencies (A(gen)->result);
+  gen = args->get(4)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::noise_circle_v_d (constant * args) {
+  CIRCLE_HELPER_D (4);
+  return noise_circle_v_v (args);
+}
+
+constant * evaluate::noise_circle_v (constant * args) {
+  CIRCLE_HELPER_A ();
+  return noise_circle_v_v (args);
+}
+
+// ***************** s-parameter stability circles *****************
+constant * evaluate::stab_circle_l_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  vector * arc = V (args->getResult (1));
+  constant * res = new constant (TAG_VECTOR);
+  vector D = norm (S->get (1, 1)) - norm (det (*S));
+  vector C = (conj (S->get (1, 1)) - S->get (0, 0) * conj (det (*S))) / D;
+  vector R = abs (S->get (0, 1)) * abs (S->get (1, 0)) / D;
+  vector * circle = new vector (S->getSize () * arc->getSize ());
+  int a, d, i; complex v;
+  for (i = 0, d = 0; i < S->getSize (); i++) {
+    for (a = 0; a < arc->getSize (); a++, d++) {
+      v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+      circle->set (v, d);
+    }
+  }
+  node * gen = args->get(1)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::stab_circle_l_d (constant * args) {
+  CIRCLE_HELPER_D (1);
+  return stab_circle_l_v (args);
+}
+
+constant * evaluate::stab_circle_l (constant * args) {
+  CIRCLE_HELPER_A ();
+  return stab_circle_l_v (args);
+}
+
+constant * evaluate::stab_circle_s_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  vector * arc = V (args->getResult (1));
+  constant * res = new constant (TAG_VECTOR);
+  vector D = norm (S->get (0, 0)) - norm (det (*S));
+  vector C = (conj (S->get (0, 0)) - S->get (1, 1) * conj (det (*S))) / D;
+  vector R = abs (S->get (0, 1)) * abs (S->get (1, 0)) / D;
+  vector * circle = new vector (S->getSize () * arc->getSize ());
+  int a, d, i; complex v;
+  for (i = 0, d = 0; i < S->getSize (); i++) {
+    for (a = 0; a < arc->getSize (); a++, d++) {
+      v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+      circle->set (v, d);
+    }
+  }
+  node * gen = args->get(1)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::stab_circle_s_d (constant * args) {
+  CIRCLE_HELPER_D (1);
+  return stab_circle_s_v (args);
+}
+
+constant * evaluate::stab_circle_s (constant * args) {
+  CIRCLE_HELPER_A ();
+  return stab_circle_s_v (args);
+}
+
+// ***************** s-parameter gain circles *****************
+constant * evaluate::ga_circle_d_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  nr_double_t G = D (args->getResult (1));
+  vector * arc = V (args->getResult (2));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector g, D, c, s, k, C, R, d;
+  D = det (*S);
+  c = S->get (0, 0) - conj (S->get (1, 1)) * D;
+  k = rollet (*S);
+  s = S->get (0, 1) * S->get (1, 0);
+  g = G / norm (S->get (1, 0));
+  d = 1 + g * (norm (S->get (0, 0)) - norm (D));
+  C = g * conj (c) / d;
+  R = sqrt (1 - 2 * k * g * abs (s) + g * g * norm (s)) / abs (d);
+
+  vector * circle = new vector (S->getSize () * arc->getSize ());
+  int i, a, j; complex v;
+  for (i = 0, j = 0; i < C.getSize (); i++) {
+    for (a = 0; a < arc->getSize (); a++, j++) {
+      v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+      circle->set (v, j);
+    }
+  }
+
+  node * gen = args->get(2)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::ga_circle_d_d (constant * args) {
+  CIRCLE_HELPER_D (2);
+  return ga_circle_d_v (args);
+}
+
+constant * evaluate::ga_circle_d (constant * args) {
+  CIRCLE_HELPER_A ();
+  return ga_circle_d_v (args);
+}
+
+constant * evaluate::ga_circle_v_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  vector * G = V (args->getResult (1));
+  vector * arc = V (args->getResult (2));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector * circle =
+    new vector (S->getSize () * arc->getSize () * G->getSize ());
+  int i, a, j, f; complex v; vector g, D, c, s, k, R, C, d;
+  D = det (*S);
+  c = S->get (0, 0) - conj (S->get (1, 1)) * D;
+  k = rollet (*S);
+  s = S->get (0, 1) * S->get (1, 0);
+  for (f = 0; f < G->getSize (); f++) {
+    g = G->get (f) / norm (S->get (1, 0));
+    d = 1 + g * (norm (S->get (0, 0)) - norm (D));
+    C = g * conj (c) / d;
+    R = sqrt (1 - 2 * k * g * abs (s) + g * g * norm (s)) / abs (d);
+    for (i = 0; i < C.getSize (); i++) {
+      for (a = 0; a < arc->getSize (); a++) {
+	v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+	j = i * G->getSize () * arc->getSize () + f * arc->getSize () + a;
+	circle->set (v, j);
+      }
+    }
+  }
+
+  node * gen;
+  gen = args->get(1)->solvee->addGeneratedEquation (G, "Ga");
+  res->addPrepDependencies (A(gen)->result);
+  gen = args->get(2)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::ga_circle_v_d (constant * args) {
+  CIRCLE_HELPER_D (2);
+  return ga_circle_v_v (args);
+}
+
+constant * evaluate::ga_circle_v (constant * args) {
+  CIRCLE_HELPER_A ();
+  return ga_circle_v_v (args);
+}
+
+constant * evaluate::gp_circle_d_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  nr_double_t G = D (args->getResult (1));
+  vector * arc = V (args->getResult (2));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector g, D, c, s, k, C, R, d;
+  D = det (*S);
+  c = S->get (1, 1) - conj (S->get (0, 0)) * D;
+  k = rollet (*S);
+  s = S->get (0, 1) * S->get (1, 0);
+  g = G / norm (S->get (1, 0));
+  d = 1 + g * (norm (S->get (1, 1)) - norm (D));
+  C = g * conj (c) / d;
+  R = sqrt (1 - 2 * k * g * abs (s) + g * g * norm (s)) / abs (d);
+
+  vector * circle = new vector (S->getSize () * arc->getSize ());
+  int i, a, j; complex v;
+  for (i = 0, j = 0; i < C.getSize (); i++) {
+    for (a = 0; a < arc->getSize (); a++, j++) {
+      v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+      circle->set (v, j);
+    }
+  }
+
+  node * gen = args->get(2)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::gp_circle_d_d (constant * args) {
+  CIRCLE_HELPER_D (2);
+  return gp_circle_d_v (args);
+}
+
+constant * evaluate::gp_circle_d (constant * args) {
+  CIRCLE_HELPER_A ();
+  return gp_circle_d_v (args);
+}
+
+constant * evaluate::gp_circle_v_v (constant * args) {
+  matvec * S = MV (args->getResult (0));
+  vector * G = V (args->getResult (1));
+  vector * arc = V (args->getResult (2));
+
+  constant * res = new constant (TAG_VECTOR);
+  vector * circle =
+    new vector (S->getSize () * arc->getSize () * G->getSize ());
+  int i, a, j, f; complex v; vector g, D, c, s, k, R, C, d;
+  D = det (*S);
+  c = S->get (1, 1) - conj (S->get (0, 0)) * D;
+  k = rollet (*S);
+  s = S->get (0, 1) * S->get (1, 0);
+  for (f = 0; f < G->getSize (); f++) {
+    g = G->get (f) / norm (S->get (1, 0));
+    d = 1 + g * (norm (S->get (1, 1)) - norm (D));
+    C = g * conj (c) / d;
+    R = sqrt (1 - 2 * k * g * abs (s) + g * g * norm (s)) / abs (d);
+    for (i = 0; i < C.getSize (); i++) {
+      for (a = 0; a < arc->getSize (); a++) {
+	v = C.get (i) + R.get (i) * exp (rect (0, 1) * rad (arc->get (a)));
+	j = i * G->getSize () * arc->getSize () + f * arc->getSize () + a;
+	circle->set (v, j);
+      }
+    }
+  }
+
+  node * gen;
+  gen = args->get(1)->solvee->addGeneratedEquation (G, "Gp");
+  res->addPrepDependencies (A(gen)->result);
+  gen = args->get(2)->solvee->addGeneratedEquation (arc, "Arcs");
+  res->addPrepDependencies (A(gen)->result);
+  res->v = circle;
+  return res;
+}
+
+constant * evaluate::gp_circle_v_d (constant * args) {
+  CIRCLE_HELPER_D (2);
+  return gp_circle_v_v (args);
+}
+
+constant * evaluate::gp_circle_v (constant * args) {
+  CIRCLE_HELPER_A ();
+  return gp_circle_v_v (args);
+}
+
+constant * evaluate::plot_vs_v (constant * args) {
+  vector * v = V (args->getResult (0));
+  vector * a = V (args->getResult (1));
+  constant * res = new constant (TAG_VECTOR);
+  res->v = new vector (*v);
+  node * gen = args->get(1)->solvee->addGeneratedEquation (a, "Versus");
+  res->addPrepDependencies (A(gen)->result);
+  res->dropdeps = 1;
+  return res;
+}
+
+constant * evaluate::plot_vs_mv (constant * args) {
+  matvec * mv = MV (args->getResult (0));
+  vector * a = V (args->getResult (1));
+  constant * res = new constant (TAG_MATVEC);
+  res->mv = new matvec (*mv);
+  node * gen = args->get(1)->solvee->addGeneratedEquation (a, "Versus");
+  res->addPrepDependencies (A(gen)->result);
+  res->dropdeps = 1;
+  return res;
+}
+
 // Array containing all kinds of applications.
 struct application_t eqn::applications[] = {
   { "+", TAG_DOUBLE,  evaluate::plus_d, 1, { TAG_DOUBLE  } },
@@ -2100,6 +2640,17 @@ struct application_t eqn::applications[] = {
   { "arg", TAG_MATRIX,  evaluate::arg_m,  1, { TAG_MATRIX } },
   { "arg", TAG_MATVEC,  evaluate::arg_mv, 1, { TAG_MATVEC } },
 
+  { "unwrap", TAG_VECTOR,  evaluate::unwrap_v_1, 1, { TAG_VECTOR  } },
+  { "unwrap", TAG_VECTOR,  evaluate::unwrap_v_2, 2,
+    { TAG_VECTOR, TAG_DOUBLE } },
+
+  { "deg2rad", TAG_DOUBLE,  evaluate::deg2rad_d, 1, { TAG_DOUBLE  } },
+  { "deg2rad", TAG_DOUBLE,  evaluate::deg2rad_c, 1, { TAG_COMPLEX } },
+  { "deg2rad", TAG_VECTOR,  evaluate::deg2rad_v, 1, { TAG_VECTOR  } },
+  { "rad2deg", TAG_DOUBLE,  evaluate::rad2deg_d, 1, { TAG_DOUBLE  } },
+  { "rad2deg", TAG_DOUBLE,  evaluate::rad2deg_c, 1, { TAG_COMPLEX } },
+  { "rad2deg", TAG_VECTOR,  evaluate::rad2deg_v, 1, { TAG_VECTOR  } },
+
   { "dB", TAG_DOUBLE,  evaluate::dB_d, 1, { TAG_DOUBLE  } },
   { "dB", TAG_DOUBLE,  evaluate::dB_c, 1, { TAG_COMPLEX } },
   { "dB", TAG_VECTOR,  evaluate::dB_v, 1, { TAG_VECTOR  } },
@@ -2295,5 +2846,65 @@ struct application_t eqn::applications[] = {
   { "Mu2",    TAG_DOUBLE,  evaluate::mu2_m,     1, { TAG_MATRIX } },
   { "Mu2",    TAG_VECTOR,  evaluate::mu2_mv,    1, { TAG_MATVEC } },
 
+  { "linspace", TAG_VECTOR, evaluate::linspace, 3,
+    { TAG_DOUBLE, TAG_DOUBLE, TAG_DOUBLE } },
+  { "logspace", TAG_VECTOR, evaluate::logspace, 3,
+    { TAG_DOUBLE, TAG_DOUBLE, TAG_DOUBLE } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_d_v, 5,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_DOUBLE, TAG_VECTOR } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_d_d, 5,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_DOUBLE, TAG_DOUBLE } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_d, 4,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_DOUBLE } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_v_v, 5,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_VECTOR } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_v_d, 5,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_DOUBLE } },
+  { "NoiseCircle", TAG_VECTOR, evaluate::noise_circle_v, 4,
+    { TAG_VECTOR, TAG_VECTOR, TAG_VECTOR, TAG_VECTOR } },
+  { "StabCircleL", TAG_VECTOR, evaluate::stab_circle_l_v, 2,
+    { TAG_MATVEC, TAG_VECTOR } },
+  { "StabCircleL", TAG_VECTOR, evaluate::stab_circle_l_d, 2,
+    { TAG_MATVEC, TAG_DOUBLE } },
+  { "StabCircleL", TAG_VECTOR, evaluate::stab_circle_l, 1, { TAG_MATVEC } },
+  { "StabCircleS", TAG_VECTOR, evaluate::stab_circle_s_v, 2,
+    { TAG_MATVEC, TAG_VECTOR } },
+  { "StabCircleS", TAG_VECTOR, evaluate::stab_circle_s_d, 2,
+    { TAG_MATVEC, TAG_DOUBLE } },
+  { "StabCircleS", TAG_VECTOR, evaluate::stab_circle_s, 1, { TAG_MATVEC } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_d_v, 3,
+    { TAG_MATVEC, TAG_DOUBLE, TAG_VECTOR } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_d_d, 3,
+    { TAG_MATVEC, TAG_DOUBLE, TAG_DOUBLE } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_d, 2,
+    { TAG_MATVEC, TAG_DOUBLE } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_v_v, 3,
+    { TAG_MATVEC, TAG_VECTOR, TAG_VECTOR } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_v_d, 3,
+    { TAG_MATVEC, TAG_VECTOR, TAG_DOUBLE } },
+  { "GaCircle", TAG_VECTOR, evaluate::ga_circle_v, 2,
+    { TAG_MATVEC, TAG_VECTOR } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_d_v, 3,
+    { TAG_MATVEC, TAG_DOUBLE, TAG_VECTOR } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_d_d, 3,
+    { TAG_MATVEC, TAG_DOUBLE, TAG_DOUBLE } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_d, 2,
+    { TAG_MATVEC, TAG_DOUBLE } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_v_v, 3,
+    { TAG_MATVEC, TAG_VECTOR, TAG_VECTOR } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_v_d, 3,
+    { TAG_MATVEC, TAG_VECTOR, TAG_DOUBLE } },
+  { "GpCircle", TAG_VECTOR, evaluate::gp_circle_v, 2,
+    { TAG_MATVEC, TAG_VECTOR } },
+  { "PlotVs", TAG_VECTOR, evaluate::plot_vs_v, 2,
+    { TAG_VECTOR, TAG_VECTOR } },
+  { "PlotVs", TAG_MATVEC, evaluate::plot_vs_mv, 2,
+    { TAG_MATVEC, TAG_VECTOR } },
+
+  { "interpolate", TAG_VECTOR, evaluate::interpolate_v_v_d, 3,
+    { TAG_VECTOR, TAG_VECTOR, TAG_DOUBLE } },
+  { "interpolate", TAG_VECTOR, evaluate::interpolate_v_v, 2,
+    { TAG_VECTOR, TAG_VECTOR } },
+
   { NULL, 0, NULL, 0, { 0 } /* end of list */ }
 };
