@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: matrix.cpp,v 1.17 2005/02/08 23:08:32 raimi Exp $
+ * $Id: matrix.cpp,v 1.24 2005/06/02 18:17:50 raimi Exp $
  *
  */
 
@@ -94,19 +94,19 @@ matrix::~matrix () {
 
 // Returns the matrix element at the given row and column.
 complex matrix::get (int r, int c) {
-  return data[(r - 1) * cols + c - 1];
+  return data[r * cols + c];
 }
 
 // Sets the matrix element at the given row and column.
 void matrix::set (int r, int c, complex z) {
-  data[(r - 1) * cols + c - 1] = z;
+  data[r * cols + c] = z;
 }
 
 #ifdef DEBUG
 // Debug function: Prints the matrix object.
 void matrix::print (void) {
-  for (int r = 1; r <= rows; r++) {
-    for (int c = 1; c <= cols; c++) {
+  for (int r = 0; r < rows; r++) {
+    for (int c = 0; c < cols; c++) {
       fprintf (stderr, "%+.2e,%+.2e ", (double) real (get (r, c)),
       	       (double) imag (get (r, c)));
     }
@@ -119,8 +119,8 @@ void matrix::print (void) {
 matrix operator + (matrix a, matrix b) {
   assert (a.getRows () == b.getRows () && a.getCols () == b.getCols ());
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, a.get (r, c) + b.get (r, c));
   return res;
 }
@@ -129,8 +129,8 @@ matrix operator + (matrix a, matrix b) {
 matrix matrix::operator += (matrix a) {
   assert (a.getRows () == rows && a.getCols () == cols);
   int r, c, i;
-  for (i = 0, r = 1; r <= a.getRows (); r++, i++)
-    for (c = 1; c <= a.getCols (); c++, i++)
+  for (i = 0, r = 0; r < a.getRows (); r++)
+    for (c = 0; c < a.getCols (); c++, i++)
       data[i] += a.get (r, c);
   return *this;
 }
@@ -139,8 +139,8 @@ matrix matrix::operator += (matrix a) {
 matrix operator - (matrix a, matrix b) {
   assert (a.getRows () == b.getRows () && a.getCols () == b.getCols ());
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, a.get (r, c) - b.get (r, c));
   return res;
 }
@@ -149,8 +149,8 @@ matrix operator - (matrix a, matrix b) {
 matrix matrix::operator - () {
   matrix res (getRows (), getCols ());
   int r, c, i;
-  for (i = 0, r = 1; r <= getRows (); r++, i++)
-    for (c = 1; c <= getCols (); c++, i++)
+  for (i = 0, r = 0; r < getRows (); r++)
+    for (c = 0; c < getCols (); c++, i++)
       res.set (r, c, -data[i]);
   return res;
 }
@@ -159,8 +159,8 @@ matrix matrix::operator - () {
 matrix matrix::operator -= (matrix a) {
   assert (a.getRows () == rows && a.getCols () == cols);
   int r, c, i;
-  for (i = 0, r = 1; r <= a.getRows (); r++, i++)
-    for (c = 1; c <= a.getCols (); c++, i++)
+  for (i = 0, r = 0; r < a.getRows (); r++)
+    for (c = 0; c < a.getCols (); c++, i++)
       data[i] -= a.get (r, c);
   return *this;
 }
@@ -168,8 +168,8 @@ matrix matrix::operator -= (matrix a) {
 // Matrix scaling.
 matrix operator * (matrix a, complex z) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, a.get (r, c) * z);
   return res;
 }
@@ -177,8 +177,8 @@ matrix operator * (matrix a, complex z) {
 // Matrix scaling.
 matrix operator * (matrix a, nr_double_t d) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, a.get (r, c) * d);
   return res;
 }
@@ -191,8 +191,8 @@ matrix operator * (nr_double_t d, matrix a) {
 // Matrix scaling.
 matrix operator / (matrix a, complex z) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, a.get (r, c) / z);
   return res;
 }
@@ -208,9 +208,9 @@ matrix operator * (matrix a, matrix b) {
   int r, c, i, n = a.getCols ();
   complex z;
   matrix res (a.getRows (), b.getCols ());
-  for (r = 1; r <= a.getRows (); r++) {
-    for (c = 1; c <= b.getCols (); c++) {
-      for (i = 1, z = 0; i <= n; i++) z += a.get (r, i) * b.get (i, c);
+  for (r = 0; r < a.getRows (); r++) {
+    for (c = 0; c < b.getCols (); c++) {
+      for (i = 0, z = 0; i < n; i++) z += a.get (r, i) * b.get (i, c);
       res.set (r, c, z);
     }
   }
@@ -220,8 +220,8 @@ matrix operator * (matrix a, matrix b) {
 // Transpose the matrix.
 matrix transpose (matrix a) {
   matrix res (a.getCols (), a.getRows ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (c, r, a.get (r, c));
   return res;
 }
@@ -229,8 +229,8 @@ matrix transpose (matrix a) {
 // Conjugate complex matrix.
 matrix conj (matrix a) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, conj (a.get (r, c)));
   return res;
 }
@@ -238,8 +238,8 @@ matrix conj (matrix a) {
 // Computes magnitude of each matrix element.
 matrix abs (matrix a) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, abs (a.get (r, c)));
   return res;
 }
@@ -247,8 +247,8 @@ matrix abs (matrix a) {
 // Computes the argument of each matrix element.
 matrix arg (matrix a) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, arg (a.get (r, c)));
   return res;
 }
@@ -256,8 +256,8 @@ matrix arg (matrix a) {
 // Real part matrix.
 matrix real (matrix a) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, real (a.get (r, c)));
   return res;
 }
@@ -265,8 +265,8 @@ matrix real (matrix a) {
 // Imaginary part matrix.
 matrix imag (matrix a) {
   matrix res (a.getRows (), a.getCols ());
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, imag (a.get (r, c)));
   return res;
 }
@@ -280,8 +280,8 @@ matrix adjoint (matrix a) {
 // Create identity matrix with specified number of rows and columns.
 matrix eye (int rs, int cs) {
   matrix res (rs, cs);
-  for (int r = 1; r <= res.getRows (); r++)
-    for (int c = 1; c <= res.getCols (); c++)
+  for (int r = 0; r < res.getRows (); r++)
+    for (int c = 0; c < res.getCols (); c++)
       if (r == c) res.set (r, c, 1);
   return res;
 }
@@ -296,9 +296,9 @@ matrix eye (int s) {
 complex cofactor (matrix a, int u, int v) {
   matrix res (a.getRows () - 1, a.getCols () - 1);
   int r, c, ra, ca;
-  for (ra = r = 1; r <= res.getRows (); r++, ra++) {
+  for (ra = r = 0; r < res.getRows (); r++, ra++) {
     if (ra == u) ra++;
-    for (ca = c = 1; c <= res.getCols (); c++, ca++) {
+    for (ca = c = 0; c < res.getCols (); c++, ca++) {
       if (ca == v) ca++;
       res.set (r, c, a.get (ra, ca));
     }
@@ -315,14 +315,14 @@ complex detLaplace (matrix a) {
   if (s > 1) {
     /* always use the first row for sub-determinant, but you should
        use the row or column with most zeros in it */
-    int r = 1;
-    for (int i = 1; i <= s; i++) {
+    int r = 0;
+    for (int i = 0; i < s; i++) {
       res += a.get (r, i) * cofactor (a, r, i);
     }
     return res;
   }
   else if (s == 1) {
-    return a.get (1, 1);
+    return a (0, 0);
   }
   return 1;
 }
@@ -339,15 +339,15 @@ complex detGauss (matrix a) {
 
   // return special matrix cases
   if (n == 0) return 1;
-  if (n == 1) return a.get (1, 1);
+  if (n == 1) return a (0, 0);
 
   // make copy of original matrix
   b = matrix (a);
 
   // triangulate the matrix
-  for (res = 1, i = 1; i < n; i++) {
+  for (res = 1, i = 0; i < n; i++) {
     // find maximum column value for pivoting
-    for (MaxPivot = 0, pivot = r = i; r <= n; r++) {
+    for (MaxPivot = 0, pivot = r = i; r < n; r++) {
       if (abs (b.get (r, i)) > MaxPivot) {
 	MaxPivot = abs (b.get (r, i));
 	pivot = r;
@@ -357,16 +357,16 @@ complex detGauss (matrix a) {
     assert (MaxPivot != 0);
     if (i != pivot) { b.exchangeRows (i, pivot); res = -res; }
     // compute new rows and columns
-    for (r = i + 1; r <= n; r++) {
+    for (r = i + 1; r < n; r++) {
       f = b.get (r, i) / b.get (i, i);
-      for (c = i + 1; c <= n; c++) {
+      for (c = i + 1; c < n; c++) {
 	b.set (r, c, b.get (r, c) - f * b.get (i, c));
       }
     }
   }
 
   // now compute determinant by multiplying diagonal
-  for (i = 1; i <= n; i++) res *= b.get (i, i);
+  for (i = 0; i < n; i++) res *= b.get (i, i);
   return res;
 }
 
@@ -384,8 +384,8 @@ matrix inverseLaplace (matrix a) {
   matrix res (a.getRows (), a.getCols ());
   complex d = detLaplace (a);
   assert (abs (d) != 0); // singular matrix
-  for (int r = 1; r <= a.getRows (); r++)
-    for (int c = 1; c <= a.getCols (); c++)
+  for (int r = 0; r < a.getRows (); r++)
+    for (int c = 0; c < a.getCols (); c++)
       res.set (r, c, cofactor (a, c, r) / d);
   return res;
 }
@@ -403,11 +403,11 @@ matrix inverseGaussJordan (matrix a) {
   e = eye (n);
 
   // create the eye matrix in 'b' and the result in 'e'
-  for (i = 1; i <= n; i++) {
+  for (i = 0; i < n; i++) {
     // find maximum column value for pivoting
-    for (MaxPivot = 0, pivot = r = i; r <= n; r++) {
-      if (abs (b.get (r, i)) > MaxPivot) {
-	MaxPivot = abs (b.get (r, i));
+    for (MaxPivot = 0, pivot = r = i; r < n; r++) {
+      if (abs (b (r, i)) > MaxPivot) {
+	MaxPivot = abs (b (r, i));
 	pivot = r;
       }
     }
@@ -419,19 +419,17 @@ matrix inverseGaussJordan (matrix a) {
     }
 
     // compute current row
-    f = b.get (i, i);
-    for (c = 1; c <= n; c++) {
-      b.set (i, c, b.get (i, c) / f);
-      e.set (i, c, e.get (i, c) / f);
+    for (f = b (i, i), c = 0; c < n; c++) {
+      b (i, c) /= f;
+      e (i, c) /= f;
     }
 
     // compute new rows and columns
-    for (r = 1; r <= n; r++) {
+    for (r = 0; r < n; r++) {
       if (r != i) {
-	f = b.get (r, i);
-	for (c = 1; c <= n; c++) {
-	  b.set (r, c, b.get (r, c) - f * b.get (i, c));
-	  e.set (r, c, e.get (r, c) - f * e.get (i, c));
+	for (f = b (r, i), c = 0; c < n; c++) {
+	  b (r, c) -= f * b (i, c);
+	  e (r, c) -= f * e (i, c);
 	}
       }
     }
@@ -512,92 +510,92 @@ matrix ytos (matrix y, complex z0) {
 
 // Converts scattering parameters to chain matrix.
 matrix stoa (matrix s, complex z1) {
-  assert (s.getRows () == 2 && s.getCols () == 2);
+  assert (s.getRows () >= 2 && s.getCols () >= 2);
   complex z2 = z1;
-  complex d = s.get (1, 1) * s.get (2, 2) - s.get (1, 2) * s.get (2, 1);
-  complex n = 2.0 * s.get (2, 1) * sqrt (fabs (real (z1) * real (z2)));
+  complex d = s (0, 0) * s (1, 1) - s (0, 1) * s (1, 0);
+  complex n = 2.0 * s (1, 0) * sqrt (fabs (real (z1) * real (z2)));
   matrix a (2);
-  a.set (1, 1, (conj (z1) + z1 * s.get (1, 1) - 
-		conj (z1) * s.get (2, 2) - z1 * d) / n);
-  a.set (1, 2, (conj (z1) * conj (z2) + z1 * conj (z2) * s.get (1, 1) +
-		conj (z1) * z2 * s.get (2, 2) + z1 * z2 * d) / n);
-  a.set (2, 1, (1.0 - s.get (1, 1) - s.get (2, 2) + d) / n);
-  a.set (2, 2, (conj (z2) - conj (z2) * s.get (1, 1) +
-		z2 * s.get (2, 2) - z2 * d) / n);
+  a.set (0, 0, (conj (z1) + z1 * s (0, 0) - 
+		conj (z1) * s (1, 1) - z1 * d) / n);
+  a.set (0, 1, (conj (z1) * conj (z2) + z1 * conj (z2) * s (0, 0) +
+		conj (z1) * z2 * s (1, 1) + z1 * z2 * d) / n);
+  a.set (1, 0, (1.0 - s (0, 0) - s (1, 1) + d) / n);
+  a.set (1, 1, (conj (z2) - conj (z2) * s (0, 0) +
+		z2 * s (1, 1) - z2 * d) / n);
   return a;
 }
 
 // Converts chain matrix to scattering parameters.
 matrix atos (matrix a, complex z1) {
-  assert (a.getRows () == 2 && a.getCols () == 2);
+  assert (a.getRows () >= 2 && a.getCols () >= 2);
   complex z2 = z1;
   complex d = 2.0 * sqrt (fabs (real (z1) * real (z2)));
-  complex n = a.get (1, 1) * z2 + a.get (1, 2) + 
-    a.get (2, 1) * z1 * z2 + a.get (2, 2) * z1;
+  complex n = a (0, 0) * z2 + a (0, 1) + 
+    a (1, 0) * z1 * z2 + a (1, 1) * z1;
   matrix s (2);
-  s.set (1, 1, (a.get (1, 1) * z2 + a.get (1, 2) - a.get (2, 1) * 
-		conj (z1) * z2 - a.get (2, 2) * conj (z1)) / n);
-  s.set (1, 2, (a.get (1, 1) * a.get (2, 2) - 
-		a.get (1, 2) * a.get (2, 1)) * d / n);
-  s.set (2, 1, d / n);
-  s.set (2, 2, (a.get (2, 2) * z1 - a.get (1, 1) * conj (z2) +
-		a.get (1, 2) - a.get (2, 1) * z1 * conj (z2)) / n);
+  s.set (0, 0, (a (0, 0) * z2 + a (0, 1) - a (1, 0) * 
+		conj (z1) * z2 - a (1, 1) * conj (z1)) / n);
+  s.set (0, 1, (a (0, 0) * a (1, 1) - 
+		a (0, 1) * a (1, 0)) * d / n);
+  s.set (1, 0, d / n);
+  s.set (1, 1, (a (1, 1) * z1 - a (0, 0) * conj (z2) +
+		a (0, 1) - a (1, 0) * z1 * conj (z2)) / n);
   return s;
 }
 
 // Converts scattering parameters to hybrid matrix.
 matrix stoh (matrix s, complex z1) {
-  assert (s.getRows () == 2 && s.getCols () == 2);
+  assert (s.getRows () >= 2 && s.getCols () >= 2);
   complex z2 = z1;
-  complex n = s.get (1, 2) * s.get (2, 1);
-  complex d = (1.0 - s.get (1, 1)) * (1.0 + s.get (2, 2)) + n;
+  complex n = s (0, 1) * s (1, 0);
+  complex d = (1.0 - s (0, 0)) * (1.0 + s (1, 1)) + n;
   matrix h (2);
-  h.set (1, 1, ((1.0 + s.get (1, 1)) * (1.0 + s.get (2, 2)) - n) * z1 / d);
-  h.set (1, 2, +2.0 * s.get (1, 2) / d);
-  h.set (2, 1, -2.0 * s.get (2, 1) / d);
-  h.set (2, 2, ((1.0 - s.get (1, 1)) * (1.0 - s.get (2, 2)) - n) / z2 / d);
+  h.set (0, 0, ((1.0 + s (0, 0)) * (1.0 + s (1, 1)) - n) * z1 / d);
+  h.set (0, 1, +2.0 * s (0, 1) / d);
+  h.set (1, 0, -2.0 * s (1, 0) / d);
+  h.set (1, 1, ((1.0 - s (0, 0)) * (1.0 - s (1, 1)) - n) / z2 / d);
   return h;
 }
 
 // Converts hybrid matrix to scattering parameters.
 matrix htos (matrix h, complex z1) {
-  assert (h.getRows () == 2 && h.getCols () == 2);
+  assert (h.getRows () >= 2 && h.getCols () >= 2);
   complex z2 = z1;
-  complex n = h.get (1, 2) * h.get (2, 1);
-  complex d = (1.0 + h.get (1, 1) / z1) * (1.0 + z2 * h.get (2, 2)) - n;
+  complex n = h (0, 1) * h (1, 0);
+  complex d = (1.0 + h (0, 0) / z1) * (1.0 + z2 * h (1, 1)) - n;
   matrix s (2);
-  s.set (1, 1, ((h.get (1, 1) / z1 - 1) * (1 + z2 * h.get (2, 2)) - n) / d);
-  s.set (1, 2, +2.0 * h.get (1, 2) / d);
-  s.set (2, 1, -2.0 * h.get (2, 1) / d);
-  s.set (2, 2, ((1 + h.get (1, 1) / z1) * (1 - z2 * h.get (2, 2)) + n) / d);
+  s.set (0, 0, ((h (0, 0) / z1 - 1) * (1 + z2 * h (1, 1)) - n) / d);
+  s.set (0, 1, +2.0 * h (0, 1) / d);
+  s.set (1, 0, -2.0 * h (1, 0) / d);
+  s.set (1, 1, ((1 + h (0, 0) / z1) * (1 - z2 * h (1, 1)) + n) / d);
   return s;
 }
 
 // Converts scattering parameters to second hybrid matrix.
 matrix stog (matrix s, complex z1) {
-  assert (s.getRows () == 2 && s.getCols () == 2);
+  assert (s.getRows () >= 2 && s.getCols () >= 2);
   complex z2 = z1;
-  complex n = s.get (1, 2) * s.get (2, 1);
-  complex d = (1.0 + s.get (1, 1)) * (1.0 - s.get (2, 2)) + n;
+  complex n = s (0, 1) * s (1, 0);
+  complex d = (1.0 + s (0, 0)) * (1.0 - s (1, 1)) + n;
   matrix g (2);
-  g.set (1, 1, ((1.0 - s.get (1, 1)) * (1.0 - s.get (2, 2)) - n) / z1 / d);
-  g.set (1, 2, -2.0 * s.get (1, 2) / d);
-  g.set (2, 1, +2.0 * s.get (2, 1) / d);
-  g.set (2, 2, ((1.0 + s.get (1, 1)) * (1.0 + s.get (2, 2)) - n) * z2 / d);
+  g.set (0, 0, ((1.0 - s (0, 0)) * (1.0 - s (1, 1)) - n) / z1 / d);
+  g.set (0, 1, -2.0 * s (0, 1) / d);
+  g.set (1, 0, +2.0 * s (1, 0) / d);
+  g.set (1, 1, ((1.0 + s (0, 0)) * (1.0 + s (1, 1)) - n) * z2 / d);
   return g;
 }
 
 // Converts second hybrid matrix to scattering parameters.
 matrix gtos (matrix g, complex z1) {
-  assert (g.getRows () == 2 && g.getCols () == 2);
+  assert (g.getRows () >= 2 && g.getCols () >= 2);
   complex z2 = z1;
-  complex n = g.get (1, 2) * g.get (2, 1);
-  complex d = (1.0 + g.get (1, 1) * z1) * (1.0 + g.get (2, 2) / z2) - n;
+  complex n = g (0, 1) * g (1, 0);
+  complex d = (1.0 + g (0, 0) * z1) * (1.0 + g (1, 1) / z2) - n;
   matrix s (2);
-  s.set (1, 1, ((1 - g.get (1, 1) * z1) * (1 + g.get (2, 2) / z2) + n) / d);
-  s.set (1, 2, -2.0 * g.get (1, 2) / d);
-  s.set (2, 1, +2.0 * g.get (2, 1) / d);
-  s.set (2, 2, ((g.get (1, 1) * z1 + 1) * (g.get (2, 2) / z2 - 1) - n) / d);
+  s.set (0, 0, ((1 - g (0, 0) * z1) * (1 + g (1, 1) / z2) + n) / d);
+  s.set (0, 1, -2.0 * g (0, 1) / d);
+  s.set (1, 0, +2.0 * g (1, 0) / d);
+  s.set (1, 1, ((g (0, 0) * z1 + 1) * (g (1, 1) / z2 - 1) - n) / d);
   return s;
 }
 
@@ -661,21 +659,19 @@ matrix cstocz (matrix cs, matrix z) {
 
 // The function swaps the given rows with each other.
 void matrix::exchangeRows (int r1, int r2) {
-  assert (r1 >= 1 && r2 >= 1 && r1 <= rows && r2 <= rows);
+  assert (r1 >= 0 && r2 >= 0 && r1 < rows && r2 < rows);
   complex * s = new complex[cols];
   int len = sizeof (complex) * cols;
-  r1--; r2--;
   memcpy (s, &data[r1 * cols], len);
   memcpy (&data[r1 * cols], &data[r2 * cols], len);
   memcpy (&data[r2 * cols], s, len);
-  delete s;
+  delete[] s;
 }
 
 // The function swaps the given columns with each other.
 void matrix::exchangeCols (int c1, int c2) {
-  assert (c1 >= 1 && c2 >= 1 && c1 <= cols && c2 <= cols);
+  assert (c1 >= 0 && c2 >= 0 && c1 < cols && c2 < cols);
   complex s;
-  c1--; c2--;
   for (int r = 0; r < rows * cols; r += cols) {
     s = data[r + c1];
     data[r + c1] = data[r + c2];
@@ -687,7 +683,7 @@ void matrix::exchangeCols (int c1, int c2) {
    Z, H, G and A to any other.  Also converts S<->(A, T, H, Y and Z)
    matrices. */
 matrix twoport (matrix m, char in, char out) {
-  assert (m.getRows () == 2 && m.getCols () == 2);
+  assert (m.getRows () >= 2 && m.getCols () >= 2);
   complex d;
   matrix res (2);
 
@@ -698,32 +694,32 @@ matrix twoport (matrix m, char in, char out) {
       res = m;
       break;
     case 'Z': // Y to Z
-      d = m.get (1, 1) * m.get (2, 2) - m.get (1, 2) * m.get (2, 1);
-      res.set (1, 1, m.get (2, 2) / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, m.get (1, 1) / d);
+      d = m (0, 0) * m (1, 1) - m (0, 1) * m (1, 0);
+      res.set (0, 0, m (1, 1) / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, m (0, 0) / d);
       break;
     case 'H': // Y to H
-      d = m.get (1, 1);
-      res.set (1, 1, 1.0 / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, m.get (2, 1) / d);
-      res.set (2, 2, m.get (2, 2) - m.get (1, 2) * m.get (2, 1) / d);
+      d = m (0, 0);
+      res.set (0, 0, 1.0 / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, m (1, 0) / d);
+      res.set (1, 1, m (1, 1) - m (0, 1) * m (1, 0) / d);
       break;
     case 'G': // Y to G
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (1, 2, m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, 1.0 / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (0, 1, m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, 1.0 / d);
       break;
     case 'A': // Y to A
-      d = m.get (2, 1);
-      res.set (1, 1, -m.get (2, 2) / d);
-      res.set (1, 2, -1.0 / d);
-      res.set (2, 1, m.get (1, 2) - m.get (2, 2) * m.get (1, 1) / d);
-      res.set (2, 2, -m.get (1, 1) / d);
+      d = m (1, 0);
+      res.set (0, 0, -m (1, 1) / d);
+      res.set (0, 1, -1.0 / d);
+      res.set (1, 0, m (0, 1) - m (1, 1) * m (0, 0) / d);
+      res.set (1, 1, -m (0, 0) / d);
       break;
     case 'S': // Y to S
       res = ytos (m);
@@ -733,35 +729,35 @@ matrix twoport (matrix m, char in, char out) {
   case 'Z':
     switch (out) {
     case 'Y': // Z to Y
-      d = m.get (1, 1) * m.get (2, 2) - m.get (1, 2) * m.get (2, 1);
-      res.set (1, 1, m.get (2, 2) / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, m.get (1, 1) / d);
+      d = m (0, 0) * m (1, 1) - m (0, 1) * m (1, 0);
+      res.set (0, 0, m (1, 1) / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, m (0, 0) / d);
       break;
     case 'Z': // Z to Z
       res = m;
       break;
     case 'H': // Z to H
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (1, 2, m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, 1.0 / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (0, 1, m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, 1.0 / d);
       break;
     case 'G': // Z to G
-      d = m.get (1, 1);
-      res.set (1, 1, 1.0 / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, m.get (2, 1) / d);
-      res.set (2, 2, m.get (2, 2) - m.get (1, 2) * m.get (2, 1) / d);
+      d = m (0, 0);
+      res.set (0, 0, 1.0 / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, m (1, 0) / d);
+      res.set (1, 1, m (1, 1) - m (0, 1) * m (1, 0) / d);
       break;
     case 'A': // Z to A
-      d = m.get (2, 1);
-      res.set (1, 1, m.get (1, 1) / d);
-      res.set (1, 2, m.get (1, 1) * m.get (2, 2) / d - m.get (1, 2));
-      res.set (2, 1, 1.0 / d);
-      res.set (2, 2, m.get (2, 2) / d);
+      d = m (1, 0);
+      res.set (0, 0, m (0, 0) / d);
+      res.set (0, 1, m (0, 0) * m (1, 1) / d - m (0, 1));
+      res.set (1, 0, 1.0 / d);
+      res.set (1, 1, m (1, 1) / d);
       break;
     case 'S': // Z to S
       res = ztos (m);
@@ -771,35 +767,35 @@ matrix twoport (matrix m, char in, char out) {
   case 'H':
     switch (out) {
     case 'Y': // H to Y
-      d = m.get (1, 1);
-      res.set (1, 1, 1.0 / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, m.get (2, 1) / d);
-      res.set (2, 2, m.get (2, 2) - m.get (1, 2) * m.get(2, 1) / d);
+      d = m (0, 0);
+      res.set (0, 0, 1.0 / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, m (1, 0) / d);
+      res.set (1, 1, m (1, 1) - m (0, 1) * m.get(2, 1) / d);
       break;
     case 'Z': // H to Z
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (1, 2, m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, 1.0 / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (0, 1, m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, 1.0 / d);
       break;
     case 'H': // H to H
       res = m;
       break;
     case 'G': // H to G
-      d = m.get (1, 1) * m.get (2, 2) - m.get (1, 2) * m.get (2, 1);
-      res.set (1, 1, m.get (2, 2) / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, m.get (1, 1) / d);
+      d = m (0, 0) * m (1, 1) - m (0, 1) * m (1, 0);
+      res.set (0, 0, m (1, 1) / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, m (0, 0) / d);
       break;
     case 'A': // H to A
-      d = m.get (2, 1);
-      res.set (1, 1, m.get (1, 2) - m.get (1, 1) * m.get (2, 2) / d);
-      res.set (1, 2, -m.get (1, 1) / d);
-      res.set (2, 1, -m.get (2, 2) / d);
-      res.set (2, 2, -1.0 / d);
+      d = m (1, 0);
+      res.set (0, 0, m (0, 1) - m (0, 0) * m (1, 1) / d);
+      res.set (0, 1, -m (0, 0) / d);
+      res.set (1, 0, -m (1, 1) / d);
+      res.set (1, 1, -1.0 / d);
       break;
     case 'S': // H to S
       res = htos (m);
@@ -809,35 +805,35 @@ matrix twoport (matrix m, char in, char out) {
   case 'G':
     switch (out) {
     case 'Y': // G to Y
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (1, 2, m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, 1.0 / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (0, 1, m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, 1.0 / d);
       break;
     case 'Z': // G to Z
-      d = m.get (1, 1);
-      res.set (1, 1, 1.0 / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, m.get (2, 1) / d);
-      res.set (2, 2, m.get (2, 2) - m.get (1, 2) * m.get (2, 1) / d);
+      d = m (0, 0);
+      res.set (0, 0, 1.0 / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, m (1, 0) / d);
+      res.set (1, 1, m (1, 1) - m (0, 1) * m (1, 0) / d);
       break;
     case 'H': // G to H
-      d = m.get (1, 1) * m.get (2, 2) - m.get (1, 2) * m.get (2, 1);
-      res.set (1, 1, m.get (2, 2) / d);
-      res.set (1, 2, -m.get (1, 2) / d);
-      res.set (2, 1, -m.get (2, 1) / d);
-      res.set (2, 2, m.get (1, 1) / d);
+      d = m (0, 0) * m (1, 1) - m (0, 1) * m (1, 0);
+      res.set (0, 0, m (1, 1) / d);
+      res.set (0, 1, -m (0, 1) / d);
+      res.set (1, 0, -m (1, 0) / d);
+      res.set (1, 1, m (0, 0) / d);
       break;
     case 'G': // G to G
       res = m;
       break;
     case 'A': // G to A
-      d = m.get (2, 1);
-      res.set (1, 1, 1.0 / d);
-      res.set (1, 2, m.get (2, 2) / d);
-      res.set (2, 1, m.get (1, 1) / d);
-      res.set (2, 2, m.get (1, 1) * m.get (2, 2) / d - m.get (1, 2));
+      d = m (1, 0);
+      res.set (0, 0, 1.0 / d);
+      res.set (0, 1, m (1, 1) / d);
+      res.set (1, 0, m (0, 0) / d);
+      res.set (1, 1, m (0, 0) * m (1, 1) / d - m (0, 1));
       break;
     case 'S': // G to S
       res = gtos (m);
@@ -847,32 +843,32 @@ matrix twoport (matrix m, char in, char out) {
   case 'A':
     switch (out) {
     case 'Y': // A to Y
-      d = m.get (1, 2);
-      res.set (1, 1, m.get (2, 2) / d);
-      res.set (1, 2, m.get (2, 1) - m.get (1, 1) * m.get (2, 2) / d);
-      res.set (2, 1, -1.0 / d);
-      res.set (2, 2, m.get (1, 1) / d);
+      d = m (0, 1);
+      res.set (0, 0, m (1, 1) / d);
+      res.set (0, 1, m (1, 0) - m (0, 0) * m (1, 1) / d);
+      res.set (1, 0, -1.0 / d);
+      res.set (1, 1, m (0, 0) / d);
       break;
     case 'Z': // A to Z
-      d = m.get (2, 1);
-      res.set (1, 1, m.get (1, 1) / d);
-      res.set (1, 2, m.get (1, 1) * m.get (2, 2) / d - m.get (1, 2));
-      res.set (2, 1, 1.0 / d);
-      res.set (2, 2, m.get (2, 2) / d);
+      d = m (1, 0);
+      res.set (0, 0, m (0, 0) / d);
+      res.set (0, 1, m (0, 0) * m (1, 1) / d - m (0, 1));
+      res.set (1, 0, 1.0 / d);
+      res.set (1, 1, m (1, 1) / d);
       break;
     case 'H': // A to H
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 2) / d);
-      res.set (1, 2, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (2, 1, -1.0 / d);
-      res.set (2, 2, m.get (2, 1) / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 1) / d);
+      res.set (0, 1, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (1, 0, -1.0 / d);
+      res.set (1, 1, m (1, 0) / d);
       break;
     case 'G': // A to G
-      d = m.get (1, 1);
-      res.set (1, 1, m.get (2, 1) / d);
-      res.set (1, 2, m.get (2, 1) * m.get (1, 2) / d - m.get (2, 2));
-      res.set (2, 1, 1.0 / d);
-      res.set (2, 2, m.get (1, 2) / d);
+      d = m (0, 0);
+      res.set (0, 0, m (1, 0) / d);
+      res.set (0, 1, m (1, 0) * m (0, 1) / d - m (1, 1));
+      res.set (1, 0, 1.0 / d);
+      res.set (1, 1, m (0, 1) / d);
       break;
     case 'A': // A to A
       res = m;
@@ -888,11 +884,11 @@ matrix twoport (matrix m, char in, char out) {
       res = m;
       break;
     case 'T': // S to T
-      d = m.get (2, 1);
-      res.set (1, 1, m.get (1, 2) - m.get (1, 1) * m.get (2, 2) / d);
-      res.set (1, 2, m.get (1, 1) / d);
-      res.set (2, 1, -m.get (2, 2) / d);
-      res.set (1, 2, 1.0 / d);
+      d = m (1, 0);
+      res.set (0, 0, m (0, 1) - m (0, 0) * m (1, 1) / d);
+      res.set (0, 1, m (0, 0) / d);
+      res.set (1, 0, -m (1, 1) / d);
+      res.set (0, 1, 1.0 / d);
       break;
     case 'A': // S to A
       res = stoa (m);
@@ -914,11 +910,11 @@ matrix twoport (matrix m, char in, char out) {
   case 'T':
     switch (out) {
     case 'S': // T to S
-      d = m.get (2, 2);
-      res.set (1, 1, m.get (1, 2) / d); 
-      res.set (1, 2, m.get (1, 1) - m.get (1, 2) * m.get (2, 1) / d);
-      res.set (2, 1, 1.0 / d);
-      res.set (1, 2, -m.get (2, 1) / d);
+      d = m (1, 1);
+      res.set (0, 0, m (0, 1) / d); 
+      res.set (0, 1, m (0, 0) - m (0, 1) * m (1, 0) / d);
+      res.set (1, 0, 1.0 / d);
+      res.set (0, 1, -m (1, 0) / d);
       break;
     case 'T': // T to T
       res = m;
@@ -928,3 +924,13 @@ matrix twoport (matrix m, char in, char out) {
   }
   return res;
 }
+
+/* The function returns the Rollet stability factor of the given
+   S-parameter matrix. */
+nr_double_t rollet (matrix m) {
+  assert (m.getRows () >= 2 && m.getCols () >= 2);
+  nr_double_t res;
+  res = (1 - norm (m (0, 0)) - norm (m (1, 1)) + norm (det (m))) /
+    2 / abs (m (0, 1) * m (1, 0));
+  return res;
+}
