@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: matvec.cpp,v 1.12 2004/11/29 19:03:37 raimi Exp $
+ * $Id: matvec.cpp,v 1.17 2005/06/02 18:17:50 raimi Exp $
  *
  */
 
@@ -85,6 +85,7 @@ matvec::matvec (const matvec & m) {
 
 // Destructor deletes a matvec object.
 matvec::~matvec () {
+  if (name) free (name);
   if (data) delete[] data;
 }
 
@@ -103,7 +104,7 @@ char * matvec::getName (void) {
    appropriate matrix indices. */
 void matvec::set (vector v, int r, int c) {
   assert (v.getSize () == size && 
-	  r >= 1 && r <= rows && c >= 1 && c <= cols);
+	  r >= 0 && r < rows && c >= 0 && c < cols);
   for (int i = 0; i < size; i++) data[i].set (r, c, v.get (i));
 }
 
@@ -111,7 +112,7 @@ void matvec::set (vector v, int r, int c) {
    indices.  If the matrix vector has a valid name 'A' the returned
    vector gets the name 'A[r,c]'. */
 vector matvec::get (int r, int c) {
-  assert (r >= 1 && r <= rows && c >= 1 && c <= cols);
+  assert (r >= 0 && r < rows && c >= 0 && c < cols);
   vector res;
   for (int i = 0; i < size; i++) res.add (data[i].get (r, c));
   if (name != NULL) {
@@ -124,7 +125,7 @@ vector matvec::get (int r, int c) {
    'n[r,c]' scheme indicating a matrix (vector) entry. */
 char * matvec::createMatrixString (char * n, int r, int c) {
   static char str[256]; // hopefully enough
-  sprintf (str, "%s[%d,%d]", n, r, c);
+  sprintf (str, "%s[%d,%d]", n, r + 1, c + 1);
   return str;
 }
 
@@ -133,7 +134,7 @@ char * matvec::createMatrixString (char * n, int r, int c) {
    different arguments. */
 char * matvec::createMatrixString (char n, int r, int c) {
   static char str[256]; // hopefully enough
-  sprintf (str, "%c[%d,%d]", n, r, c);
+  sprintf (str, "%c[%d,%d]", n, r + 1, c + 1);
   return str;
 }
 
@@ -146,9 +147,9 @@ char * matvec::isMatrixVector (char * n, int& r, int& c) {
   char * p; int len;
   if (n == NULL) return NULL;              // nothing todo here
   if ((p = strchr (n, '[')) != NULL) {     // find first '['
-    r = atoi (p + 1);                      // get first index
+    r = atoi (p + 1) - 1;                  // get first index
     if ((p = strchr (p, ',')) != NULL) {   // find the ','
-      c = atoi (p + 1);                    // get second index
+      c = atoi (p + 1) - 1;                // get second index
       if ((p = strchr (p, ']')) != NULL) { // find trailing ']'
 	if (p[1] == '\0') {                // identifier must end in ']'
 	  // parse actual identifier
@@ -444,9 +445,18 @@ matvec ytoz (matvec y) {
    forms Y, Z, H, G and A to any other.  Also converts S<->(A, T, H, Y
    and Z) matrix vectors. */
 matvec twoport (matvec m, char in, char out) {
-  assert (m.getCols () == 2 && m.getRows () == 2);
-  matvec res (m.getSize (), m.getCols (), m.getRows ());
+  assert (m.getCols () >= 2 && m.getRows () >= 2);
+  matvec res (m.getSize (), 2, 2);
   for (int i = 0; i < m.getSize (); i++)
     res.set (twoport (m.get (i), in, out), i);
   return res;  
 }
+
+/* The function returns the Rollet stability factor vector of the
+   given S-parameter matrix vector. */
+vector rollet (matvec m) {
+  assert (m.getCols () >= 2 && m.getRows () >= 2);
+  vector res (m.getSize ());
+  for (int i = 0; i < m.getSize (); i++) res.set (rollet (m.get (i)), i);
+  return res;
+}
