@@ -1,7 +1,7 @@
 /*
  * strlist.cpp - string list class implementation
  *
- * Copyright (C) 2003, 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: strlist.cpp,v 1.6 2004/11/29 19:03:39 raimi Exp $
+ * $Id: strlist.cpp,v 1.8 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -68,6 +68,16 @@ void strlist::add (char * str) {
   root = s;
 }
 
+// The function adds the given string list to the list.
+void strlist::add (strlist * lst) {
+  if (lst) for (int i = lst->length () - 1; i >= 0; i--) add (lst->get (i));
+}
+
+// The function apends the given string list to the list.
+void strlist::append (strlist * lst) {
+  if (lst) for (int i = 0; i < lst->length (); i++) append (lst->get (i));
+}
+
 // This function append a string to the list.
 void strlist::append (char * str) {
   struct strlist_t * s;
