@@ -1,7 +1,7 @@
 /*
  * strlist.h - string list class definitions
  *
- * Copyright (C) 2003 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2005 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: strlist.h,v 1.5 2004/11/29 19:03:39 raimi Exp $
+ * $Id: strlist.h,v 1.7 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -37,7 +37,9 @@ class strlist
   strlist (const strlist &);
   ~strlist ();
   void add (char *);
+  void add (strlist *);
   void append (char *);
+  void append (strlist *);
   struct strlist_t * getRoot (void) { return root; }
   int length (void);
   int contains (char *);
