@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: trsolver.cpp,v 1.33 2005/02/21 20:50:59 raimi Exp $
+ * $Id: trsolver.cpp,v 1.36 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -95,6 +95,7 @@ void trsolver::initSteps (void) {
 void trsolver::solve (void) {
   nr_double_t time;
   int error = 0, convError = 0;
+  char * solver = getPropertyString ("Solver");
   runs++;
   current = 0;
   fixpoint = 0;
@@ -110,6 +111,16 @@ void trsolver::solve (void) {
   solve_pre ();
   applyNodeset ();
 
+  // Choose a solver.
+  if (!strcmp (solver, "CroutLU"))
+    eqnAlgo = ALGO_LU_DECOMPOSITION;
+  else if (!strcmp (solver, "DoolittleLU"))
+    eqnAlgo = ALGO_LU_DECOMPOSITION_DOOLITTLE;
+  else if (!strcmp (solver, "HouseholderQR"))
+    eqnAlgo = ALGO_QR_DECOMPOSITION;
+  else if (!strcmp (solver, "HouseholderLQ"))
+    eqnAlgo = ALGO_QR_DECOMPOSITION_LS;
+
   // Run the DC solver once.
   try_running () {
     error = solve_nonlinear ();
@@ -306,7 +317,7 @@ void trsolver::predictBashford (void) {
   nr_double_t xn, dd, hn;
 
   // go through each solution
-  for (int r = 1; r <= N + M; r++) {
+  for (int r = 0; r < N + M; r++) {
     xn = predCoeff[0] * SOL(1)->get (r); // a0 coefficient
     for (int o = 1; o <= predOrder; o++) {
       hn = getState (dState, o);         // previous time-step
@@ -326,7 +337,7 @@ void trsolver::predictEuler (void) {
   int M = countVoltageSources ();
   nr_double_t xn, dd, hn;
 
-  for (int r = 1; r <= N + M; r++) {
+  for (int r = 0; r < N + M; r++) {
     xn = predCoeff[0] * SOL(1)->get (r);
     hn = getState (dState, 1);
     dd = (SOL(1)->get (r) - SOL(2)->get (r)) / hn;
@@ -343,7 +354,7 @@ void trsolver::predictGear (void) {
   nr_double_t xn;
 
   // go through each solution
-  for (int r = 1; r <= N + M; r++) {
+  for (int r = 0; r < N + M; r++) {
     xn = 0;
     for (int o = 0; o <= predOrder; o++) {
       // a0, a1, ... coefficients
@@ -560,10 +571,10 @@ nr_double_t trsolver::checkDelta (void) {
   nr_double_t pec = getPredictorError (predType, predOrder);
 
   // go through each solution
-  for (int r = 1; r <= N + M; r++) {
+  for (int r = 0; r < N + M; r++) {
     
     // skip real voltage sources
-    if (r > N) {
+    if (r >= N) {
       if (findVoltageSource (r - N)->isVSource ())
 	continue;
     }
