@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: tvector.cpp,v 1.7 2005/02/14 19:56:44 raimi Exp $
+ * $Id: tvector.cpp,v 1.12 2005/06/27 14:18:08 raimi Exp $
  *
  */
 
@@ -46,6 +46,7 @@
 // Constructor creates an unnamed instance of the tvector class.
 template <class nr_type_t>
 tvector<nr_type_t>::tvector () {
+  external = 0;
   size = 0;
   data = NULL;
 }
@@ -54,6 +55,7 @@ tvector<nr_type_t>::tvector () {
    certain length. */
 template <class nr_type_t>
 tvector<nr_type_t>::tvector (int s)  {
+  external = 0;
   size = s;
   if (s > 0) {
     data = new nr_type_t[s];
@@ -66,6 +68,7 @@ tvector<nr_type_t>::tvector (int s)  {
    tvector object. */
 template <class nr_type_t>
 tvector<nr_type_t>::tvector (const tvector & v) {
+  external = 0;
   size = v.size;
   data = NULL;
 
@@ -83,7 +86,8 @@ const tvector<nr_type_t>&
 tvector<nr_type_t>::operator=(const tvector<nr_type_t> & v) {
   if (&v != this) {
     size = v.size;
-    if (data) { delete[] data; data = NULL; }
+    if (data && !external) { delete[] data; data = NULL; }
+    external = 0;
     if (size > 0) {
       data = new nr_type_t[size];
       memcpy (data, v.data, sizeof (nr_type_t) * size);
@@ -95,19 +99,19 @@ tvector<nr_type_t>::operator=(const tvector<nr_type_t> & v) {
 // Destructor deletes a tvector object.
 template <class nr_type_t>
 tvector<nr_type_t>::~tvector () {
-  if (data) delete[] data;
+  if (data && !external) delete[] data;
 }
 
 // Returns the tvector element at the given position.
 template <class nr_type_t>
 nr_type_t tvector<nr_type_t>::get (int i) {
-  return data[i - 1];
+  return data[i];
 }
 
 // Sets the tvector element at the given position.
 template <class nr_type_t>
 void tvector<nr_type_t>::set (int i, nr_type_t z) {
-  data[i - 1] = z;
+  data[i] = z;
 }
 
 // Sets all the tvector elements to the given value.
@@ -119,20 +123,28 @@ void tvector<nr_type_t>::set (nr_type_t z) {
 // Sets the specified tvector elements to the given value.
 template <class nr_type_t>
 void tvector<nr_type_t>::set (nr_type_t z, int start, int stop) {
-  for (int i = start; i <= stop; i++) data[i - 1] = z;
+  for (int i = start; i < stop; i++) data[i] = z;
 }
 
 // Copies the specified elements from the given tvector.
 template <class nr_type_t>
 void tvector<nr_type_t>::set (tvector<nr_type_t> a, int start, int stop) {
-  for (int i = start; i <= stop; i++) data[i - 1] = a.get (i);
+  for (int i = start; i < stop; i++) data[i] = a.get (i);
+}
+
+// Applies external data vector to the vector.
+template <class nr_type_t>
+void tvector<nr_type_t>::setData (nr_type_t * d, int len) {
+  if (data && !external) delete[] data;
+  external = 1;
+  data = d;
+  size = len;
 }
 
 // The function swaps the given rows with each other.
 template <class nr_type_t>
 void tvector<nr_type_t>::exchangeRows (int r1, int r2) {
-  assert (r1 >= 1 && r2 >= 1 && r1 <= size && r2 <= size);
-  r1--; r2--;
+  assert (r1 >= 0 && r2 >= 0 && r1 < size && r2 < size);
   nr_type_t s = data[r1];
   data[r1] = data[r2];
   data[r2] = s;
@@ -144,7 +156,7 @@ tvector<nr_type_t> operator + (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, a.get (i) + b.get (i));
+  for (int i = 0; i < n; i++) res.set (i, a.get (i) + b.get (i));
   return res;
 }
 
@@ -154,7 +166,7 @@ tvector<nr_type_t> operator - (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, a.get (i) - b.get (i));
+  for (int i = 0; i < n; i++) res.set (i, a.get (i) - b.get (i));
   return res;
 }
 
@@ -163,7 +175,7 @@ template <class nr_type_t>
 tvector<nr_type_t> operator * (nr_double_t s, tvector<nr_type_t> a) {
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, s * a.get (i));
+  for (int i = 0; i < n; i++) res.set (i, s * a.get (i));
   return res;
 }
 
@@ -178,7 +190,7 @@ tvector<nr_type_t> operator * (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, a.get (i) * b.get (i));
+  for (int i = 0; i < n; i++) res.set (i, a.get (i) * b.get (i));
   return res;
 }
 
@@ -187,7 +199,7 @@ template <class nr_type_t>
 nr_type_t scalar (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   nr_type_t n = 0;
-  for (int i = 1; i <= a.getSize (); i++) n += a.get (i) * b.get (i);
+  for (int i = 0; i < a.getSize (); i++) n += a.get (i) * b.get (i);
   return n;
 }
 
@@ -211,7 +223,7 @@ template <class nr_type_t>
 tvector<nr_type_t> operator - (tvector<nr_type_t> a) {
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, -a.get (i));
+  for (int i = 0; i < n; i++) res.set (i, -a.get (i));
   return res;
 }
 
@@ -220,7 +232,7 @@ template <class nr_type_t>
 bool operator < (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   int n = a.getSize ();
-  for (int i = 1; i <= n; i++) if (a.get (i) >= b.get (i)) return false;
+  for (int i = 0; i < n; i++) if (a.get (i) >= b.get (i)) return false;
   return true;
 }
 
@@ -229,7 +241,7 @@ template <class nr_type_t>
 bool operator > (tvector<nr_type_t> a, tvector<nr_type_t> b) {
   assert (a.getSize () == b.getSize ());
   int n = a.getSize ();
-  for (int i = 1; i <= n; i++) if (a.get (i) <= b.get (i)) return false;
+  for (int i = 0; i < n; i++) if (a.get (i) <= b.get (i)) return false;
   return true;
 }
 
@@ -238,7 +250,7 @@ template <class nr_type_t>
 tvector<nr_type_t> operator + (nr_type_t s, tvector<nr_type_t> a) {
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, s + a.get (i));
+  for (int i = 0; i < n; i++) res.set (i, s + a.get (i));
   return res;
 }
 
@@ -250,16 +262,47 @@ tvector<nr_type_t> operator + (tvector<nr_type_t> a, nr_type_t s) {
 // Mean square norm.
 template <class nr_type_t>
 nr_double_t norm (tvector<nr_type_t> a) {
-  nr_double_t n = 0;
-  for (int i = 1; i <= a.getSize (); i++) n += norm (a.get (i));
+#if 0
+  nr_double_t k = 0;
+  for (int i = 0; i < a.getSize (); i++) k += norm (a.get (i));
   return n;
+#else
+  nr_double_t scale = 0, n = 1, x, ax;
+  for (int i = 0; i < a.getSize (); i++) {
+    if ((x = real (a (i))) != 0) {
+      ax = fabs (x);
+      if (scale < ax) {
+	x = scale / ax;
+	n = 1 + n * x * x;
+	scale = ax;
+      }
+      else {
+	x = ax / scale;
+	n += x * x;
+      }
+    }
+    if ((x = imag (a (i))) != 0) {
+      ax = fabs (x);
+      if (scale < ax) {
+	x = scale / ax;
+	n = 1 + n * x * x;
+	scale = ax;
+      }
+      else {
+	x = ax / scale;
+	n += x * x;
+      }
+    }
+  }
+  return scale * scale * n;
+#endif
 }
 
 // Maximum norm.
 template <class nr_type_t>
 nr_double_t maxnorm (tvector<nr_type_t> a) {
   nr_double_t nMax = 0, n;
-  for (int i = 1; i <= a.getSize (); i++) {
+  for (int i = 0; i < a.getSize (); i++) {
     n = norm (a.get (i));
     if (n > nMax) nMax = n;
   }
@@ -271,7 +314,7 @@ template <class nr_type_t>
 tvector<nr_type_t> conj (tvector<nr_type_t> a) {
   int n = a.getSize ();
   tvector<nr_type_t> res (n);
-  for (int i = 1; i <= n; i++) res.set (i, conj (a.get (i)));
+  for (int i = 0; i < n; i++) res.set (i, conj (a.get (i)));
   return res;
 }
 
@@ -294,7 +337,7 @@ void tvector<nr_type_t>::reorder (int * idx) {
 // Debug function: Prints the vector object.
 template <class nr_type_t>
 void tvector<nr_type_t>::print (void) {
-  for (int r = 1; r <= size; r++) {
+  for (int r = 0; r < size; r++) {
     fprintf (stderr, "%+.2e\n", (double) real (get (r)));
   }
 }
