@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: ucs.cpp,v 1.17 2005/02/08 23:08:36 raimi Exp $
+ * $Id: ucs.cpp,v 1.20 2005/06/02 18:17:51 raimi Exp $
  *
  */
 
@@ -120,7 +120,7 @@ int main (int argc, char ** argv) {
 
   // attach a ground to the netlist
   gnd = new ground ();
-  gnd->setNode (1, "gnd");
+  gnd->setNode (0, "gnd");
   gnd->setName ("GND");
   subnet->insertCircuit (gnd);
 
@@ -137,5 +137,6 @@ int main (int argc, char ** argv) {
   delete subnet;
   delete in;
   delete out;
+  delete root;
   return 0;
 }
