@@ -1,7 +1,7 @@
 /*
  * vector.cpp - vector class implementation
  *
- * Copyright (C) 2003, 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
+ * Boston, MA 02110-1301, USA.  
  *
- * $Id: vector.cpp,v 1.16 2004/10/13 14:43:17 ela Exp $
+ * $Id: vector.cpp,v 1.21 2005/06/07 07:49:06 raimi Exp $
  *
  */
 
@@ -153,7 +153,7 @@ void vector::set (nr_double_t d, int i) {
 }
 
 void vector::set (const complex z, int i) {
-  data[i] = z;
+  data[i] = complex (z);
 }
 
 // The function returns the current size of the vector.
@@ -198,6 +198,24 @@ nr_double_t vector::minimum (void) {
   return min_D;
 }
 
+/* Unwraps a phase vector in radians.  Adds +/- 2*Pi if consecutive
+   values jump about |Pi|. */
+vector unwrap (vector v, nr_double_t tol) {
+  vector result (v.getSize ());
+  nr_double_t add = 0;
+  result (0) = v (0);
+  for (int i = 1; i < v.getSize (); i++) {
+    nr_double_t diff = real (v (i) - v (i-1));
+    if (diff > +tol) {
+      add -= 2 * M_PI;
+    } else if (diff < -tol) {
+      add += 2 * M_PI;
+    }
+    result (i) = v (i) + add;
+  }
+  return result;
+}
+
 complex sum (vector v) {
   complex result (0.0);
   for (int i = 0; i < v.getSize (); i++) result += v.get (i);
@@ -484,7 +502,7 @@ vector diff (vector var, vector dep, int n) {
 }
 
 vector vector::operator=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] = c;
+  for (int i = 0; i < getSize (); i++) data[i] = complex (c);
   return *this;
 }
 
@@ -735,3 +753,73 @@ void vector::setOrigin (char * n) {
 char * vector::getOrigin (void) {
   return origin;
 }
+
+/* The function returns the number of entries with the given value
+   deviating no more than the given epsilon. */
+int vector::contains (complex val, nr_double_t eps) {
+  int count = 0;
+  for (int i = 0; i < size; i++) {
+    if (abs (data[i] - val) <= eps) count++;
+  }
+  return count;
+}
+
+// Sorts the vector either in ascending or descending order.
+void vector::sort (bool ascending) {
+  complex t;
+  for (int i = 0; i < size; i++) {
+    for (int n = 0; n < size - 1; n++) {
+      if (ascending ? data[n] > data[n+1] : data[n] < data[n+1]) {
+	t = data[n];
+	data[n] = data[n+1];
+	data[n+1] = t;
+      }
+    }
+  }
+}
+
+/* The function creates a linear stepped vector of values starting at
+   the given start value, ending with the given stop value and
+   containing points elements. */
+vector linspace (nr_double_t start, nr_double_t stop, int points) {
+  vector result (points);
+  nr_double_t val, step = (stop - start) / (points - 1);
+  for (int i = 0; i < points; i++) {
+    val = start + (i * step);
+    if (i != 0 && fabs (val) < fabs (step) / 2 &&
+	fabs (val - (start + (i - 1) * step)) < fabs (step))
+      val = 0.0;
+    result.set (val, i);
+  }
+  return result;
+}
+
+/* The function creates a logarithmic stepped vector of values
+   starting at the given start value, ending with the given stop value
+   and containing points elements. */
+vector logspace (nr_double_t start, nr_double_t stop, int points) {
+  assert (start * stop > 0);
+  vector result (points);
+  nr_double_t step, first, last, d;
+
+  // ensure the last value being larger than the first
+  if (fabs (start) > fabs (stop)) {
+    first = fabs (stop);
+    last = fabs (start);
+  }
+  else {
+    first = fabs (start);
+    last = fabs (stop);
+  }
+  // check direction and sign of values
+  d = fabs (start) > fabs (stop) ? -1 : 1;
+  // compute logarithmic step size
+  step = (log (last) - log (first)) / (points - 1);
+  for (int i = 0, j = points - 1; i < points; i++, j--) {
+    if (d > 0)
+      result.set (start * exp (step * i), i);
+    else
+      result.set (stop * exp (step * i), j);
+  }
+  return result;
+}
