@@ -1,6 +1,6 @@
 /***************************************************************************
-                          component.cpp  -  description
-                             -------------------
+                               component.cpp
+                              ---------------
     begin                : Sat Aug 23 2003
     copyright            : (C) 2003 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -17,6 +17,8 @@
 
 #include "components.h"
 #include "main.h"
+#include "qucs.h"
+#include "qucsview.h"
 #include "node.h"
 #include "viewpainter.h"
 
@@ -26,12 +28,8 @@
 #include <qpen.h>
 #include <qmessagebox.h>
 #include <qdir.h>
-#include <qfileinfo.h>
-#include <qregexp.h>
 
 
-extern QDir QucsWorkDir;
-
 // ***********************************************************************
 // **********                                                   **********
 // **********                  class "Component"                **********
@@ -95,7 +93,7 @@ void Component::TextSize(int& _dx, int& _dy)
 
 // -------------------------------------------------------
 // Boundings including the component text.
-void Component::entireBounds(int& _x1, int& _y1, int& _x2, int& _y2)
+void Component::entireBounds(int& _x1, int& _y1, int& _x2, int& _y2, float Corr)
 {
   _x1 = x1+cx;
   _y1 = y1+cy;
@@ -111,14 +109,16 @@ void Component::entireBounds(int& _x1, int& _y1, int& _x2, int& _y2)
   if((tx+r.width()) > x2) _x2 = tx+r.width()+cx;
   if((ty+r.height()) > y2) _y2 = ty+r.height()+cy;
 
-  int dy=r.height(); // due to 'Name' text
+  int dy=1; // due to 'Name' text
   for(Property *pp = Props.first(); pp != 0; pp = Props.next())
     if(pp->display) {
       // get width of text
       r = metrics.size(0, pp->Name+"="+pp->Value);
       if((tx+r.width()) > x2) _x2 = tx+r.width()+cx;
-      dy += r.height();
+      dy++;
     }
+    
+  dy = int(float(dy) / Corr);  // correction for unproportional font scaling
   if((ty+dy) > y2) _y2 = ty+dy+cy;
 }
 
@@ -136,10 +136,42 @@ void Component::getCenter(int& x, int& y)
   y = cy;
 }
 
+// -------------------------------------------------------
+int Component::getTextSelected(int x_, int y_, float Corr)
+{
+  x_ -= cx;
+  y_ -= cy;
+  if(x_ < tx) return -1;
+  if(y_ < ty) return -1;
+
+  x_ -= tx;
+  y_ -= ty;
+  QFontMetrics  metrics(QucsSettings.font);
+  int w  = metrics.width(Name);
+  int dy = int(float(y_) * Corr);  // correction for font scaling
+  if(dy < 1) {
+    if(x_ < w) return 0;
+    return -1;
+  }
+
+  Property *pp;
+  for(pp = Props.first(); pp != 0; pp = Props.next())
+    if(pp->display)
+      if((--dy) < 1) break;
+  if(!pp) return -1;
+
+  // get width of text
+  w = metrics.width(pp->Name+"="+pp->Value);
+  if(x_ > w) return -1;
+  return Props.at()+1;  // number the property
+}
+
 // -------------------------------------------------------
 bool Component::getSelected(int x_, int y_)
 {
-  if(x_ >= x1+cx) if(x_ <= x2+cx) if(y_ >= y1+cy) if(y_ <= y2+cy)
+  x_ -= cx;
+  y_ -= cy;
+  if(x_ >= x1) if(x_ <= x2) if(y_ >= y1) if(y_ <= y2)
     return true;
 
   return false;
@@ -149,7 +181,7 @@ bool Component::getSelected(int x_, int y_)
 void Component::paint(ViewPainter *p)
 {
   Text *pt;
-  int x, y, a, b, xb, yb; //, Size;
+  int x, y, a, b, xb, yb;
   QFont f = p->Painter->font();   // save current font
   QFont newFont = f;
 
@@ -213,14 +245,21 @@ void Component::paint(ViewPainter *p)
     p->Painter->setBrush(QBrush::NoBrush);
 
     newFont.setWeight(QFont::Light);
-    p->Painter->setPen(QPen(QPen::black,1));
+    QWMatrix wm = p->Painter->worldMatrix();
     // write all text
     for(pt = Texts.first(); pt != 0; pt = Texts.next()) {
+      p->Painter->setWorldMatrix(
+          QWMatrix(pt->mCos, -pt->mSin, pt->mSin, pt->mCos,
+                   p->DX + float(cx+pt->x) * p->Scale,
+                   p->DY + float(cy+pt->y) * p->Scale));
       newFont.setPointSizeFloat(p->Scale * pt->Size);
       p->Painter->setFont(newFont);
       p->Painter->setPen(pt->Color);
-      p->drawText(pt->s, cx+pt->x, cy+pt->y);
+      p->Painter->drawText(0, 0, 0, 0, Qt::DontClip, pt->s);
+//      p->Painter->drawText(0, 0, pt->s);
     }
+    p->Painter->setWorldMatrix(wm);
+    p->Painter->setWorldXForm(false);
   }
   p->Painter->setFont(f);
 
@@ -254,8 +293,27 @@ void Component::paint(ViewPainter *p)
 void Component::paintScheme(QPainter *p)
 {
   if(Model.at(0) == '.') {   // is simulation component (dc, ac, ...)
-    int xb = x2-x1-13;
-    int yb = y2-y1-13;
+    Text *pt;
+    int a, b, xb, yb;
+    QFont newFont = p->font();
+
+    float Scale = QucsMain->view->Docs.current()->Scale;
+    newFont.setPointSizeFloat(float(Scale) * QucsSettings.largeFontSize);
+    newFont.setWeight(QFont::DemiBold);
+    QFontMetrics  metrics(newFont);
+
+    a = b = 0;
+    QSize r;
+    for(pt = Texts.first(); pt != 0; pt = Texts.next()) {
+      r = metrics.size(0, pt->s);
+      b += r.height();
+      if(a < r.width())  a = r.width();
+    }
+    xb = a + int(12.0*Scale);
+    yb = b + int(10.0*Scale);
+    x2 = x1+25 + int(float(a) / Scale);
+    y2 = y1+23 + int(float(b) / Scale);
+    if(ty < y2+1) if(ty > y1-r.height())  ty = y2 + 1;
 
     p->drawRect(cx-6, cy-5, xb, yb);
     p->drawLine(cx-1, cy+yb, cx-6, cy+yb-5);
@@ -368,15 +426,15 @@ void Component::rotate()
   }
 
   // rotate all text
-  QFont f = QucsSettings.font;
+  float ftmp;
   for(Text *pt = Texts.first(); pt != 0; pt = Texts.next()) {
-    f.setPointSizeFloat(pt->Size);
-    QFontMetrics  smallMetrics(f);
-    dx = smallMetrics.width(pt->s) >> 1;
-    dy = smallMetrics.lineSpacing() >> 1;
     tmp = -pt->x;
-    pt->x = pt->y + dy - dx;
-    pt->y = tmp - dx - dy;
+    pt->x = pt->y;
+    pt->y = tmp;
+
+    ftmp = -pt->mSin;
+    pt->mSin = pt->mCos;
+    pt->mCos = ftmp;
   }
 
   tmp = -x1;   // rotate boundings
@@ -440,21 +498,19 @@ void Component::mirrorX()
     pa->y = -pa->y - pa->h;
 
   QFont f = QucsSettings.font;
-  int dy;
   // mirror all text
   for(Text *pt = Texts.first(); pt != 0; pt = Texts.next()) {
     f.setPointSizeFloat(pt->Size);
     QFontMetrics  smallMetrics(f);
-    dy = smallMetrics.lineSpacing();   // height of text
-//    pt->y = -pt->y - (dy & 0xFFFFFFFE); // erase LSB to be rotate consistent
-    pt->y = -pt->y - dy;
+    QSize s = smallMetrics.size(0, pt->s);   // use size for more lines
+    pt->y = -pt->y - s.height();
   }
 
   int tmp = y1;
   y1  = -y2; y2 = -tmp;   // mirror boundings
 
   QFontMetrics  metrics(QucsSettings.font);   // get size of text
-  dy = metrics.lineSpacing();    // for "Name"
+  int dy = metrics.lineSpacing();    // for "Name"
   for(Property *pp = Props.first(); pp != 0; pp = Props.next())
     if(pp->display)  dy += metrics.lineSpacing();
   if((tx > x1) && (tx < x2)) ty = -ty-dy;     // mirror text position
@@ -503,9 +559,8 @@ void Component::mirrorY()
   for(Text *pt = Texts.first(); pt != 0; pt = Texts.next()) {
     f.setPointSizeFloat(pt->Size);
     QFontMetrics  smallMetrics(f);
-    tmp = smallMetrics.width(pt->s);   // width of text
-//    pt->x = -pt->x - (tmp & 0xFFFFFFFE); // erase LSB to be rotate consistent
-    pt->x = -pt->x - tmp;
+    QSize s = smallMetrics.size(0, pt->s);   // use size for more lines
+    pt->x = -pt->x - s.width();
   }
 
   tmp = x1;
@@ -531,12 +586,9 @@ void Component::mirrorY()
 // -------------------------------------------------------
 QString Component::NetList()
 {
-  if(Name.isEmpty()) return QString("");  // dummy elements (e.g. ground)
   if(!isActive) return QString("");       // should it be simulated ?
-  if(Model == "Port") return QString("");  // do not mention subcircuit ports
 
   QString s = Model+":"+Name;
-  if(s.at(0) == '#')  s.remove(0,1);
 
   // output all node names
   for(Port *p1 = Ports.first(); p1 != 0; p1 = Ports.next())
@@ -554,22 +606,7 @@ QString Component::NetList()
   // output all properties
   for(Property *p2 = Props.first(); p2 != 0; p2 = Props.next())
     if(p2->Name != "Symbol")
-      if(p2->Name == "File") {
-	if(Model == "Sub") {
-	  QString  Type = p2->Value;
-	  QFileInfo Info(Type);
-	  if(Info.extension() == "sch")  Type = Type.left(Type.length()-4);
-	  if(Type.at(0) <= '9') if(Type.at(0) >= '0') Type = '_' + Type;
-	  Type.replace(QRegExp("\\W"), "_"); // none [a-zA-Z0-9] into "_"
-	  s += " Type=\""+Type+"\"";   // type for subcircuit
-	  continue;
-	}
-        QFileInfo info(p2->Value);
-	if(info.isRelative())  info.setFile(QucsWorkDir, p2->Value);
-	s += " "+p2->Name+"=\"{"+info.absFilePath()+"}\"";    // properties
-      }
-      else
-	s += " "+p2->Name+"=\""+p2->Value+"\"";    // properties *****
+      s += " "+p2->Name+"=\""+p2->Value+"\"";
 
   return s;
 }
@@ -656,25 +693,24 @@ if(Model.at(0) != '.') {  // is simulation component (dc, ac, ...) ?
 
   tx = ttx; ty = tty; // restore text position (was changed by rotate/mirror)
 
-  int z=0;
-  unsigned int counts = s.contains('"') >> 1;
+  unsigned int z=0, counts = s.contains('"');
   // load all properties
   for(Property *p1 = Props.first(); p1 != 0; p1 = Props.next()) {
     z++;
     n = s.section('"',z,z);    // property value
+    z++;
     // not all properties have to be mentioned (backward compatible)
-    if(n.isEmpty()) {
+    if(z > counts) {
       if(p1->Description.isEmpty())
         Props.remove();    // remove if allocated in vain
       return true;
     }
 
-    z++;
     if(p1->Description.isEmpty()) {  // unknown number of properties ?
       p1->Name = n.section('=',0,0);
       n = n.section('=',1,1);
       // allocate memory for a new property (e.g. for equations)
-      if(Props.count() < counts) {
+      if(Props.count() < (counts>>1)) {
 	Props.insert(z >> 1, new Property("y", "1", true));
 	Props.prev();
       }
@@ -687,12 +723,277 @@ if(Model.at(0) != '.') {  // is simulation component (dc, ac, ...) ?
     if(!ok) return false;
   }
 
-  if(Model.at(0) != '.') {  // is simulation component (dc, ac, ...) ?
-  }
-  
   return true;
 }
 
+// *******************************************************************
+// ***  The following functions are used to load the schematic symbol
+// ***  from file. (e.g. subcircuit, library component)
+
+int Component::analyseLine(const QString& Row)
+{
+  QPen Pen;
+  QBrush Brush;
+  QColor Color;
+  QString s;
+  int i1, i2, i3, i4, i5, i6;
+
+  s = Row.section(' ',0,0);    // component type
+  if((s == "PortSym") || (s == ".PortSym")) {  // backward compatible
+    if(!getIntegers(Row, &i1, &i2, &i3))
+      return -1;
+    for(i6 = Ports.count(); i6<i3; i6++)  // if ports not in numerical order
+      Ports.append(new Port(0, 0));
+
+    Ports.at(i3-1)->x  = i1;
+    Ports.current()->y = i2;
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1 > x2)  x2 = i1;
+    if(i2 < y1)  y1 = i2;
+    if(i2 > y2)  y2 = i2;
+    return 0;   // do not count Ports
+  }
+  else if(s == "Line") {
+    if(!getIntegers(Row, &i1, &i2, &i3, &i4))  return -1;
+    if(!getPen(Row, Pen, 5))  return -1;
+    i3 += i1;
+    i4 += i2;
+    Lines.append(new Line(i1, i2, i3, i4, Pen));
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1 > x2)  x2 = i1;
+    if(i2 < y1)  y1 = i2;
+    if(i2 > y2)  y2 = i2;
+    if(i3 < x1)  x1 = i3;
+    if(i3 > x2)  x2 = i3;
+    if(i4 < y1)  y1 = i4;
+    if(i4 > y2)  y2 = i4;
+    return 1;
+  }
+  else if(s == "EArc") {
+    if(!getIntegers(Row, &i1, &i2, &i3, &i4, &i5, &i6))
+      return -1;
+    if(!getPen(Row, Pen, 7))  return -1;
+    Arcs.append(new struct Arc(i1, i2, i3, i4, i5, i6, Pen));
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1+i3 > x2)  x2 = i1+i3;
+    if(i2 < y1)  y1 = i2;
+    if(i2+i4 > y2)  y2 = i2+i4;
+    return 1;
+  }
+  else if(s == ".ID") {
+    if(!getIntegers(Row, &i1, &i2))  return -1;
+    tx = i1;
+    ty = i2;
+    Name = Row.section(' ',3,3);
+    if(Name.isEmpty())  Name = "SUB";
+    return 0;   // do not count IDs
+  }
+  else if(s == "Arrow") {
+    if(!getIntegers(Row, &i1, &i2, &i3, &i4, &i5, &i6))  return -1;
+    if(!getPen(Row, Pen, 7))  return -1;
+
+    double beta   = atan2(double(i6), double(i5));
+    double phi    = atan2(double(i4), double(i3));
+    double Length = sqrt(double(i6*i6 + i5*i5));
+
+    i3 += i1;
+    i4 += i2;
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1 > x2)  x2 = i1;
+    if(i3 < x1)  x1 = i3;
+    if(i3 > x2)  x2 = i3;
+    if(i2 < y1)  y1 = i2;
+    if(i2 > y2)  y2 = i2;
+    if(i4 < y1)  y1 = i4;
+    if(i4 > y2)  y2 = i4;
+
+    Lines.append(new Line(i1, i2, i3, i4, Pen));   // base line
+
+    double w = beta+phi;
+    i5 = i3-int(Length*cos(w));
+    i6 = i4-int(Length*sin(w));
+    Lines.append(new Line(i3, i4, i5, i6, Pen)); // arrow head
+    if(i5 < x1)  x1 = i5;  // keep track of component boundings
+    if(i5 > x2)  x2 = i5;
+    if(i6 < y1)  y1 = i6;
+    if(i6 > y2)  y2 = i6;
+
+    w = phi-beta;
+    i5 = i3-int(Length*cos(w));
+    i6 = i4-int(Length*sin(w));
+    Lines.append(new Line(i3, i4, i5, i6, Pen));
+    if(i5 < x1)  x1 = i5;  // keep track of component boundings
+    if(i5 > x2)  x2 = i5;
+    if(i6 < y1)  y1 = i6;
+    if(i6 > y2)  y2 = i6;
+
+    return 1;
+  }
+  else if(s == "Ellipse") {
+    if(!getIntegers(Row, &i1, &i2, &i3, &i4))  return -1;
+    if(!getPen(Row, Pen, 5))  return -1;
+    if(!getBrush(Row, Brush, 8))  return -1;
+    Ellips.append(new Area(i1, i2, i3, i4, Pen, Brush));
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1 > x2)  x2 = i1;
+    if(i2 < y1)  y1 = i2;
+    if(i2 > y2)  y2 = i2;
+    if(i1+i3 < x1)  x1 = i1+i3;
+    if(i1+i3 > x2)  x2 = i1+i3;
+    if(i2+i4 < y1)  y1 = i2+i4;
+    if(i2+i4 > y2)  y2 = i2+i4;
+    return 1;
+  }
+  else if(s == "Rectangle") {
+    if(!getIntegers(Row, &i1, &i2, &i3, &i4))  return -1;
+    if(!getPen(Row, Pen, 5))  return -1;
+    if(!getBrush(Row, Brush, 8))  return -1;
+    Rects.append(new Area(i1, i2, i3, i4, Pen, Brush));
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i1 > x2)  x2 = i1;
+    if(i2 < y1)  y1 = i2;
+    if(i2 > y2)  y2 = i2;
+    if(i1+i3 < x1)  x1 = i1+i3;
+    if(i1+i3 > x2)  x2 = i1+i3;
+    if(i2+i4 < y1)  y1 = i2+i4;
+    if(i2+i4 > y2)  y2 = i2+i4;
+    return 1;
+  }
+  else if(s == "Text") {  // must be last in order to reuse "s" *********
+    if(!getIntegers(Row, &i1, &i2, &i3))  return -1;
+    Color.setNamedColor(Row.section(' ',4,4));
+    if(!Color.isValid()) return -1;
+
+    s = Row.mid(Row.find('"')+1);    // Text (can contain " !!!)
+    s = s.left(s.length()-1);
+    if(s.isEmpty()) return -1;
+    convert2Unicode(s);
+
+    Texts.append(new Text(i1, i2, s, Color, float(i3)));
+
+    QFont Font(QucsSettings.font);
+    Font.setPointSizeFloat(float(i3));
+    QFontMetrics  metrics(Font);
+    QSize r = metrics.size(0, s);    // get size of text
+    i3 = i1 + r.width();
+    i4 = i2 + r.height();
+
+    if(i1 < x1)  x1 = i1;  // keep track of component boundings
+    if(i3 > x2)  x2 = i3;
+    if(i2 < y1)  y1 = i2;
+    if(i4 > y2)  y2 = i4;
+  }
+
+  return 0;
+}
+
+// ---------------------------------------------------------------------
+bool Component::getIntegers(const QString& s, int *i1, int *i2, int *i3,
+			     int *i4, int *i5, int *i6)
+{
+  bool ok;
+  QString n;
+
+  if(!i1) return true;
+  n  = s.section(' ',1,1);
+  *i1 = n.toInt(&ok);
+  if(!ok) return false;
+
+  if(!i2) return true;
+  n  = s.section(' ',2,2);
+  *i2 = n.toInt(&ok);
+  if(!ok) return false;
+
+  if(!i3) return true;
+  n  = s.section(' ',3,3);
+  *i3 = n.toInt(&ok);
+  if(!ok) return false;
+
+  if(!i4) return true;
+  n  = s.section(' ',4,4);
+  *i4 = n.toInt(&ok);
+  if(!ok) return false;
+
+  if(!i5) return true;
+  n  = s.section(' ',5,5);
+  *i5 = n.toInt(&ok);
+  if(!ok) return false;
+
+  if(!i6) return true;
+  n  = s.section(' ',6,6);
+  *i6 = n.toInt(&ok);
+  if(!ok) return false;
+
+  return true;
+}
+
+// ---------------------------------------------------------------------
+bool Component::getPen(const QString& s, QPen& Pen, int i)
+{
+  bool ok;
+  QString n;
+
+  n = s.section(' ',i,i);    // color
+  QColor co;
+  co.setNamedColor(n);
+  Pen.setColor(co);
+  if(!Pen.color().isValid()) return false;
+
+  i++;
+  n = s.section(' ',i,i);    // thickness
+  Pen.setWidth(n.toInt(&ok));
+  if(!ok) return false;
+
+  i++;
+  n = s.section(' ',i,i);    // line style
+  Pen.setStyle((Qt::PenStyle)n.toInt(&ok));
+  if(!ok) return false;
+
+  return true;
+}
+
+// ---------------------------------------------------------------------
+bool Component::getBrush(const QString& s, QBrush& Brush, int i)
+{
+  bool ok;
+  QString n;
+
+  n = s.section(' ',i,i);    // fill color
+  QColor co;
+  co.setNamedColor(n);
+  Brush.setColor(co);
+  if(!Brush.color().isValid()) return false;
+
+  i++;
+  n = s.section(' ',i,i);    // fill style
+  Brush.setStyle((Qt::BrushStyle)n.toInt(&ok));
+  if(!ok) return false;
+
+  i++;
+  n = s.section(' ',i,i);    // filled
+  if(n.toInt(&ok) == 0) Brush.setStyle(QBrush::NoBrush);
+  if(!ok) return false;
+
+  return true;
+}
+
+// ---------------------------------------------------------------------
+void Component::performModification()
+{
+  bool mmir = mirroredX;
+  int  rrot = rotated;
+  if(mmir)  mirrorX();   // mirror
+  for(int z=0; z<rrot; z++)  rotate(); // rotate
+
+
+  rotated = rrot;   // restore properties (were changed by rotate/mirror)
+  mirroredX = mmir;
+}
 
 
 // ***********************************************************************
@@ -729,8 +1030,13 @@ Component* getComponentFromName(QString& Line)
 	else if(cstr == "CVS") c = new CCVS();
 	else if(cstr == "irculator") c = new Circulator();
 	else if(cstr == "LIN") c = new Coplanar();
+	else if(cstr == "OPEN") c = new CPWopen();
+	else if(cstr == "SHORT") c = new CPWshort();
+	else if(cstr == "GAP") c = new CPWgap();
+	else if(cstr == "STEP") c = new CPWstep();
 	break;
   case 'L' : if(cstr.isEmpty()) c = new Inductor();
+	else if(cstr == "ib") c = new LibComp();
 	break;
   case 'G' : if(cstr == "ND") c = new Ground();
         else if(cstr == "yrator") c = new Gyrator();
@@ -766,7 +1072,7 @@ Component* getComponentFromName(QString& Line)
 	       c = new SParamFile(cstr.mid(5).toInt()); }
         else if(cstr == "ub")   c = new Subcircuit();
         else if(cstr == "UBST") c = new Substrate();
-        else if(cstr == "PICE") c = new SpiceFile(2);
+        else if(cstr == "PICE") c = new SpiceFile();
         break;
   case 'D' : if(cstr == "CBlock") c = new dcBlock();
 	else if(cstr == "CFeed") c = new dcFeed();
@@ -818,7 +1124,7 @@ Component* getComponentFromName(QString& Line)
     return 0;
   }
 
-  cstr = c->Name;   // is perhaps changed in "recreate" (z.B. subcircuit)
+  cstr = c->Name;   // is perhaps changed in "recreate" (e.g. subcircuit)
   int x = c->tx, y = c->ty;
   c->recreate();
   c->Name = cstr;
