@@ -44,6 +44,11 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
   QWidget *myParent = this;
   ValInteger = new QIntValidator(1, 1000000, this);
 
+  Expr.setPattern("[^\"=]+");  // valid expression for property 'edit' etc
+  Validator = new QRegExpValidator(Expr, this);
+  Expr.setPattern("[\\w_]+");  // valid expression for property 'NameEdit' etc
+  ValRestrict = new QRegExpValidator(Expr, this);
+
   checkSim  = 0;  editSim   = 0;  comboType = 0;  checkParam = 0;
   editStart = 0;  editStop  = 0;  editNumber = 0;
 
@@ -63,13 +68,15 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
 
     int row=1;
     editParam = new QLineEdit(Tab1);
+    editParam->setValidator(ValRestrict);
     connect(editParam, SIGNAL(returnPressed()), SLOT(slotParamEntered()));
     checkParam = new QCheckBox(tr("display in schematic"), Tab1);
     if(Comp->Model == ".SW") {   // parameter sweep
       textSim = new QLabel(tr("Simulation:"), Tab1);
       gp->addWidget(textSim, row,0);
-      editSim = new QLineEdit(Tab1);
-      connect(editSim, SIGNAL(returnPressed()), SLOT(slotSimEntered()));
+      editSim = new QComboBox(Tab1);
+      editSim->setEditable(true);
+      connect(editSim, SIGNAL(activated(int)), SLOT(slotSimEntered(int)));
       gp->addWidget(editSim, row,1);
       checkSim = new QCheckBox(tr("display in schematic"), Tab1);
       gp->addWidget(checkSim, row++,2);
@@ -106,6 +113,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     textValues = new QLabel(tr("Values:"), Tab1);
     gp->addWidget(textValues, row,0);
     editValues = new QLineEdit(Tab1);
+    editValues->setValidator(Validator);
     connect(editValues, SIGNAL(returnPressed()), SLOT(slotValuesEntered()));
     gp->addWidget(editValues, row,1);
     checkValues = new QCheckBox(tr("display in schematic"), Tab1);
@@ -114,6 +122,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     textStart  = new QLabel(tr("Start:"), Tab1);
     gp->addWidget(textStart, row,0);
     editStart  = new QLineEdit(Tab1);
+    editStart->setValidator(Validator);
     connect(editStart, SIGNAL(returnPressed()), SLOT(slotStartEntered()));
     gp->addWidget(editStart, row,1);
     checkStart = new QCheckBox(tr("display in schematic"), Tab1);
@@ -122,6 +131,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     textStop   = new QLabel(tr("Stop:"), Tab1);
     gp->addWidget(textStop, row,0);
     editStop   = new QLineEdit(Tab1);
+    editStop->setValidator(Validator);
     connect(editStop, SIGNAL(returnPressed()), SLOT(slotStopEntered()));
     gp->addWidget(editStop, row,1);
     checkStop = new QCheckBox(tr("display in schematic"), Tab1);
@@ -130,6 +140,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     textStep   = new QLabel(tr("Step:"), Tab1);
     gp->addWidget(textStep, row,0);
     editStep   = new QLineEdit(Tab1);
+    editStep->setValidator(Validator);
     connect(editStep, SIGNAL(returnPressed()), SLOT(slotStepEntered()));
     gp->addWidget(editStep, row++,1);
 
@@ -144,7 +155,12 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
 
 
     if(Comp->Model == ".SW") {   // parameter sweep
-      editSim->setText(Comp->Props.first()->Value);
+      for(Component *pc=Doc->Comps->first(); pc!=0; pc=Doc->Comps->next())
+        if(pc != Comp)
+          if(pc->Model[0] == '.')
+            editSim->insertItem(pc->Name);
+      editSim->setCurrentText(Comp->Props.first()->Value);
+
       checkSim->setChecked(Comp->Props.current()->display);
       s = Comp->Props.next()->Value;
       checkType->setChecked(Comp->Props.current()->display);
@@ -185,8 +201,8 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     }
     slotNumberChanged(0);
 
-    connect(editValues, SIGNAL(textChanged(const QString&)),
-	    SLOT(slotTextChanged(const QString&)));
+/*    connect(editValues, SIGNAL(textChanged(const QString&)),
+	    SLOT(slotTextChanged(const QString&)));*/
     connect(editStart, SIGNAL(textChanged(const QString&)),
 	    SLOT(slotNumberChanged(const QString&)));
     connect(editStop, SIGNAL(textChanged(const QString&)),
@@ -196,13 +212,13 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
     connect(editNumber, SIGNAL(textChanged(const QString&)),
 	    SLOT(slotNumberChanged(const QString&)));
 
-    if(checkSim)
+/*    if(checkSim)
       connect(checkSim, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
     connect(checkType, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
     connect(checkParam, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
     connect(checkStart, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
     connect(checkStop, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
-    connect(checkNumber, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));
+    connect(checkNumber, SIGNAL(stateChanged(int)), SLOT(slotSetChanged(int)));*/
 
 
     QWidget *Tab2 = new QWidget(t);
@@ -224,6 +240,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
   gp2->addWidget(h5, 1,0);
   new QLabel(tr("Name:"), h5);
   CompNameEdit = new QLineEdit(h5);
+  CompNameEdit->setValidator(ValRestrict);
   connect(CompNameEdit, SIGNAL(returnPressed()), SLOT(slotButtOK()));
 
   prop = new QListView(myParent);
@@ -238,11 +255,6 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
   Name = new QLabel(myParent);
   gp2->addWidget(Name, 2,1);
 
-  Expr.setPattern("[^\"=]+");  // valid expression for property input 'edit'
-  Validator = new QRegExpValidator(Expr, this);
-  Expr.setPattern("[\\w_]+");  // valid expression for property input 'edit'
-  ValRestrict = new QRegExpValidator(Expr, this);
-
   Description = new QLabel(myParent);
   gp2->addWidget(Description, 3,1);
 
@@ -309,7 +321,7 @@ ComponentDialog::ComponentDialog(Component *c, QucsDoc *d, QWidget *parent)
 
   // ------------------------------------------------------------
   CompNameEdit->setText(Comp->Name);
-  changed = transfered = false;
+  changed = false;
 
   Comp->TextSize(tx_Dist, ty_Dist);
   int tmp = Comp->tx+tx_Dist - Comp->x1;
@@ -453,7 +465,6 @@ void ComponentDialog::slotApplyChange(const QString& Text)
 {
   edit->setText(Text);
   prop->currentItem()->setText(1, Text);	// apply edit line
-  changed = true;
 
   ComboEdit->setFocus();
   QListViewItem *item = prop->currentItem()->itemBelow();
@@ -468,21 +479,20 @@ void ComponentDialog::slotApplyChange(const QString& Text)
 void ComponentDialog::slotApplyProperty()
 {
   QListViewItem *item = prop->currentItem();
+  if(!item) return;
 
   if(ComboEdit->isShown())   // take text from ComboBox ?
     edit->setText(ComboEdit->currentText());
 
-  if(item->text(1) != edit->text()) {
+  if(item->text(1) != edit->text())
     item->setText(1, edit->text());    // apply edit line
-    changed = true;
-  }
+
   if(NameEdit->isShown())	// also apply property name ?
     if(item->text(0) != NameEdit->text()) {
 //      if(NameEdit->text() == "Export")
 //        item->setText(0, "Export_");   // name must not be "Export" !!!
 //      else
       item->setText(0, NameEdit->text());  // apply property name
-      changed = true;
     }
 
   item = item->itemBelow();
@@ -508,7 +518,6 @@ void ComponentDialog::slotApplyPropName()
 //    }
 //      else
     item->setText(0, NameEdit->text());  // apply property name
-    changed = true;
   }
   edit->setFocus();   // cursor into "edit" widget
 }
@@ -526,7 +535,6 @@ void ComponentDialog::slotApplyState(int State)
 
   if(item->text(2) != ButtonState) {
     item->setText(2, ButtonState);
-    changed = true;
   }
 }
 
@@ -535,16 +543,22 @@ void ComponentDialog::slotApplyState(int State)
 void ComponentDialog::slotButtOK()
 {
   slotApplyInput();
-  if(changed) done(1);
-  else done(0);
+  slotButtCancel();
 }
 
 // -------------------------------------------------------------------------
 // Is called if the "Cancel"-button is pressed.
 void ComponentDialog::slotButtCancel()
 {
-  if(transfered) done(1);	// changed could have been done before
-  else done(0);			// (by "Apply"-button)
+  if(changed) done(1);	// changed could have been done before
+  else done(0);		// (by "Apply"-button)
+}
+
+//-----------------------------------------------------------------
+// To get really all close events (even <Escape> key).
+void ComponentDialog::reject()
+{
+  slotButtCancel();
 }
 
 // -------------------------------------------------------------------------
@@ -566,53 +580,111 @@ void ComponentDialog::slotApplyInput()
     }
   }
 
-
   bool display;
-  Comp->Props.clear();
-  // apply all the new property values, i.e. rebuild the properties
+  Property *pp = Comp->Props.first();
+  // apply all the new property values
   if(editSim) {
     display = checkSim->isChecked();
-    Comp->Props.append(new
-	Property("Sim", editSim->text(), display, QString("--")));
+    if(pp->display != display) {
+      pp->display = display;
+      changed = true;
+    }
+    if(pp->Value != editSim->currentText()) {
+      pp->Value = editSim->currentText();
+      changed = true;
+    }
+    pp = Comp->Props.next();
   }
   if(comboType) {
     display = checkType->isChecked();
+    if(pp->display != display) {
+      pp->display = display;
+      changed = true;
+    }
     switch(comboType->currentItem()) {
       case 1:  tmp = "log";   break;
       case 2:  tmp = "list";  break;
       case 3:  tmp = "const"; break;
       default: tmp = "lin";   break;
     }
-    Comp->Props.append(new
-	Property("Type", tmp, display, QString("--")));
+    if(pp->Value != tmp) {
+      pp->Value = tmp;
+      changed = true;
+    }
+    pp = Comp->Props.next();
   }
   if(checkParam) if(checkParam->isEnabled()) {
     display = checkParam->isChecked();
-    Comp->Props.append(new
-	Property("Param", editParam->text(), display, QString("--")));
+    if(pp->display != display) {
+      pp->display = display;
+      changed = true;
+    }
+    if(pp->Value != editParam->text()) {
+      pp->Value = editParam->text();
+      changed = true;
+    }
+    pp = Comp->Props.next();
   }
   if(editStart) {
     if(comboType->currentItem() < 2) {
       display = checkStart->isChecked();
-      Comp->Props.append(new
-	Property("Start", editStart->text(), display, QString("--")));
+      if(pp->display != display) {
+        pp->display = display;
+        changed = true;
+      }
+      pp->Name  = "Start";
+      if(pp->Value != editStart->text()) {
+        pp->Value = editStart->text();
+        changed = true;
+      }
+      pp = Comp->Props.next();
 
       display = checkStop->isChecked();
-      Comp->Props.append(new
-	Property("Stop", editStop->text(), display, QString("--")));
+      if(pp->display != display) {
+        pp->display = display;
+        changed = true;
+      }
+      pp->Name  = "Stop";
+      if(pp->Value != editStop->text()) {
+        pp->Value = editStop->text();
+        changed = true;
+      }
+      pp = Comp->Props.next();
 
       display = checkNumber->isChecked();
-      Comp->Props.append(new
-	Property("Points", editNumber->text(), display, QString("--")));
+      if(pp->display != display) {
+        pp->display = display;
+        changed = true;
+      }
+      if((pp->Value != editNumber->text()) || (pp->Name != "Points")) {
+        pp->Value = editNumber->text();
+        pp->Name  = "Points";
+        changed = true;
+      }
+      pp = Comp->Props.next();
     }
     else {
-      // Call them "Symbol" to omit them in the netlist.
-      Comp->Props.append(new Property("Symbol", "0", false, QString("--")));
-      Comp->Props.append(new Property("Symbol", "0", false, QString("--")));
+      // If a value list is used, the properties "Start" and "Stop" are not
+      // used. -> Call them "Symbol" to omit them in the netlist.
+      pp->Name = "Symbol";
+      pp->display = false;
+      pp = Comp->Props.next();
+      pp->Name = "Symbol";
+      pp->display = false;
+      pp = Comp->Props.next();
 
       display = checkValues->isChecked();
-      Comp->Props.append(new Property("Values", "["+editValues->text()+"]",
-			     display, QString("--")));
+      if(pp->display != display) {
+        pp->display = display;
+        changed = true;
+      }
+      tmp = "["+editValues->text()+"]";
+      if((pp->Value != tmp) || (pp->Name != "Values")) {
+        pp->Value = tmp;
+        pp->Name  = "Values";
+        changed = true;
+      }
+      pp = Comp->Props.next();
     }
   }
 
@@ -621,25 +693,46 @@ void ComponentDialog::slotApplyInput()
  if(item != 0) {
 
   item = prop->currentItem();
-  if(item->text(1) != edit->text()) {
+  if(item->text(1) != edit->text())
     item->setText(1, edit->text());    // apply edit line
-    changed = true;
-  }
   if(NameEdit->isShown())	// also apply property name ?
-    if(item->text(0) != NameEdit->text()) {
+    if(item->text(0) != NameEdit->text())
       item->setText(0, NameEdit->text());  // apply property name
-      changed = true;
-    }
 
 
   // apply all the new property values in the ListView
   for(item = prop->firstChild(); item != 0; item = item->itemBelow()) {
     display = (item->text(2) == tr("yes"));
-    Comp->Props.append(new
-	Property(item->text(0), item->text(1), display, item->text(3)));
+    if(pp) {
+      if(pp->display != display) {
+        pp->display = display;
+        changed = true;
+      }
+      if(pp->Value != item->text(1)) {
+        pp->Value = item->text(1);
+        changed = true;
+      }
+      if(pp->Name != item->text(0)) {
+        pp->Name = item->text(0);   // override if previous one was removed
+        changed = true;
+      }
+      pp->Description = item->text(3);
+    }
+    else {  // if less properties than in ListView -> create new
+      Comp->Props.append(new
+	  Property(item->text(0), item->text(1), display, item->text(3)));
+      changed = true;
+    }
+    pp = Comp->Props.next();
+  }
+  if(pp) {  // if more properties than in ListView -> delete the rest
+    pp = Comp->Props.prev();
+    Comp->Props.last();
+    while(pp != Comp->Props.current())
+      Comp->Props.remove();
+    changed = true;
   }
  }
-  transfered = true;     // applied changes to the component itself
 
   if(changed) {
     int dx, dy;
@@ -676,7 +769,6 @@ void ComponentDialog::slotBrowseFile()
     if(QucsWorkDir.exists(file.fileName()) &&
        QucsWorkDir.absPath() == file.dirPath(true)) s = file.fileName();
     edit->setText(s);
-    changed = true;
   }
   prop->currentItem()->setText(1, s);
 }
@@ -684,7 +776,7 @@ void ComponentDialog::slotBrowseFile()
 // -------------------------------------------------------------------------
 void ComponentDialog::slotEditFile()
 {
-  Doc->App->editFile(QucsWorkDir.filePath(edit->text()));
+  Doc->App->Acts.editFile(QucsWorkDir.filePath(edit->text()));
 }
 
 // -------------------------------------------------------------------------
@@ -700,7 +792,6 @@ void ComponentDialog::slotButtAdd()
 
   prop->setSelected(new QListViewItem(prop, item,
 			NameEdit->text(), edit->text(), s), true);
-  changed = true;
 }
 
 // -------------------------------------------------------------------------
@@ -716,7 +807,6 @@ void ComponentDialog::slotButtRem()
   if(next_item == 0) next_item = item->itemAbove();
   prop->takeItem(item);     // remove from ListView
   delete item;              // delete item
-  changed = true;
 
   slotSelectProperty(next_item);
 }
@@ -724,8 +814,6 @@ void ComponentDialog::slotButtRem()
 // -------------------------------------------------------------------------
 void ComponentDialog::slotSimTypeChange(int Type)
 {
-  changed = true;
-
   if(Type < 2) {  // new type is "linear" or "logarithmic"
     if(!editNumber->isEnabled()) {  // was the other mode before ?
       // this text change, did not emit the textChange signal !??!
@@ -786,8 +874,6 @@ void ComponentDialog::slotSimTypeChange(int Type)
 // Is called when "Start", "Stop" or "Number" is edited.
 void ComponentDialog::slotNumberChanged(const QString&)
 {
-  changed = true;
-
   QString Unit, tmp;
   double x, y, Factor, ftmp;
   if(comboType->currentItem() == 1) {   // logarithmic ?
@@ -825,8 +911,6 @@ void ComponentDialog::slotNumberChanged(const QString&)
 // -------------------------------------------------------------------------
 void ComponentDialog::slotStepChanged(const QString& Step)
 {
-  changed = true;
-
   QString Unit;
   double x, y, Factor;
   if(comboType->currentItem() == 1) {   // logarithmic ?
@@ -859,20 +943,6 @@ void ComponentDialog::slotStepChanged(const QString& Step)
   editNumber->blockSignals(false);
 }
 
-// -------------------------------------------------------------------------
-// Is called if one of the "display on schematic" CheckBoxes is clicked.
-void ComponentDialog::slotSetChanged(int)
-{
-  changed = true;
-}
-
-// -------------------------------------------------------------------------
-// Is called if text in the LineEdit (e.g. "editValues") is changed.
-void ComponentDialog::slotTextChanged(const QString&)
-{
-  changed = true;
-}
-
 // -------------------------------------------------------------------------
 // Is called if return is pressed in LineEdit "Parameter".
 void ComponentDialog::slotParamEntered()
@@ -885,7 +955,7 @@ void ComponentDialog::slotParamEntered()
 
 // -------------------------------------------------------------------------
 // Is called if return is pressed in LineEdit "Simulation".
-void ComponentDialog::slotSimEntered()
+void ComponentDialog::slotSimEntered(int)
 {
   editParam->setFocus();
 }
