@@ -1,6 +1,6 @@
 /***************************************************************************
-                          components.h  -  description
-                             -------------------
+                               components.h
+                              --------------
     begin                : Sat Aug 23 2003
     copyright            : (C) 2003 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -64,6 +64,10 @@
 #include "msgap.h"
 #include "msvia.h"
 #include "coplanar.h"
+#include "cpwopen.h"
+#include "cpwshort.h"
+#include "cpwgap.h"
+#include "cpwstep.h"
 #include "diode.h"
 #include "bjt.h"
 #include "bjtsub.h"
@@ -84,6 +88,7 @@
 #include "vrect.h"
 #include "irect.h"
 #include "spicefile.h"
+#include "libcomp.h"
 
 
 #endif
