@@ -26,7 +26,8 @@ public:
   Ground();
   ~Ground();
   Component* newOne();
-  static Component* info(QString&, char* &, bool getNewOne=false);
+  static Element* info(QString&, char* &, bool getNewOne=false);
+  QString NetList();
 };
 
 #endif
