@@ -26,9 +26,9 @@ public:
   MOSFET_sub();
   ~MOSFET_sub();
   Component* newOne();
-  static Component* info(QString&, char* &, bool getNewOne=false);
-  static Component* info_p(QString&, char* &, bool getNewOne=false);
-  static Component* info_depl(QString&, char* &, bool getNewOne=false);
+  static Element* info(QString&, char* &, bool getNewOne=false);
+  static Element* info_p(QString&, char* &, bool getNewOne=false);
+  static Element* info_depl(QString&, char* &, bool getNewOne=false);
   void recreate();
 };
 
