@@ -69,7 +69,7 @@ Component* SP_Sim::newOne()
   return new SP_Sim();
 }
 
-Component* SP_Sim::info(QString& Name, char* &BitmapFile, bool getNewOne)
+Element* SP_Sim::info(QString& Name, char* &BitmapFile, bool getNewOne)
 {
   Name = QObject::tr("S-parameter simulation");
   BitmapFile = "sparameter";
