@@ -26,7 +26,8 @@ public:
   SParamFile(int No);
   ~SParamFile();
   Component* newOne();
-  static Component* info(QString&, char* &, bool getNewOne=false);
+  QString NetList();
+  static Element* info(QString&, char* &, bool getNewOne=false);
 };
 
 #endif
