@@ -1,7 +1,7 @@
 /***************************************************************************
                                  spicefile.h
-                               ---------------
-    begin                : Tue Dez 28 2004
+                                -------------
+    begin                : Tue Dec 28 2004
     copyright            : (C) 2004 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
  ***************************************************************************/
@@ -23,11 +23,14 @@
 
 class SpiceFile : public Component  {
 public:
-  SpiceFile(int No);
+  SpiceFile();
   ~SpiceFile();
   Component* newOne();
-  static Component* info(QString&, char* &, bool getNewOne=false);
+  static Element* info(QString&, char* &, bool getNewOne=false);
   void recreate();
+  QString NetList();
+
+  bool withSim;
 };
 
 #endif
