@@ -72,6 +72,9 @@ TR_Sim::TR_Sim()
 	QObject::tr("absolute tolerance of local truncation error")));
   Props.append(new Property("LTEfactor", "1", false,
 	QObject::tr("overestimation of local truncation error")));
+  Props.append(new Property("Solver", "CroutLU", false,
+	QObject::tr("method for solving the circuit matrix")+
+	" [CroutLU, DoolittleLU, HouseholderQR, HouseholderLQ]"));
 }
 
 TR_Sim::~TR_Sim()
@@ -83,7 +86,7 @@ Component* TR_Sim::newOne()
   return new TR_Sim();
 }
 
-Component* TR_Sim::info(QString& Name, char* &BitmapFile, bool getNewOne)
+Element* TR_Sim::info(QString& Name, char* &BitmapFile, bool getNewOne)
 {
   Name = QObject::tr("Transient simulation");
   BitmapFile = "tran";
