@@ -26,7 +26,7 @@ public:
   TR_Sim();
   ~TR_Sim();
   Component* newOne();
-  static Component* info(QString&, char* &, bool getNewOne=false);
+  static Element* info(QString&, char* &, bool getNewOne=false);
   void recreate();
 };
 
