@@ -1,6 +1,6 @@
 /***************************************************************************
-                          diagram.h  -  description
-                             -------------------
+                               diagram.h
+                              -----------
     begin                : Thu Oct 2 2003
     copyright            : (C) 2003 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -26,9 +26,7 @@
 #include <qfile.h>
 
 
-/**
-  *@author Michael Margraf
-  */
+#define INVALID_STR QObject::tr(" <invalid>")
 
 struct Axis {
   double  min, max; // least and greatest values of all graph data
@@ -49,14 +47,15 @@ public:
   virtual ~Diagram();
 
   virtual Diagram* newOne();
-  virtual int  calcDiagram();
-  virtual void calcCoordinate(double* &, double* &, int*, int*, Axis*) {};
+  virtual int  calcDiagram() { return 0; };
+  virtual void calcCoordinate
+               (double* &, double* &, double* &, int*, int*, Axis*) {};
   virtual void calcLimits() {};
-  void    calcData(Graph*, int);
+  
+  virtual void paint(ViewPainter*);
   void    setCenter(int, int, bool relative=false);
   void    getCenter(int&, int&);
   void    paintScheme(QPainter*);
-  void    paint(ViewPainter*);
   void    Bounding(int&, int&, int&, int&);
   bool    getSelected(int, int);
   bool    ResizeTouched(int, int);
@@ -66,18 +65,10 @@ public:
   void updateGraphData();
   void loadGraphData(const QString&);
   void recalcGraphData();
-  bool loadVarData(const QString&);
-  int  loadIndepVarData(const QString&, const QString&);
+  bool loadVarData(const QString&, Graph*);
+  int  loadIndepVarData(const QString&, char*, Axis*, Graph*);
 
-  void calcSmithAxisScale(Axis*, int&, int&);
-  void createSmithChart(Axis*, int Mode=7);
-  void calcPolarAxisScale(Axis*, double&, double&, double&);
-  void createPolarDiagram(Axis*, int Mode=3);
-
-  int  regionCode(int, int);
-  bool insideDiagram(int, int);
-  void clip(int* &);
-  void roundClip(int* &);
+  virtual bool insideDiagram(int, int);
 
   QString Name; // identity of diagram type (e.g. Polar), used for saving etc.
   QPen    GridPen;
@@ -90,6 +81,29 @@ public:
   int x3, y3;
   Axis  xAxis, yAxis, zAxis;   // axes (x, y left, y right)
   int State;  // to remember which resize area was touched
+
+  int rotX, rotY, rotZ;  // for "Rect3D": rotation around x, y and z axis
+
+
+protected:
+  void calcSmithAxisScale(Axis*, int&, int&);
+  void createSmithChart(Axis*, int Mode=7);
+  void calcPolarAxisScale(Axis*, double&, double&, double&);
+  void createPolarDiagram(Axis*, int Mode=3);
+
+  bool calcAxisScale(Axis*, double&, double&, double&, double&, double);
+  bool calcAxisLogScale(Axis*, int&, double&, double&, double&, int);
+  bool calcYAxis(Axis*, int);
+  virtual void createAxisLabels();
+
+  int  regionCode(int, int);
+  virtual void clip(int* &);
+  void rectClip(int* &);
+
+private:
+  void calcData(Graph*);
+
+  int Bounding_x1, Bounding_x2, Bounding_y1, Bounding_y2;
 };
 
 #endif
