@@ -1,5 +1,5 @@
 /***************************************************************************
-                          diagramdialog.cpp  -  description
+                              diagramdialog.cpp
                              -------------------
     begin                : Sun Oct 5 2003
     copyright            : (C) 2003 by Michael Margraf
@@ -18,6 +18,7 @@
 #include "diagramdialog.h"
 #include "qucsview.h"
 #include "qucs.h"
+#include "rect3ddiagram.h"
 
 #include <math.h>
 
@@ -36,6 +37,56 @@
 #include <qcolordialog.h>
 #include <qlineedit.h>
 #include <qcheckbox.h>
+#include <qslider.h>
+
+
+#define CROSS3D_SIZE   30
+#define WIDGET3D_SIZE  2*CROSS3D_SIZE
+// This widget class paints a small 3-dimensional coordinate cross.
+class Cross3D : public QWidget  {
+public:
+  Cross3D(float rx_, float ry_, float rz_, QWidget *parent = 0)
+          : QWidget(parent) {
+    rotX = rx_;
+    rotY = ry_;
+    rotZ = rz_;
+    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+    setMinimumSize(WIDGET3D_SIZE, WIDGET3D_SIZE);
+    resize(WIDGET3D_SIZE, WIDGET3D_SIZE);
+  };
+ ~Cross3D() {};
+
+  double rotX, rotY, rotZ;   // in radians !!!!
+
+private:
+  void  paintEvent(QPaintEvent*) {
+    QPainter Painter(this);
+    float cxx = cos(rotZ);
+    float cxy = sin(rotZ);
+    float cxz = sin(rotY);
+    float cyz = sin(rotX);
+    float cyy = cos(rotX);
+    float cyx = cyy * cxy + cyz * cxz * cxx;
+    cyy = cyy * cxx - cyz * cxz * cxy;
+    cyz *= cos(rotY);
+    cxx *= cos(rotY);
+    cxy *= cos(rotY);
+
+    Painter.setPen(QPen(QPen::red,2));
+    Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
+		     int(CROSS3D_SIZE * (1.0+cxx)),
+		     int(CROSS3D_SIZE * (1.0-cyx)));
+    Painter.setPen(QPen(QPen::green,2));
+    Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
+		     int(CROSS3D_SIZE * (1.0-cxy)),
+		     int(CROSS3D_SIZE * (1.0-cyy)));
+    Painter.setPen(QPen(QPen::blue,2));
+    Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
+		     int(CROSS3D_SIZE * (1.0+cxz)),
+		     int(CROSS3D_SIZE * (1.0+cyz)));
+  };
+};
+
 
 
 // standard colors: blue, red, magenta, green, cyan, yellow, black
@@ -46,8 +97,8 @@ static const QRgb DefaultColors[]
 
 
 DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
-                             QWidget *parent, const char *name )
-                    : QDialog(parent, name, TRUE, Qt::WDestructiveClose)
+                             QWidget *parent, Graph *currentGraph)
+                    : QDialog(parent, "", TRUE, Qt::WDestructiveClose)
 {
   Diag = d;
   Graphs.setAutoDelete(true);
@@ -55,12 +106,39 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
   defaultDataSet = _DataSet;
   setCaption(tr("Edit Diagram Properties"));
   changed = false;
+  transfer = false;  // have changes be applied ? (used by "Cancel")
   toTake = false;   // double-clicked variable be inserted into graph list ?
-//  setFixedSize(QSize(400, 400));
-//  setMinimumSize(QSize(400, 400));
 
-  ValDouble = new QDoubleValidator(-1e200, 1e200, 6, this);
+  Expr.setPattern("[^\"]+");
+  Validator  = new QRegExpValidator(Expr, this);
+  ValInteger = new QIntValidator(0, 360, this);
+  ValDouble  = new QDoubleValidator(-1e200, 1e200, 6, this);
 
+  QString NameY, NameZ;
+  if((Diag->Name == "Rect") || (Diag->Name == "Curve")) {
+    NameY = tr("left Axis");
+    NameZ = tr("right Axis");
+  }
+  else if(Diag->Name == "Polar") {
+    NameY = tr("y-Axis");
+  }
+  else if((Diag->Name == "Smith") || (Diag->Name == "ySmith")) {
+    NameY = tr("y-Axis");
+  }
+  else if(Diag->Name == "PS") {
+    NameY = tr("smith Axis");
+    NameZ = tr("polar Axis");
+  }
+  else if(Diag->Name == "SP") {
+    NameY = tr("polar Axis");
+    NameZ = tr("smith Axis");
+  }
+  else if(Diag->Name == "Rect3D") {
+    NameY = tr("y-Axis");
+    NameZ = tr("z-Axis");
+  }
+
+  
   all = new QVBoxLayout(this); // to provide neccessary size
   QTabWidget *t = new QTabWidget(this);
   all->addWidget(t);
@@ -69,18 +147,18 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
   QVBox *Tab1 = new QVBox(this);
   Tab1->setSpacing(5);
 
-  Label4 = 0;
+  Label4 = 0;     // different types with same content
   yrLabel = 0;
   yAxisBox = 0;
+  rotationY = rotationZ = 0;
 
   QVButtonGroup *InputGroup = new QVButtonGroup(tr("Graph Input"), Tab1);
   GraphInput = new QLineEdit(InputGroup);
+  GraphInput->setValidator(Validator);
   connect(GraphInput, SIGNAL(textChanged(const QString&)),
 		      SLOT(slotResetToTake(const QString&)));
   QHBox *Box2 = new QHBox(InputGroup);
   Box2->setSpacing(5);
-  Expr.setPattern("[0-9]{1,2}");  // valid expression for property input
-  Validator = new QRegExpValidator(Expr, this);
 
   if(Diag->Name == "Tab") {
     Label1 = new QLabel(tr("Number Notation: "), Box2);
@@ -93,7 +171,8 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
 
     Label2 = new QLabel(tr("Precision:"), Box2);
     Property2 = new QLineEdit(Box2);
-    Property2->setValidator(Validator);
+    Property2->setValidator(ValInteger);
+    Property2->setMaxLength(2);
     Property2->setMaximumWidth(25);
     Property2->setText("3");
   }
@@ -121,29 +200,21 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
 
     Label2 = new QLabel(tr("Thickness:"),Box2);
     Property2 = new QLineEdit(Box2);
-    Property2->setValidator(Validator);
+    Property2->setValidator(ValInteger);
     Property2->setMaximumWidth(25);
+    Property2->setMaxLength(2);
     Property2->setText("0");
 
-    if((Diag->Name=="Rect") || (Diag->Name=="PS") || (Diag->Name=="SP")){
+    if((Diag->Name=="Rect") || (Diag->Name=="PS") || (Diag->Name=="SP") ||
+       (Diag->Name=="Curve")) {
       QHBox *Box3 = new QHBox(InputGroup);
       Box3->setSpacing(5);
 
       Label4 = new QLabel(tr("y-Axis:"),Box3);
       Label4->setEnabled(false);
       yAxisBox = new QComboBox(Box3);
-      if(Diag->Name == "PS") {
-        yAxisBox->insertItem(tr("smith Axis"));
-        yAxisBox->insertItem(tr("polar Axis"));
-      }
-      else if(Diag->Name == "SP") {
-        yAxisBox->insertItem(tr("polar Axis"));
-        yAxisBox->insertItem(tr("smith Axis"));
-      }
-      else {
-        yAxisBox->insertItem(tr("left Axis"));
-        yAxisBox->insertItem(tr("right Axis"));
-      }
+      yAxisBox->insertItem(NameY);
+      yAxisBox->insertItem(NameZ);
       yAxisBox->setEnabled(false);
       connect(yAxisBox, SIGNAL(activated(int)), SLOT(slotSetYAxis(int)));
       Box3->setStretchFactor(new QWidget(Box3), 5); // stretchable placeholder
@@ -168,9 +239,6 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
   ChooseVars->addColumn(tr("Name"));
   ChooseVars->addColumn(tr("Status"));
   ChooseVars->addColumn(tr("Size"));
-// QT 3.2
-//  connect(Content, SIGNAL(doubleClicked(QListViewItem*, constQPoint&,int)),
-//                   SLOT(slotTakeVar(QListViewItem*, constQPoint&,int)));
   connect(ChooseVars, SIGNAL(doubleClicked(QListViewItem*)),
 		      SLOT(slotTakeVar(QListViewItem*)));
 
@@ -187,71 +255,162 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
   t->addTab(Tab1, tr("Data"));
 
   // ...........................................................
+  int Row = 0;
   if(Diag->Name != "Tab") {
     QWidget *Tab2 = new QWidget(t);
-    QGridLayout *gp = new QGridLayout(Tab2,10,2,5,5);
+    QGridLayout *gp = new QGridLayout(Tab2,13,3,5,5);
 
-    gp->addWidget(new QLabel(tr("x-Axis Label:"), Tab2), 0,0);
+    gp->addMultiCellWidget(new QLabel(tr("x-Axis Label:"), Tab2), Row,Row,0,0);
     xLabel = new QLineEdit(Tab2);
-    gp->addWidget(xLabel,0,1);
+    xLabel->setValidator(Validator);
+    gp->addMultiCellWidget(xLabel, Row,Row,1,2);
+    Row++;
 
-    gp->addWidget(new QLabel(tr("left y-Axis Label:"), Tab2), 1,0);
+    gp->addMultiCellWidget(
+		new QLabel(NameY+" "+tr("Label:"), Tab2), Row,Row,0,0);
     ylLabel = new QLineEdit(Tab2);
-    gp->addWidget(ylLabel,1,1);
+    ylLabel->setValidator(Validator);
+    gp->addMultiCellWidget(ylLabel, Row,Row,1,2);
+    Row++;
 
-    if(Diag->Name == "Rect") {
-      gp->addWidget(new QLabel(tr("right y-Axis Label:"), Tab2), 2,0);
+    if((Diag->Name != "Smith") && (Diag->Name != "Polar")) {
+      gp->addMultiCellWidget(
+		new QLabel(NameZ +" "+tr("Label:"), Tab2), Row,Row,0,0);
       yrLabel = new QLineEdit(Tab2);
-      gp->addWidget(yrLabel,2,1);
+      yrLabel->setValidator(Validator);
+      gp->addMultiCellWidget(yrLabel, Row,Row,1,2);
+      Row++;
     }
 
-    GridOn = new QCheckBox(tr("show Grid"), Tab2);
-    connect(GridOn, SIGNAL(stateChanged(int)), SLOT(slotSetGridBox(int)));
-    gp->addMultiCellWidget(GridOn,3,3,0,1);
+    gp->addMultiCellWidget(new QLabel(
+        tr("<b>Label text</b>: Use LaTeX style for special characters, e.g. \\tau"),
+        Tab2),  Row,Row,0,2);
+    Row++;
 
-    GridLabel1 = new QLabel(tr("Grid Color:"),Tab2);
-    gp->addWidget(GridLabel1, 4,0);
-    GridColorButt = new QPushButton("        ",Tab2);
-    connect(GridColorButt, SIGNAL(clicked()), SLOT(slotSetGridColor()));
-    gp->addWidget(GridColorButt, 4,1);
+    if(Diag->Name != "Rect3D") {
+      GridOn = new QCheckBox(tr("show Grid"), Tab2);
+      gp->addMultiCellWidget(GridOn, Row,Row,0,2);
+      Row++;
 
-    GridLabel2 = new QLabel(tr("Grid Style: "), Tab2);
-    gp->addWidget(GridLabel2, 5,0);
-    GridStyleBox = new QComboBox(Tab2);
-    GridStyleBox->insertItem(tr("solid line"));
-    GridStyleBox->insertItem(tr("dash line"));
-    GridStyleBox->insertItem(tr("dot line"));
-    GridStyleBox->insertItem(tr("dash dot line"));
-    GridStyleBox->insertItem(tr("dash dot dot line"));
-    gp->addWidget(GridStyleBox, 5,1);
+      GridLabel1 = new QLabel(tr("Grid Color:"),Tab2);
+      gp->addMultiCellWidget(GridLabel1, Row,Row,0,0);
+      GridColorButt = new QPushButton("        ",Tab2);
+      connect(GridColorButt, SIGNAL(clicked()), SLOT(slotSetGridColor()));
+      gp->addMultiCellWidget(GridColorButt, Row,Row,1,2);
+      Row++;
+      GridColorButt->setPaletteBackgroundColor(Diag->GridPen.color());
+
+      GridLabel2 = new QLabel(tr("Grid Style: "), Tab2);
+      gp->addMultiCellWidget(GridLabel2, Row,Row,0,0);
+      GridStyleBox = new QComboBox(Tab2);
+      GridStyleBox->insertItem(tr("solid line"));
+      GridStyleBox->insertItem(tr("dash line"));
+      GridStyleBox->insertItem(tr("dot line"));
+      GridStyleBox->insertItem(tr("dash dot line"));
+      GridStyleBox->insertItem(tr("dash dot dot line"));
+      gp->addMultiCellWidget(GridStyleBox, Row,Row,1,2);
+      Row++;
+      GridStyleBox->setCurrentItem(Diag->GridPen.style()-1);
+    
+      GridOn->setChecked(Diag->xAxis.GridOn);
+      if(!Diag->xAxis.GridOn) slotSetGridBox(QButton::Off);
+      connect(GridOn, SIGNAL(stateChanged(int)), SLOT(slotSetGridBox(int)));
+    }
+    else {
+      GridOn = 0;
+      GridColorButt = 0;
+      GridStyleBox = 0;
+    }
 
     // ...........................................................
     // transfer the diagram properties to the dialog
     xLabel->setText(Diag->xAxis.Label);
     ylLabel->setText(Diag->yAxis.Label);
     if(yrLabel)  yrLabel->setText(Diag->zAxis.Label);
-    GridOn->setChecked(Diag->xAxis.GridOn);
-    if(!Diag->xAxis.GridOn) slotSetGridBox(QButton::Off);
-    GridColorButt->setPaletteBackgroundColor(Diag->GridPen.color());
-    GridStyleBox->setCurrentItem(Diag->GridPen.style()-1);
 
-    if(Diag->Name == "Rect") {
+    if((Diag->Name.left(4) == "Rect") || (Diag->Name == "Curve")) {
       GridLogX = new QCheckBox(tr("logarithmical X Axis Grid"), Tab2);
-      gp->addMultiCellWidget(GridLogX,6,6,0,1);
+      gp->addMultiCellWidget(GridLogX, Row,Row,0,2);
+      Row++;
 
-      GridLogY = new QCheckBox(tr("logarithmical left Y Axis Grid"), Tab2);
-      gp->addMultiCellWidget(GridLogY,7,7,0,1);
+      GridLogY = new QCheckBox(tr("logarithmical")+" "+NameY+" "+tr("Grid"), Tab2);
+      gp->addMultiCellWidget(GridLogY, Row,Row,0,2);
+      Row++;
 
-      GridLogZ = new QCheckBox(tr("logarithmical right Y Axis Grid"), Tab2);
-      gp->addMultiCellWidget(GridLogZ,8,8,0,1);
+      GridLogZ = new QCheckBox(tr("logarithmical")+" "+NameZ+" "+tr("Grid"), Tab2);
+      gp->addMultiCellWidget(GridLogZ, Row,Row,0,2);
+      Row++;
 
       // ...........................................................
       // transfer the diagram properties to the dialog
       GridLogX->setChecked(Diag->xAxis.log);
       GridLogY->setChecked(Diag->yAxis.log);
       GridLogZ->setChecked(Diag->zAxis.log);
+
+
+      if(Diag->Name == "Rect3D") {
+	QLabel *LabelRotX = new QLabel(tr("Rotation around x-Axis:"), Tab2);
+	LabelRotX->setPaletteForegroundColor(Qt::red);
+	gp->addWidget(LabelRotX, Row,0);
+	SliderRotX = new QSlider(0,360,20, ((Rect3DDiagram*)Diag)->rotX,
+				 Qt::Horizontal, Tab2);
+	gp->addWidget(SliderRotX, Row,1);
+	connect(SliderRotX, SIGNAL(valueChanged(int)), SLOT(slotNewRotX(int)));
+	rotationX = new QLineEdit(Tab2);
+	rotationX->setValidator(ValInteger);
+	rotationX->setMaxLength(3);
+	rotationX->setMaximumWidth(40);
+	gp->addWidget(rotationX, Row,2);
+	connect(rotationX, SIGNAL(textChanged(const QString&)),
+			   SLOT(slotEditRotX(const QString&)));
+	Row++;
+
+	QLabel *LabelRotY = new QLabel(tr("Rotation around y-Axis:"), Tab2);
+	LabelRotY->setPaletteForegroundColor(Qt::green);
+	gp->addWidget(LabelRotY, Row,0);
+	SliderRotY = new QSlider(0,360,20, ((Rect3DDiagram*)Diag)->rotY,
+				 Qt::Horizontal, Tab2);
+	gp->addWidget(SliderRotY, Row,1);
+	connect(SliderRotY, SIGNAL(valueChanged(int)), SLOT(slotNewRotY(int)));
+	rotationY = new QLineEdit(Tab2);
+	rotationY->setValidator(ValInteger);
+	rotationY->setMaxLength(3);
+	rotationY->setMaximumWidth(40);
+	gp->addWidget(rotationY, Row,2);
+	connect(rotationY, SIGNAL(textChanged(const QString&)),
+			   SLOT(slotEditRotY(const QString&)));
+	Row++;
+
+	QLabel *LabelRotZ = new QLabel(tr("Rotation around z-Axis:"), Tab2);
+	LabelRotZ->setPaletteForegroundColor(Qt::blue);
+	gp->addWidget(LabelRotZ, Row,0);
+	SliderRotZ = new QSlider(0,360,20, ((Rect3DDiagram*)Diag)->rotZ,
+				 Qt::Horizontal, Tab2);
+	gp->addWidget(SliderRotZ, Row,1);
+	connect(SliderRotZ, SIGNAL(valueChanged(int)), SLOT(slotNewRotZ(int)));
+	rotationZ = new QLineEdit(Tab2);
+	rotationZ->setValidator(ValInteger);
+	rotationZ->setMaxLength(3);
+	rotationZ->setMaximumWidth(40);
+	gp->addWidget(rotationZ, Row,2);
+	connect(rotationZ, SIGNAL(textChanged(const QString&)),
+			   SLOT(slotEditRotZ(const QString&)));
+	Row++;
+
+	gp->addWidget(new QLabel(tr("2D-projection:"), Tab2), Row,0);
+	DiagCross = new Cross3D(((Rect3DDiagram*)Diag)->rotX,
+				((Rect3DDiagram*)Diag)->rotY,
+				((Rect3DDiagram*)Diag)->rotZ, Tab2);
+	gp->addWidget(DiagCross, Row,1);
+
+	// transfer the diagram properties to the dialog
+	rotationX->setText(QString::number(((Rect3DDiagram*)Diag)->rotX));
+	rotationY->setText(QString::number(((Rect3DDiagram*)Diag)->rotY));
+	rotationZ->setText(QString::number(((Rect3DDiagram*)Diag)->rotZ));
+
+      }
     }
-    else  GridLogX = GridLogY = GridLogZ = 0;
+    else GridLogX = GridLogY = GridLogZ = 0;
 
     t->addTab(Tab2, tr("Properties"));
 
@@ -283,12 +442,7 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
 
 
     QHGroupBox *axisY;
-    if(Diag->Name == "PS")
-      axisY = new QHGroupBox(tr("Smith Axis"), Tab3);
-    else if(Diag->Name == "SP")
-      axisY = new QHGroupBox(tr("Polar Axis"), Tab3);
-    else
-      axisY = new QHGroupBox(tr("left y-Axis"), Tab3);
+    axisY = new QHGroupBox(NameY, Tab3);
 
     QVBox *VBox5 = new QVBox(axisY);
     VBox5->setStretchFactor(new QWidget(VBox5),5); // stretchable placeholder
@@ -314,12 +468,7 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
 
 
     QHGroupBox *axisZ;
-    if(Diag->Name == "PS")
-      axisZ = new QHGroupBox(tr("Polar Axis"), Tab3);
-    else if(Diag->Name == "SP")
-      axisZ = new QHGroupBox(tr("Smith Axis"), Tab3);
-    else
-      axisZ = new QHGroupBox(tr("right y-Axis"), Tab3);
+    axisZ = new QHGroupBox(NameZ, Tab3);
 
     QVBox *VBox9 = new QVBox(axisZ);
     VBox9->setStretchFactor(new QWidget(VBox9),5); // stretchable placeholder
@@ -374,11 +523,12 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
        (Diag->Name == "Polar")) {
        axisZ->setEnabled(false);
     }
-    if(Diag->Name != "Rect") {
-      axisX->setEnabled(false);
-      startY->setEnabled(false);
-      startZ->setEnabled(false);
-    }
+    if(Diag->Name.left(4) != "Rect")   // cartesian 2D and 3D
+      if(Diag->Name != "Curve") {
+        axisX->setEnabled(false);
+        startY->setEnabled(false);
+        startZ->setEnabled(false);
+      }
   }
   else  stepX = 0;
 
@@ -414,21 +564,29 @@ DiagramDialog::DiagramDialog(Diagram *d, const QString& _DataSet,
   }
   slotReadVars(0);  // put variables into the ListView
 
-  // ...........................................................
-  // put all graphs into the ListBox
-  for(Graph *pg = Diag->Graphs.first(); pg != 0; pg = Diag->Graphs.next())
-    GraphList->insertItem(pg->Var);
-
   if(Diag->Name != "Tab")
     ColorButt->setPaletteBackgroundColor
 	(QColor(DefaultColors[GraphList->count()]));
+
+  // ...........................................................
+  // put all graphs into the ListBox
+  Row = 0;
+  for(Graph *pg = Diag->Graphs.first(); pg != 0; pg = Diag->Graphs.next()) {
+    GraphList->insertItem(pg->Var);
+    if(pg == currentGraph) {
+      GraphList->setCurrentItem(Row);   // select current graph
+      SelectGraph(currentGraph);
+    }
+    Row++;
+  }
 }
 
 DiagramDialog::~DiagramDialog()
 {
   delete all;   // delete all widgets from heap
-  delete Validator;
+  delete ValInteger;
   delete ValDouble;
+  delete Validator;
 }
 
 // --------------------------------------------------------------------------
@@ -500,6 +658,7 @@ void DiagramDialog::slotTakeVar(QListViewItem *Item)
         yAxisBox->setEnabled(true);
         Label4->setEnabled(true);
       }
+      else if(Diag->Name == "Rect3D") g->yAxisNo = 1;
 
       Label3->setEnabled(true);
       ColorButt->setEnabled(true);
@@ -523,7 +682,7 @@ void DiagramDialog::slotTakeVar(QListViewItem *Item)
 }
 
 // --------------------------------------------------------------------------
-// Puts the text of the selected graph into the line edit.
+// Is called if a graph text is clicked in the BistBox.
 void DiagramDialog::slotSelectGraph(QListBoxItem *item)
 {
   if(item == 0) {
@@ -531,12 +690,17 @@ void DiagramDialog::slotSelectGraph(QListBoxItem *item)
     return;
   }
 
-  int index = GraphList->index(item);
+  SelectGraph (Graphs.at (GraphList->index(item)));
+}
+
+// --------------------------------------------------------------------------
+// Puts the text of the selected graph into the line edit.
+void DiagramDialog::SelectGraph(Graph *g)
+{
   GraphInput->blockSignals(true);
-  GraphInput->setText(GraphList->text(index));
+  GraphInput->setText(g->Var);
   GraphInput->blockSignals(false);
 
-  Graph *g = Graphs.at(index);
   if(Diag->Name != "Tab") {
     Property2->setText(QString::number(g->Thick));
     ColorButt->setPaletteBackgroundColor(g->Color);
@@ -610,6 +774,7 @@ void DiagramDialog::slotNewGraph()
     g->Thick = Property2->text().toInt();
     g->Style = PropertyBox->currentItem();
     if(yAxisBox)  g->yAxisNo = yAxisBox->currentItem();
+    else if(Diag->Name == "Rect3D")  g->yAxisNo = 1;
   }
   else {
     g->Precision = Property2->text().toInt();
@@ -625,8 +790,7 @@ void DiagramDialog::slotNewGraph()
 void DiagramDialog::slotOK()
 {
   slotApply();
-  if(changed) done(QDialog::Accepted);
-  else done(QDialog::Rejected);
+  slotCancel();
 }
 
 // --------------------------------------------------------------------------
@@ -648,20 +812,23 @@ void DiagramDialog::slotApply()
       Diag->yAxis.Label = ylLabel->text();
       changed = true;
     }
-    if(Diag->xAxis.GridOn != GridOn->isChecked()) {
+    
+    if(GridOn) if(Diag->xAxis.GridOn != GridOn->isChecked()) {
       Diag->xAxis.GridOn = GridOn->isChecked();
       Diag->yAxis.GridOn = GridOn->isChecked();
       changed = true;
     }
-    if(Diag->GridPen.color() != GridColorButt->paletteBackgroundColor()) {
+    if(GridColorButt)
+      if(Diag->GridPen.color() != GridColorButt->paletteBackgroundColor()) {
       Diag->GridPen.setColor(GridColorButt->paletteBackgroundColor());
       changed = true;
     }
-    if(Diag->GridPen.style()!=(Qt::PenStyle)(GridStyleBox->currentItem()+1)) {
+    if(GridStyleBox)
+      if(Diag->GridPen.style()!=(Qt::PenStyle)(GridStyleBox->currentItem()+1)) {
       Diag->GridPen.setStyle((Qt::PenStyle)(GridStyleBox->currentItem()+1));
       changed = true;
     }
-    if(Diag->Name == "Rect") {
+    if((Diag->Name != "Smith") && (Diag->Name != "Polar")) {
       if(Diag->zAxis.Label.isEmpty())
         Diag->zAxis.Label = ""; // can be not 0 and empty!
       if(yrLabel->text().isEmpty()) yrLabel->setText("");
@@ -669,7 +836,9 @@ void DiagramDialog::slotApply()
         Diag->zAxis.Label = yrLabel->text();
         changed = true;
       }
+    }
 
+    if(Diag->Name.left(4) == "Rect") {
       if(Diag->xAxis.log != GridLogX->isChecked()) {
         Diag->xAxis.log = GridLogX->isChecked();
         changed = true;
@@ -741,7 +910,29 @@ void DiagramDialog::slotApply()
       Diag->zAxis.limit_max = stopZ->text().toDouble();
       changed = true;
     }
-  }
+
+    if(Diag->Name == "Rect3D") {
+      if(rotationX) {
+        if(((Rect3DDiagram*)Diag)->rotX != rotationX->text().toInt()) {
+          ((Rect3DDiagram*)Diag)->rotX = rotationX->text().toInt();
+          changed = true;
+        }
+      }
+      if(rotationY) {
+        if(((Rect3DDiagram*)Diag)->rotY != rotationY->text().toInt()) {
+          ((Rect3DDiagram*)Diag)->rotY = rotationY->text().toInt();
+          changed = true;
+        }
+      }
+      if(rotationZ) {
+        if(((Rect3DDiagram*)Diag)->rotZ != rotationZ->text().toInt()) {
+          ((Rect3DDiagram*)Diag)->rotZ = rotationZ->text().toInt();
+          changed = true;
+        }
+      }
+    }
+  }   // of "if(Diag->Name != "Tab")"
+
   Diag->Graphs.clear();   // delete the graphs
   Graphs.setAutoDelete(false);
   for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next())
@@ -752,6 +943,7 @@ void DiagramDialog::slotApply()
   Diag->loadGraphData(defaultDataSet);
   ((QucsView*)parent())->viewport()->repaint();
   copyDiagramGraphs();
+  if(changed) transfer = true;   // changes have been applied ?
 }
 
 
@@ -761,7 +953,15 @@ void DiagramDialog::slotCancel()
 {
 //  Diag->loadGraphData(defaultDataSet);
 //  ((QucsView*)parent())->viewport()->repaint();
-  reject();
+  if(transfer) done(QDialog::Accepted);
+  else done(QDialog::Rejected);
+}
+
+//-----------------------------------------------------------------
+// To get really all close events (even <Escape> key).
+void DiagramDialog::reject()
+{
+  slotCancel();
 }
 
 // --------------------------------------------------------------------------
@@ -887,7 +1087,7 @@ void DiagramDialog::slotSetYAxis(int axis)
 void DiagramDialog::slotManualX(int state)
 {
   if(state == QButton::On) {
-    if(Diag->Name == "Rect")
+    if((Diag->Name.left(4) == "Rect") || (Diag->Name == "Curve"))
       startX->setEnabled(true);
     stopX->setEnabled(true);
     if(GridLogX) if(GridLogX->isChecked())  return;
@@ -904,7 +1104,7 @@ void DiagramDialog::slotManualX(int state)
 void DiagramDialog::slotManualY(int state)
 {
   if(state == QButton::On) {
-    if(Diag->Name == "Rect")
+    if((Diag->Name.left(4) == "Rect") || (Diag->Name == "Curve"))
       startY->setEnabled(true);
     stopY->setEnabled(true);
     if(GridLogY) if(GridLogY->isChecked())  return;
@@ -921,7 +1121,7 @@ void DiagramDialog::slotManualY(int state)
 void DiagramDialog::slotManualZ(int state)
 {
   if(state == QButton::On) {
-    if(Diag->Name == "Rect")
+    if((Diag->Name.left(4) == "Rect") || (Diag->Name == "Curve"))
       startZ->setEnabled(true);
     stopZ->setEnabled(true);
     if(GridLogZ) if(GridLogZ->isChecked())  return;
@@ -952,3 +1152,57 @@ void DiagramDialog::slotChangeTab(QWidget*)
     else  if(manualZ->isChecked())  stepZ->setEnabled(true);
   }
 }
+
+// --------------------------------------------------------------------------
+// Is called when the slider for rotation angle is changed.
+void DiagramDialog::slotNewRotX(int Value)
+{
+  rotationX->setText(QString::number(Value));
+  DiagCross->rotX = float(Value) * M_PI/180.0;
+  DiagCross->update();
+}
+
+// --------------------------------------------------------------------------
+// Is called when the slider for rotation angle is changed.
+void DiagramDialog::slotNewRotY(int Value)
+{
+  rotationY->setText(QString::number(Value));
+  DiagCross->rotY = float(Value) * M_PI/180.0;
+  DiagCross->update();
+}
+
+// --------------------------------------------------------------------------
+// Is called when the slider for rotation angle is changed.
+void DiagramDialog::slotNewRotZ(int Value)
+{
+  rotationZ->setText(QString::number(Value));
+  DiagCross->rotZ = float(Value) * M_PI/180.0;
+  DiagCross->update();
+}
+
+// --------------------------------------------------------------------------
+// Is called when the number (text) for rotation angle is changed.
+void DiagramDialog::slotEditRotX(const QString& Text)
+{
+  SliderRotX->setValue(Text.toInt());
+  DiagCross->rotX = Text.toFloat() * M_PI/180.0;
+  DiagCross->update();
+}
+
+// --------------------------------------------------------------------------
+// Is called when the number (text) for rotation angle is changed.
+void DiagramDialog::slotEditRotY(const QString& Text)
+{
+  SliderRotY->setValue(Text.toInt());
+  DiagCross->rotY = Text.toFloat() * M_PI/180.0;
+  DiagCross->update();
+}
+
+// --------------------------------------------------------------------------
+// Is called when the number (text) for rotation angle is changed.
+void DiagramDialog::slotEditRotZ(const QString& Text)
+{
+  SliderRotZ->setValue(Text.toInt());
+  DiagCross->rotZ = Text.toFloat() * M_PI/180.0;
+  DiagCross->update();
+}
