@@ -73,8 +73,10 @@ void Graph::paintLines(ViewPainter *p, int x0, int y0)
 {
   int x, y;
   int *pp = Points;
+
   if(*pp < 0)  pp++;
-  for(int n1=countY; n1>0; n1--) {    // every branch of curves
+//  for(int n1=countY; n1>0; n1--) {    // every branch of curves
+  while(*pp > -99) {
     if(*pp >= 0)  p->drawPoint(x0+(*pp), y0-(*(pp+1)));
     while(*pp > -9) {   // until end of branch
       pp += 2;
@@ -168,19 +170,22 @@ int Graph::getSelected(int x, int y)
   int *pp = Points;
   if(pp == 0) return -1;
 
-  int A, zi;
+  int A, z=0, zi;
   int dx, dx2, x1;
   int dy, dy2, y1;
 
   int countX = cPointsX.getFirst()->count;
   if(*pp < 0) {
+    if(*pp < -9) z++;
     pp++;
     if(*pp < -9) {
+      z++;
       pp++;
       if(*pp < -10)  return -1;   // not even one point ?
     }
   }
-  for(int z=0; z<countY; z++) {  // check every branch of curves
+//  for(int z=0; z<countY; z++) {  // check every branch of curves
+  while(*pp > -99) {
     zi = 0;
     do {
       x1 = *(pp++);  y1 = *(pp++);
@@ -221,6 +226,7 @@ int Graph::getSelected(int x, int y)
       if(A <= 0)  return z*countX;// + zi;  // lies x/y onto the graph line ?
     } while(true);
     pp++;
+    z++;
   }
 
   return -1;
