@@ -62,54 +62,81 @@ void Marker::initText(int n)
   Axis *pa;
   if(pGraph->yAxisNo == 0)  pa = &(Diag->yAxis);
   else  pa = &(Diag->zAxis);
-  double *num, *py = pGraph->cPointsY + 2*n;
+  double *px, *py=0, *pz;
   Text = "";
   nVarPos = 0;
+
+  bool isCross = false;
+  int nn, nnn, m, x, y, d, dmin = INT_MAX;
   DataX *pD = pGraph->cPointsX.first();
+  px  = pD->Points;
+  nnn = pD->count;
+  DataX *pDy = pGraph->cPointsX.next();
+  if(pDy) {   // only for 3D diagram
+    nn = pGraph->countY * pD->count;
+    py  = pDy->Points;
+    if(n >= nn) {    // is on cross grid ?
+      isCross = true;
+      n -= nn;
+      n /= nnn;
+      px += (n % nnn);
+      if(pGraph->cPointsX.next())   // more than 2 indep variables ?
+        n  = (n % nnn) + (n / nnn) * nnn * pDy->count;
+      nnn = pDy->count;
+    }
+    else py += (n/pD->count) % pDy->count;
+  }
 
   // find exact marker position
-  int nn, x, y, d, dmin = INT_MAX;
-  for(nn=0; nn<pD->count; nn++) {
-    num = pD->Points + nn;
-    Diag->calcCoordinate(num, py, &x, &y, pa);
+  m  = nnn - 1;
+  pz = pGraph->cPointsY + 2*n;
+  for(nn=0; nn<nnn; nn++) {
+    Diag->calcCoordinate(px, pz, py, &x, &y, pa);
+    if(isCross) {
+      px--;
+      py++;
+      pz += 2*(pD->count-1);
+    }
     x -= cx;
     y -= cy;
     d = x*x + y*y;
     if(d < dmin) {
       dmin = d;
-      n = (n - n % pD->count) + nn;
+      m = nn;
     }
   }
-  if(dmin == INT_MAX)
-    n = n + nn - 1; // take last position
+  if(isCross) m *= pD->count;
+  n += m;
 
   // gather text of all independent variables
   nn = n;
-  for(; pD!=0; pD = pGraph->cPointsX.next()) {
-    num = pD->Points + (nn % pD->count);
-    VarPos[nVarPos++] = *num;
-    Text += pD->Var + ": " + QString::number(*num,'g',Precision) + "\n";
+  for(pD = pGraph->cPointsX.first(); pD!=0; pD = pGraph->cPointsX.next()) {
+    px = pD->Points + (nn % pD->count);
+    VarPos[nVarPos++] = *px;
+    Text += pD->Var + ": " + QString::number(*px,'g',Precision) + "\n";
     nn /= pD->count;
   }
 
   // gather text of dependent variable
-  py = (pGraph->cPointsY) + 2*n;
+  pz = (pGraph->cPointsY) + 2*n;
   Text += pGraph->Var + ": ";
   switch(numMode) {
-    case 0: Text += complexRect(*py, *(py+1), Precision);
+    case 0: Text += complexRect(*pz, *(pz+1), Precision);
 	    break;
-    case 1: Text += complexDeg(*py, *(py+1), Precision);
+    case 1: Text += complexDeg(*pz, *(pz+1), Precision);
 	    break;
-    case 2: Text += complexRad(*py, *(py+1), Precision);
+    case 2: Text += complexRad(*pz, *(pz+1), Precision);
 	    break;
   }
 
-  num = &(VarPos[0]);
-  Diag->calcCoordinate(num, py, &cx, &cy, pa);
+  px = VarPos;
+  py = VarPos + 1;
+  Diag->calcCoordinate(px, pz, py, &cx, &cy, pa);
 
   if(!Diag->insideDiagram(cx, cy))
-    if(Diag->Name != "Rect") {   // if marker out of valid bounds, ...
-      cx = Diag->x2 >> 1;        // ... point to origin
+    // if marker out of valid bounds, point to origin
+    if((Diag->Name.left(4) != "Rect") && (Diag->Name != "Curve")) {
+      cx = Diag->x2 >> 1;
       cy = Diag->y2 >> 1;
     }
     else {
@@ -153,14 +180,20 @@ void Marker::createText()
   }
 
 
-  double *py = (pGraph->cPointsY) + 2*n;
+  double *py=0, *pz = pGraph->cPointsY + 2*n;
+  pD = pGraph->cPointsX.first();
+  if(pGraph->cPointsX.next()) {
+    py = pGraph->cPointsX.current()->Points;   // only for 3D diagram
+    py += (n / pD->count) % pGraph->cPointsX.current()->count;
+  }
+
   Text += pGraph->Var + ": ";
   switch(numMode) {
-    case 0: Text += complexRect(*py, *(py+1), Precision);
+    case 0: Text += complexRect(*pz, *(pz+1), Precision);
 	    break;
-    case 1: Text += complexDeg(*py, *(py+1), Precision);
+    case 1: Text += complexDeg(*pz, *(pz+1), Precision);
 	    break;
-    case 2: Text += complexRad(*py, *(py+1), Precision);
+    case 2: Text += complexRad(*pz, *(pz+1), Precision);
 	    break;
   }
 
@@ -168,11 +201,13 @@ void Marker::createText()
   if(pGraph->yAxisNo == 0)  pa = &(Diag->yAxis);
   else  pa = &(Diag->zAxis);
   pp = &(VarPos[0]);
-  Diag->calcCoordinate(pp, py, &cx, &cy, pa);
+  Diag->calcCoordinate(pp, pz, py, &cx, &cy, pa);
+
 
   if(!Diag->insideDiagram(cx, cy))
-    if(Diag->Name != "Rect") {   // if marker out of valid bounds, ...
-      cx = Diag->x2 >> 1;        // ... point to origin
+    // if marker out of valid bounds, point to origin
+    if((Diag->Name.left(4) != "Rect") && (Diag->Name != "Curve")) {
+      cx = Diag->x2 >> 1;
       cy = Diag->y2 >> 1;
     }
     else {
@@ -188,7 +223,7 @@ void Marker::makeInvalid()
 {
   cx = 0;
   cy = 0;
-  if(Diag) if(Diag->Name != "Rect") {
+  if(Diag) if(Diag->Name.left(4) != "Rect") if(Diag->Name != "Curve") {
     cx = Diag->x2 >> 1;
     cy = Diag->y2 >> 1;
   }
@@ -280,7 +315,7 @@ bool Marker::moveUpDown(bool up)
 
     } while(px <= pD->Points);  // go to next dimension ?
 
-    px--;  // one position up
+    px--;  // one position down
     VarPos[i] = *px;
     while(i > 1) {
       pD = pGraph->cPointsX.prev();
