@@ -1,6 +1,6 @@
 /***************************************************************************
-                          rectdiagram.h  -  description
-                             -------------------
+                               rectdiagram.h
+                              ---------------
     begin                : Thu Oct 2 2003
     copyright            : (C) 2003 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -20,9 +20,6 @@
 
 #include "diagram.h"
 
-/**
-  *@author Michael Margraf
-  */
 
 class RectDiagram : public Diagram  {
 public: 
@@ -31,15 +28,13 @@ public:
 
 
   Diagram* newOne();
+  static Element* info(QString&, char* &, bool getNewOne=false);
   int  calcDiagram();
-  bool calcAxisScale(Axis*, double&, double&, double&, double&, double);
-  bool calcAxisLogScale(Axis*, int&, double&, double&, double&, int);
   void calcLimits();
-  bool calcYAxis(Axis*, int);
-  void calcData(Graph *g);
-  void calcCoordinate(double* &, double* &, int*, int*, Axis*);
-
-  QString xLabel, yLabel;
+  void calcData(Graph*);
+  void calcCoordinate(double* &, double* &, double* &, int*, int*, Axis*);
+  bool insideDiagram(int, int);
+  void clip(int* &);
 };
 
 #endif
