@@ -33,9 +33,6 @@
 
 class QucsApp;
 
-/**
-  *@author Michael Margraf
-  */
 
 class QucsDoc {
 public: 
@@ -49,6 +46,8 @@ public:
   void paintGrid(ViewPainter*, int, int, int, int);
   void print(QPainter*, bool);
 
+  float textCorr();
+  Wire *splitWire(Wire*, Node*);
   int   insertWireNode1(Wire*);
   bool  connectHWires1(Wire*);
   bool  connectVWires1(Wire*);
@@ -70,7 +69,7 @@ public:
   Wire*      selectedWire(int, int);
   Painting*  selectedPainting(int, int);
   void       selectWireLine(Element*, Node*, bool);
-  Element*   selectElement(int, int, bool);
+  Element*   selectElement(int, int, bool, int *index=0);
   int        selectElements(int, int, int, int, bool);
   void       deselectElements(Element*);
   bool  activateComponent(int, int);
