@@ -1,6 +1,6 @@
 /***************************************************************************
-                          qucsfile.cpp  -  description
-                             -------------------
+                                qucsfile.cpp
+                               --------------
     begin                : Sat Mar 27 2004
     copyright            : (C) 2003 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -21,15 +21,20 @@
 
 #include "qucsfile.h"
 
-#include"node.h"
+#include "node.h"
 #include "diagrams/diagrams.h"
 #include "paintings/paintings.h"
+#include "components/spicefile.h"
+#include "components/libcomp.h"
 #include "qucsdoc.h"
+#include "main.h"
 
 #include <qmessagebox.h>
 #include <qdir.h>
 #include <qstringlist.h>
 #include <qregexp.h>
+#include <qprocess.h>
+#include <qtextedit.h>
 
 
 extern QDir QucsWorkDir;
@@ -96,7 +101,7 @@ QString QucsFile::createClipboardFile()
   s += "<Paintings>\n";
   for(pp = Doc->Paints->first(); pp != 0; pp = Doc->Paints->next())
     if(pp->isSelected)
-      if(pp->Name != "PortSym ") {
+      if(pp->Name.at(0) != '.') {  // subcircuit specific -> do not copy
         s += "<"+pp->save()+">\n";  z++; }
   s += "</Paintings>\n";
 
@@ -467,6 +472,8 @@ bool QucsFile::loadDiagrams(QTextStream *stream, QPtrList<Diagram> *List)
     else if(cstr == "<ySmith") d = new SmithDiagram(0,0,false);
     else if(cstr == "<PS") d = new PSDiagram();
     else if(cstr == "<SP") d = new PSDiagram(0,0,false);
+    else if(cstr == "<Rect3D") d = new Rect3DDiagram();
+    else if(cstr == "<Curve") d = new CurveDiagram();
     else {
       QMessageBox::critical(0, QObject::tr("Error"),
 		   QObject::tr("Format Error:\nUnknown diagram!"));
@@ -723,7 +730,8 @@ bool QucsFile::rebuildSymbol(QString *s)
 // ---------------------------------------------------
 // Follows the wire lines in order to determine the node names for
 // each component. Output into "stream", NodeSets are collected in "NS".
-bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
+bool QucsFile::giveNodeNames(QTextStream *stream, int& countInit,
+                             QStringList& Collect, QTextEdit *ErrText)
 {
   Node *p1, *p2;
   Wire *pw;
@@ -738,6 +746,7 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
   for(pw = Wires->first(); pw != 0; pw = Wires->next())
     if(pw->Label != 0) pw->Port1->Name = pw->Label->Name;
 
+  bool r;
   QString s;
   // give the ground nodes the name "gnd", and insert subcircuits
   for(Component *pc = Comps->first(); pc != 0; pc = Comps->next())
@@ -752,12 +761,45 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
 	     QucsDoc *d = new QucsDoc(0, QucsWorkDir.filePath(s));
              if(!d->File.load()) {  // load document if possible
                delete d;
+               ErrText->insert(QObject::tr("ERROR: Cannot load subcircuit \"%1\".").arg(s));
                return false;
              }
 	     d->DocName = s;
-	     d->File.createSubNetlist(stream, countInit);
+	     r = d->File.createSubNetlist(
+				stream, countInit, Collect, ErrText);
 	     delete d;
+	     if(!r) return false;
            }
+      else if(pc->Model == "Lib") {
+	     s  = pc->Props.first()->Value + "_";
+	     s += pc->Props.next()->Value;
+	     if(StringList.findIndex(s) >= 0)
+		continue;   // insert each subcircuit just one time
+
+	     StringList.append(s);
+	     r = ((LibComp*)pc)->outputSubNetlist(stream);
+	     if(!r) {
+               ErrText->insert(
+	         QObject::tr("ERROR: Cannot load library component \"%1\".").arg(pc->Name));
+	       return false;
+	     }
+      }
+      else if(pc->Model == "SPICE") {
+	     s = pc->Props.first()->Value;
+	     if(s.isEmpty()) {
+               ErrText->insert(QObject::tr("ERROR: No file name in SPICE component \"%1\".").
+			       arg(pc->Name));
+	       return false;
+	     }
+	     if(StringList.findIndex(s) >= 0)
+		continue;   // insert each spice component just one time
+
+	     StringList.append(s);
+	     s += '"'+pc->Props.next()->Value;
+	     if(pc->Props.next()->Value == "yes")  s = "SPICE \""+s;
+	     else  s = "SPICEo\""+s;
+	     Collect.append(s);
+	   }
 
 
   bool setName=false;
@@ -767,8 +809,8 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
     if(p1->Name.isEmpty()) continue;
     if(p1->Label)
       if(!p1->Label->initValue.isEmpty())
-	NS += "NodeSet:NS" + QString::number(countInit++) + " " + p1->Name +
-	      " U=\"" + p1->Label->initValue + "\"\n";
+	Collect.append("NodeSet:NS" + QString::number(countInit++) + " " + p1->Name +
+	      " U=\"" + p1->Label->initValue + "\"");
 
     Cons.append(p1);
     for(p2 = Cons.first(); p2 != 0; p2 = Cons.next())
@@ -793,8 +835,8 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
 	    Cons.findRef(p2);   // back to current Connection
 	    if(pw->Label)
 	      if(!pw->Label->initValue.isEmpty())
-		NS += "NodeSet:NS" + QString::number(countInit++) + " " +
-		      p1->Name + " U=\"" + pw->Label->initValue + "\"\n";
+		Collect.append("NodeSet:NS" + QString::number(countInit++) + " " +
+		      p1->Name + " U=\"" + pw->Label->initValue + "\"");
 	    setName = false;
 	  }
         }
@@ -809,8 +851,8 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
     p1->Name = "_net" + QString::number(z++);   // create node name
     if(p1->Label)
       if(!p1->Label->initValue.isEmpty())
-	NS += "NodeSet:NS" + QString::number(countInit++) + " " + p1->Name +
-	      " U=\"" + p1->Label->initValue + "\"\n";
+	Collect.append("NodeSet:NS" + QString::number(countInit++) + " " + p1->Name +
+			" U=\"" + p1->Label->initValue + "\"");
 
     Cons.append(p1);
     // create list with connections to the node
@@ -836,8 +878,8 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
 	    Cons.findRef(p2);   // back to current Connection
 	    if(pw->Label)
 	      if(!pw->Label->initValue.isEmpty())
-		NS += "NodeSet:Nb" + QString::number(countInit++) + " " +
-		      p1->Name + " U=\"" + pw->Label->initValue + "\"\n";
+		Collect.append("NodeSet:Nb" + QString::number(countInit++) + " " +
+		      p1->Name + " U=\"" + pw->Label->initValue + "\"");
 	    setName = false;
 	  }
         }
@@ -849,12 +891,13 @@ bool QucsFile::giveNodeNames(QTextStream *stream, QString& NS, int& countInit)
 
 // ---------------------------------------------------
 // Write the netlist as subcircuit to the text stream 'NetlistFile'.
-bool QucsFile::createSubNetlist(QTextStream *stream, int& countInit)
+bool QucsFile::createSubNetlist(QTextStream *stream, int& countInit,
+                                QStringList& Collect, QTextEdit *Error)
 {
-  QString s;
-  if(!giveNodeNames(stream, s, countInit)) return false;
-
   int i, z;
+  QString s;
+  if(!giveNodeNames(stream, countInit, Collect, Error)) return false;
+
   QStringList sl;
   QStringList::Iterator it;
   Component *pc;
@@ -891,35 +934,29 @@ bool QucsFile::createSubNetlist(QTextStream *stream, int& countInit)
 }
 
 // ---------------------------------------------------
-// Creates the file "netlist.net" in the project directory. Returns "true"
-// if successful.
-bool QucsFile::createNetlist(QFile *NetlistFile)
+// Determines the node names and writes subcircuits into netlist file.
+bool QucsFile::prepareNetlist(QTextStream& stream, QStringList& Collect,
+                              QTextEdit *Error)
 {
-  if(!NetlistFile->open(IO_WriteOnly)) return false;
-
-  QTextStream stream(NetlistFile);
   // first line is documentation
   stream << "# Qucs " << PACKAGE_VERSION << "  " << Doc->DocName << "\n";
 
-
-  QString s;
   int countInit = 0;
-  if(!giveNodeNames(&stream, s, countInit)) {
-    NetlistFile->close();
-    StringList.clear();
+  StringList.clear();
+  if(!giveNodeNames(&stream, countInit, Collect, Error))
     return false;
-  }
-  stream << s;  // outputs the NodeSets (if any)
+  
+  return true;
+}
 
-  // .................................................
-  // write all components with node names into the netlist file
+// .................................................
+// write all components with node names into the netlist file
+void QucsFile::createNetlist(QTextStream& stream)
+{
+  QString s;
   for(Component *pc = Comps->first(); pc != 0; pc = Comps->next()) {
     s = pc->NetList();
     if(!s.isEmpty())  // not inserted: subcircuit ports, disabled components
       stream << s << "\n";
   }
-  NetlistFile->close();
-
-  StringList.clear();
-  return true;
 }
