@@ -1,6 +1,6 @@
 /***************************************************************************
-                          qucsinit.cpp  -  description
-                             -------------------
+                               qucsinit.cpp
+                              --------------
     begin                : Sat May 1 2004
     copyright            : (C) 2004 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -22,12 +22,15 @@
 #include <qaction.h>
 #include <qaccel.h>
 #include <qmenubar.h>
+#include <qlabel.h>
+#include <qtimer.h>
 
 #include "qucs.h"
 #include "qucsview.h"
 #include "qucsinit.h"
 #include "qucsactions.h"
 
+
 QucsInit::QucsInit()
 {
 }
@@ -88,7 +91,7 @@ void QucsInit::initActions()
   connect(App->fileSave, SIGNAL(activated()), App, SLOT(slotFileSave()));
 
   App->fileSaveAs =
-    new QAction(tr("Save File As"), tr("Save &as..."), 0, App);
+    new QAction(tr("Save File As"), tr("Save &as..."), CTRL+Key_Minus, App);
   App->fileSaveAs->setStatusTip(
 	tr("Saves the current document under a new filename"));
   App->fileSaveAs->setWhatsThis(
@@ -98,7 +101,7 @@ void QucsInit::initActions()
   App->fileSaveAll =
     new QAction(tr("Save All Files"),
 		QIconSet(QImage(QucsSettings.BitmapDir + "filesaveall.png")),
-		tr("Save &All"), 0, App);
+		tr("Save &All"), CTRL+Key_Plus, App);
   App->fileSaveAll->setStatusTip(tr("Saves all open documents"));
   App->fileSaveAll->setWhatsThis(
 	tr("Save All Files\n\nSaves all open documents"));
@@ -115,7 +118,7 @@ void QucsInit::initActions()
 
   App->symEdit =
     new QAction(tr("Edit Circuit Symbol"),
-		tr("Edit Circuit Symbol"), CTRL+Key_U, App);
+		tr("Edit Circuit Symbol"), Key_F3, App);
   App->symEdit->setStatusTip(tr("Edits the symbol for this schematic"));
   App->symEdit->setWhatsThis(
 	tr("Edit Circuit Symbol\n\nEdits the symbol for this schematic"));
@@ -123,7 +126,7 @@ void QucsInit::initActions()
 
   App->fileSettings =
     new QAction(tr("Document Settings"),
-		tr("Document Settings..."), 0, App);
+		tr("Document Settings..."), CTRL+Key_Period, App);
   App->fileSettings->setStatusTip(tr("Document Settings"));
   App->fileSettings->setWhatsThis(
 	tr("Settings\n\nSets properties of the file"));
@@ -156,58 +159,59 @@ void QucsInit::initActions()
 
   App->applSettings =
     new QAction(tr("Application Settings"),
-		tr("Application Settings..."), 0, App);
+		tr("Application Settings..."), CTRL+Key_Comma, App);
   App->applSettings->setStatusTip(tr("Application Settings"));
   App->applSettings->setWhatsThis(
 	tr("Qucs Settings\n\nSets properties of the application"));
   connect(App->applSettings, SIGNAL(activated()),
 	  App, SLOT(slotApplSettings()));
 
-  App->alignTop =
+  Acts->alignTop =
     new QAction(tr("Align top"), tr("Align top"), CTRL+Key_T, App);
-  App->alignTop->setStatusTip(tr("Align top selected elements"));
-  App->alignTop->setWhatsThis(
+  Acts->alignTop->setStatusTip(tr("Align top selected elements"));
+  Acts->alignTop->setWhatsThis(
 	tr("Align top\n\nAlign selected elements to their upper edge"));
-  connect(App->alignTop, SIGNAL(activated()), App, SLOT(slotAlignTop()));
+  connect(Acts->alignTop, SIGNAL(activated()), Acts, SLOT(slotAlignTop()));
 
-  App->alignBottom =
-    new QAction(tr("Align bottom"), tr("Align bottom"),	0, App);
-  App->alignBottom->setStatusTip(tr("Align bottom selected elements"));
-  App->alignBottom->setWhatsThis(
+  Acts->alignBottom =
+    new QAction(tr("Align bottom"), tr("Align bottom"), 0, App);
+  Acts->alignBottom->setStatusTip(tr("Align bottom selected elements"));
+  Acts->alignBottom->setWhatsThis(
 	tr("Align bottom\n\nAlign selected elements to their lower edge"));
-  connect(App->alignBottom, SIGNAL(activated()),
-	  App, SLOT(slotAlignBottom()));
+  connect(Acts->alignBottom, SIGNAL(activated()),
+	  Acts, SLOT(slotAlignBottom()));
 
-  App->alignLeft = new QAction(tr("Align left"), tr("Align left"), 0, App);
-  App->alignLeft->setStatusTip(tr("Align left selected elements"));
-  App->alignLeft->setWhatsThis(
+  Acts->alignLeft = new QAction(tr("Align left"), tr("Align left"), 0, App);
+  Acts->alignLeft->setStatusTip(tr("Align left selected elements"));
+  Acts->alignLeft->setWhatsThis(
 	tr("Align left\n\nAlign selected elements to their left edge"));
-  connect(App->alignLeft, SIGNAL(activated()), App, SLOT(slotAlignLeft()));
+  connect(Acts->alignLeft, SIGNAL(activated()), Acts, SLOT(slotAlignLeft()));
 
-  App->alignRight = new QAction(tr("Align right"), tr("Align right"), 0, App);
-  App->alignRight->setStatusTip(tr("Align right selected elements"));
-  App->alignRight->setWhatsThis(
+  Acts->alignRight = new QAction(tr("Align right"), tr("Align right"), 0, App);
+  Acts->alignRight->setStatusTip(tr("Align right selected elements"));
+  Acts->alignRight->setWhatsThis(
 	tr("Align right\n\nAlign selected elements to their right edge"));
-  connect(App->alignRight, SIGNAL(activated()), App, SLOT(slotAlignRight()));
+  connect(Acts->alignRight, SIGNAL(activated()), Acts, SLOT(slotAlignRight()));
 
-  App->distrHor = new QAction(tr("Distribute horizontally"),
+  Acts->distrHor = new QAction(tr("Distribute horizontally"),
 	tr("Distribute horizontally"), 0, App);
-  App->distrHor->setStatusTip(tr("Distribute equally horizontally"));
-  App->distrHor->setWhatsThis(
+  Acts->distrHor->setStatusTip(tr("Distribute equally horizontally"));
+  Acts->distrHor->setWhatsThis(
 	tr("Distribute horizontally\n\n"
 	   "Distribute horizontally selected elements"));
-  connect(App->distrHor, SIGNAL(activated()), App, SLOT(slotDistribHoriz()));
+  connect(Acts->distrHor, SIGNAL(activated()), Acts, SLOT(slotDistribHoriz()));
 
-  App->distrVert =
+  Acts->distrVert =
     new QAction(tr("Distribute vertically"),
 		tr("Distribute vertically"), 0, App);
-  App->distrVert->setStatusTip(tr("Distribute equally vertically"));
-  App->distrVert->setWhatsThis(
+  Acts->distrVert->setStatusTip(tr("Distribute equally vertically"));
+  Acts->distrVert->setWhatsThis(
 	tr("Distribute vertically\n\n"
 	   "Distribute vertically selected elements"));
-  connect(App->distrVert, SIGNAL(activated()), App, SLOT(slotDistribVert()));
+  connect(Acts->distrVert, SIGNAL(activated()), Acts, SLOT(slotDistribVert()));
 
-  Acts->onGrid = new QAction(tr("Set on Grid"), tr("Set on Grid"), 0, App);
+  Acts->onGrid = new QAction(tr("Set on Grid"), tr("Set on Grid"),
+				CTRL+Key_U, App);
   Acts->onGrid->setStatusTip(tr("Set on Grid"));
   Acts->onGrid->setWhatsThis(
 	tr("Set on Grid\n\nSets selected elements on grid"));
@@ -289,7 +293,7 @@ void QucsInit::initActions()
 		tr("&Undo"), CTRL+Key_Z, App);
   App->undo->setStatusTip(tr("Undoes the last command"));
   App->undo->setWhatsThis(tr("Undo\n\nMakes the last action undone"));
-  connect(App->undo, SIGNAL(activated()), App, SLOT(slotEditUndo()));
+  connect(App->undo, SIGNAL(activated()), Acts, SLOT(slotEditUndo()));
 
   App->redo =
     new QAction(tr("Redo"),
@@ -297,15 +301,17 @@ void QucsInit::initActions()
 		tr("&Redo"), CTRL+Key_Y, App);
   App->redo->setStatusTip(tr("Redoes the last command"));
   App->redo->setWhatsThis(tr("Redo\n\nRepeats the last action once more"));
-  connect(App->redo, SIGNAL(activated()), App, SLOT(slotEditRedo()));
+  connect(App->redo, SIGNAL(activated()), Acts, SLOT(slotEditRedo()));
 
-  App->projNew = new QAction(tr("New Project"), tr("&New Project..."), 0, App);
+  App->projNew = new QAction(tr("New Project"), tr("&New Project..."),
+			CTRL+SHIFT+Key_N, App);
   App->projNew->setStatusTip(tr("Creates a new project"));
   App->projNew->setWhatsThis(tr("New Project\n\nCreates a new project"));
   connect(App->projNew, SIGNAL(activated()), App, SLOT(slotProjNewButt()));
 
   App->projOpen =
-    new QAction(tr("Open Project"), tr("&Open Project..."), 0, App);
+    new QAction(tr("Open Project"), tr("&Open Project..."),
+		CTRL+SHIFT+Key_O, App);
   App->projOpen->setStatusTip(tr("Opens a project"));
   App->projOpen->setWhatsThis(
 	tr("Open Project\n\nOpens an existing project"));
@@ -313,14 +319,16 @@ void QucsInit::initActions()
 	  App, SLOT(slotMenuOpenProject()));
 
   App->projDel =
-    new QAction(tr("Delete Project"), tr("&Delete Project..."), 0, App);
+    new QAction(tr("Delete Project"), tr("&Delete Project..."),
+		CTRL+SHIFT+Key_D, App);
   App->projDel->setStatusTip(tr("Deletes a project"));
   App->projDel->setWhatsThis(
 	tr("Delete Project\n\nDeletes an existing project"));
   connect(App->projDel, SIGNAL(activated()), App, SLOT(slotMenuDelProject()));
 
   App->projClose =
-    new QAction(tr("Close Project"), tr("&Close Project"), 0, App);
+    new QAction(tr("Close Project"), tr("&Close Project"),
+		CTRL+SHIFT+Key_W, App);
   App->projClose->setStatusTip(tr("Close current project"));
   App->projClose->setWhatsThis(
 	tr("Close Project\n\nCloses the current project"));
@@ -330,7 +338,7 @@ void QucsInit::initActions()
   App->magAll =
     new QAction(tr("View All"),
                 QIconSet(QImage(QucsSettings.BitmapDir + "viewmagfit.png")),
-                tr("View All"), 0, App);
+                tr("View All"), Key_0, App);
   App->magAll->setStatusTip(tr("Views the whole page"));
   App->magAll->setWhatsThis(tr("View All\n\nShows the whole page content"));
   connect(App->magAll, SIGNAL(activated()), App, SLOT(slotShowAll()));
@@ -370,12 +378,12 @@ void QucsInit::initActions()
   Acts->select->setToggleAction(true);
   connect(Acts->select, SIGNAL(toggled(bool)), Acts, SLOT(slotSelect(bool)));
 
-  App->selectAll =
+  Acts->selectAll =
     new QAction(tr("Select All"), tr("Select All"), CTRL+Key_A, App);
-  App->selectAll->setStatusTip(tr("Selects all elements"));
-  App->selectAll->setWhatsThis(
+  Acts->selectAll->setStatusTip(tr("Selects all elements"));
+  Acts->selectAll->setWhatsThis(
 	tr("Select All\n\nSelects all elements of the document"));
-  connect(App->selectAll, SIGNAL(activated()), App, SLOT(slotSelectAll()));
+  connect(Acts->selectAll, SIGNAL(activated()), Acts, SLOT(slotSelectAll()));
 
   Acts->editRotate =
     new QAction(tr("Rotate"),
@@ -497,6 +505,34 @@ void QucsInit::initActions()
   connect(Acts->insLabel, SIGNAL(toggled(bool)),
 	  Acts, SLOT(slotInsertLabel(bool)));
 
+  Acts->callEditor = new QAction(tr("Text editor"), tr("Text Editor"),
+				CTRL+Key_1, App);
+  Acts->callEditor->setStatusTip(tr("Starts the Qucs text editor"));
+  Acts->callEditor->setWhatsThis(
+			tr("Text editor\n\nStarts the Qucs text editor"));
+  connect(Acts->callEditor, SIGNAL(activated()), Acts, SLOT(slotCallEditor()));
+
+  Acts->callFilter = new QAction(tr("Filter synthesis"), tr("Filter synthesis"),
+				CTRL+Key_2, App);
+  Acts->callFilter->setStatusTip(tr("Starts QucsFilter"));
+  Acts->callFilter->setWhatsThis(
+			tr("Filter synthesis\n\nStarts QucsFilter"));
+  connect(Acts->callFilter, SIGNAL(activated()), Acts, SLOT(slotCallFilter()));
+
+  Acts->callLine = new QAction(tr("Line calculation"), tr("Line calculation"),
+				CTRL+Key_3, App);
+  Acts->callLine->setStatusTip(tr("Starts QucsTrans"));
+  Acts->callLine->setWhatsThis(
+		tr("Line calculation\n\nStarts transmission line calculator"));
+  connect(Acts->callLine, SIGNAL(activated()), Acts, SLOT(slotCallLine()));
+
+  Acts->callLib = new QAction(tr("Component Library"), tr("Component Library"),
+				CTRL+Key_4, App);
+  Acts->callLib->setStatusTip(tr("Starts QucsLib"));
+  Acts->callLib->setWhatsThis(
+		tr("Component Library\n\nStarts component library program"));
+  connect(Acts->callLib, SIGNAL(activated()), Acts, SLOT(slotCallLibrary()));
+
   App->simulate =
     new QAction(tr("Simulate"),
 		QIconSet(QImage(QucsSettings.BitmapDir + "gear.png")),
@@ -527,21 +563,22 @@ void QucsInit::initActions()
   connect(Acts->setMarker, SIGNAL(toggled(bool)),
 	Acts, SLOT(slotSetMarker(bool)));
 
-  App->showMsg =
+  Acts->showMsg =
     new QAction(tr("Show Last Messages"),
 		tr("Show Last Messages"), Key_F5, App);
-  App->showMsg->setStatusTip(tr("Shows last simulation messages"));
-  App->showMsg->setWhatsThis(
+  Acts->showMsg->setStatusTip(tr("Shows last simulation messages"));
+  Acts->showMsg->setWhatsThis(
     tr("Show Last Messages\n\nShows the messages of the last simulation"));
-  connect(App->showMsg, SIGNAL(activated()), App, SLOT(slotShowLastMsg()));
+  connect(Acts->showMsg, SIGNAL(activated()), Acts, SLOT(slotShowLastMsg()));
 
-  App->showNet =
-    new QAction(tr("Show Last Netlist"), tr("Show Last Netlist"), 0, App);
-  App->showNet->setStatusTip(tr("Shows last simulation netlist"));
-  App->showNet->setWhatsThis(
+  Acts->showNet =
+    new QAction(tr("Show Last Netlist"), tr("Show Last Netlist"),
+		Key_F6, App);
+  Acts->showNet->setStatusTip(tr("Shows last simulation netlist"));
+  Acts->showNet->setWhatsThis(
 	tr("Show Last Netlist\n\nShows the netlist of the last simulation"));
-  connect(App->showNet, SIGNAL(activated()),
-	  App, SLOT(slotShowLastNetlist()));
+  connect(Acts->showNet, SIGNAL(activated()),
+	  Acts, SLOT(slotShowLastNetlist()));
 
   viewToolBar =
     new QAction(tr("Toolbar"), tr("Tool&bar"), 0, App, 0, true);
@@ -558,27 +595,27 @@ void QucsInit::initActions()
   connect(viewStatusBar, SIGNAL(toggled(bool)),
 	  this, SLOT(slotViewStatusBar(bool)));
 
-  App->helpIndex =
+  Acts->helpIndex =
     new QAction(tr("Help Index"), tr("Help Index..."), Key_F1, App);
-  App->helpIndex->setStatusTip(tr("Index of Qucs Help"));
-  App->helpIndex->setWhatsThis(tr("Help Index\n\nIndex of intern Qucs help"));
-  connect(App->helpIndex, SIGNAL(activated()), App, SLOT(slotHelpIndex()));
+  Acts->helpIndex->setStatusTip(tr("Index of Qucs Help"));
+  Acts->helpIndex->setWhatsThis(tr("Help Index\n\nIndex of intern Qucs help"));
+  connect(Acts->helpIndex, SIGNAL(activated()), Acts, SLOT(slotHelpIndex()));
 
-  App->helpGetStart =
+  Acts->helpGetStart =
     new QAction(tr("Getting Started"),
 		tr("Getting Started..."), 0, App);
-  App->helpGetStart->setStatusTip(tr("Getting Started with Qucs"));
-  App->helpGetStart->setWhatsThis(
+  Acts->helpGetStart->setStatusTip(tr("Getting Started with Qucs"));
+  Acts->helpGetStart->setWhatsThis(
 	tr("Getting Started\n\nShort introduction into Qucs"));
-  connect(App->helpGetStart, SIGNAL(activated()),
-	  App, SLOT(slotGettingStarted()));
+  connect(Acts->helpGetStart, SIGNAL(activated()),
+	  Acts, SLOT(slotGettingStarted()));
 
   helpAboutApp = new QAction(tr("About"), tr("&About Qucs..."), 0, App);
   helpAboutApp->setStatusTip(tr("About the application"));
   helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
   connect(helpAboutApp, SIGNAL(activated()), SLOT(slotHelpAbout()));
 
-  helpAboutQt = new QAction(tr("About Qt"), tr("&About Qt..."), 0, App);
+  helpAboutQt = new QAction(tr("About Qt"), tr("About Qt..."), 0, App);
   helpAboutQt->setStatusTip(tr("About Qt"));
   helpAboutQt->setWhatsThis(tr("About Qt\n\nAbout Qt by Trolltech"));
   connect(helpAboutQt, SIGNAL(activated()), SLOT(slotHelpAboutQt()));
@@ -606,13 +643,13 @@ void QucsInit::initMenuBar()
   App->fileQuit->addTo(fileMenu);
 
   alignMenu = new QPopupMenu();  // submenu for "editMenu"
-  App->alignTop->addTo(alignMenu);
-  App->alignBottom->addTo(alignMenu);
-  App->alignLeft->addTo(alignMenu);
-  App->alignRight->addTo(alignMenu);
+  Acts->alignTop->addTo(alignMenu);
+  Acts->alignBottom->addTo(alignMenu);
+  Acts->alignLeft->addTo(alignMenu);
+  Acts->alignRight->addTo(alignMenu);
   alignMenu->insertSeparator();
-  App->distrHor->addTo(alignMenu);
-  App->distrVert->addTo(alignMenu);
+  Acts->distrHor->addTo(alignMenu);
+  Acts->distrVert->addTo(alignMenu);
 
   editMenu = new QPopupMenu();  // menuBar entry editMenu
   App->undo->addTo(editMenu);
@@ -624,7 +661,7 @@ void QucsInit::initMenuBar()
   Acts->editDelete->addTo(editMenu);
   editMenu->insertSeparator();
   Acts->select->addTo(editMenu);
-  App->selectAll->addTo(editMenu);
+  Acts->selectAll->addTo(editMenu);
   Acts->editRotate->addTo(editMenu);
   Acts->editMirror->addTo(editMenu);
   Acts->editMirrorY->addTo(editMenu);
@@ -650,11 +687,17 @@ void QucsInit::initMenuBar()
   App->projClose->addTo(projMenu);
   App->projDel->addTo(projMenu);
 
+  toolMenu = new QPopupMenu();  // menuBar entry toolMenu
+  Acts->callEditor->addTo(toolMenu);
+  Acts->callFilter->addTo(toolMenu);
+  Acts->callLine->addTo(toolMenu);
+  Acts->callLib->addTo(toolMenu);
+
   simMenu = new QPopupMenu();  // menuBar entry simMenu
   App->simulate->addTo(simMenu);
   App->dpl_sch->addTo(simMenu);
-  App->showMsg->addTo(simMenu);
-  App->showNet->addTo(simMenu);
+  Acts->showMsg->addTo(simMenu);
+  Acts->showNet->addTo(simMenu);
 
   viewMenu = new QPopupMenu();  // menuBar entry viewMenu
   App->magAll->addTo(viewMenu);
@@ -667,8 +710,8 @@ void QucsInit::initMenuBar()
   viewStatusBar->addTo(viewMenu);
 
   helpMenu = new QPopupMenu();  // menuBar entry helpMenu
-  App->helpIndex->addTo(helpMenu);
-  App->helpGetStart->addTo(helpMenu);
+  Acts->helpIndex->addTo(helpMenu);
+  Acts->helpGetStart->addTo(helpMenu);
   helpMenu->insertSeparator();
   helpAboutApp->addTo(helpMenu);
   helpAboutQt->addTo(helpMenu);
@@ -677,6 +720,7 @@ void QucsInit::initMenuBar()
   App->menuBar()->insertItem(tr("&Edit"), editMenu);
   App->menuBar()->insertItem(tr("&Insert"), insMenu);
   App->menuBar()->insertItem(tr("&Project"), projMenu);
+  App->menuBar()->insertItem(tr("&Tools"), toolMenu);
   App->menuBar()->insertItem(tr("&Simulation"), simMenu);
   App->menuBar()->insertItem(tr("&View"), viewMenu);
   App->menuBar()->insertSeparator();
@@ -731,12 +775,40 @@ void QucsInit::initToolBar()
 }
 
 // #########################################################################
-// Statusbar
 void QucsInit::initStatusBar()
 {
+  // To reserve enough space, insert the longest text and rewrite it afterwards.
+  WarningLabel =
+    new QLabel(tr("Warnings in last simulation! Press F5"), App->statusBar());
+  App->statusBar()->addWidget(WarningLabel, 0, true);
+  WarningLabel->setText(tr("no warnings"));
   App->statusBar()->message(tr("Ready."), 2000);
 }
 
+// #########################################################################
+void QucsInit::slotShowWarnings()
+{
+  static int ResultState = 0;
+
+  if(ResultState == 0) {
+    QFont f = WarningLabel->font();
+    f.setWeight(QFont::DemiBold);
+    WarningLabel->setFont(f);
+    WarningLabel->setText(tr("Warnings in last simulation! Press F5"));
+  }
+
+  ResultState++;
+  if(ResultState & 1)
+    WarningLabel->setPaletteForegroundColor(Qt::red);
+  else
+    WarningLabel->setPaletteForegroundColor(Qt::black);
+
+  if(ResultState < 9)
+    QTimer::singleShot(500, this, SLOT(slotShowWarnings()));
+  else 
+    ResultState = 0;
+}
+
 // ######################################################################
 // turn Toolbar on or off
 void QucsInit::slotViewToolBar(bool toggle)
@@ -776,7 +848,7 @@ void QucsInit::slotHelpAbout()
 {
   QMessageBox::about(App, tr("About..."),
     tr("Qucs Version ")+PACKAGE_VERSION+
-    tr("\nQt universal circuit simulator\n")+
+    tr("\nQuite Universal Circuit Simulator\n")+
     tr("Copyright (C) 2003, 2004, 2005 by Michael Margraf\n")+
     "\nThis is free software; see the source for copying conditions."
     "\nThere is NO warranty; not even for MERCHANTABILITY or "
@@ -793,6 +865,7 @@ void QucsInit::slotHelpAbout()
     tr("Japanese by Toyoyuki Ishikawa\n")+
     tr("Italian by Giorgio Luparia\n")+
     tr("Hebrew by Dotan Nahum\n")+
+    tr("Swedish by Markus Gothe\n")+
     tr("Hungarian by Jozsef Bus"));
 }
 
